/*****************************************************************************
 * Axel
 *****************************************************************************
 * BSplineSurface.h
 * 2007-05-22
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# ifndef QBSPLINESURFACE_H
# define QBSPLINESURFACE_H

# include <QAxel/QParametricSurface.h>

namespace QAxel {

class QPoint3f ;
class QMesh ;
class QBoundingBox ;

/****************************************************************************/
/*!
  \class QBSplineSurface QSurface.h
  \brief The class QBSplineSurface provides ...
  
  More ...
*/
/****************************************************************************/
class QBSplineSurface : public QParametricSurface
{
    Q_OBJECT
    Q_ENUMS(ExtrusionEdge)

public :
     QBSplineSurface(QBoundingBox * boundingbox) ;
    ~QBSplineSurface(void) { /* freeSurf in glue */ }

    enum ExtrusionEdge {UP, DOWN, LEFT, RIGHT, NONE} ;
    
    void draw(void) ;
    void drawWithNames(void) ;
    void getGlData(int * nbi, int * indices, int * nbv, double * vertices) ;
									   
public slots:
    QPoint3f * eval(float u, float v) ;
    float umin(void) { return knots1[0] ; }
    float umax(void) { return knots1[number1+order1-1] ; }
    float vmin(void) { return knots2[0] ; }
    float vmax(void) { return knots2[number2+order2-1] ; }
    
public slots :
    void compute(void) ;

    void push_point(QPoint3f * p) ;
    void push_point(QPoint3f * p, int position) ;

    void computeKnots1(void) ;
    void computeKnots2(void) ;
    QList<QPoint3f *> listSelectedControlPoints(void) ;
    QBSplineSurface::ExtrusionEdge extrusionPossible(void) ;
    void extrudeUpSide(void) ;
    void extrudeDownSide(void) ;
    void extrudeLeftSide(void) ;
    void extrudeRightSide(void) ;
    
public :
    int order1 ;				//!< Order of surface in first parameter direction : ik1
    int order2 ;				//!< Order of surface in second parameter direction : ik2
    int number1 ;				//!< Number of coefficients in first parameter direction : in1
    int number2 ; 				//!< Number of coefficients in second parameter direction : in2
    double * knots1 ;			//!< Pointer to knot vector in first parameter direction : et1
    double * knots2 ;			//!< Pointer to knot vector in second parameter direction : et2
//	double * rcoef ; 		//!< Pointer to array of rational vertices and weights (in1.in2.(idim+1))
    int ikind ;				//!< Type of surface
    //!< 1 : Polynomial BSpline surface
    //!< 2 : Rational BSpline tensor-product surface (nurbs)
    //!< 3 : Polynomial Bezier tensor-product surface
    //!< 4 : Raitonal Bezier tensor-product surface
    int idim ;				//!< Dimension of the space in which the surface lies
    int icopy ; 			//!< Indicates whether the arrays of the surfacce are allocated and copied or referenced
    //!< when the surface was created
    //!< 0 : Pointer set to input arrays. The arrays are not delete by Sisl freeSurf
    //!< 1 : Array are allocated and copied. The arrays are delete by Sisl freeSurf
    //!< 2 : Pointer set to input arrays, but the arrays are to be treated as allocated and copied
    //!< The arrays are deleted by Sisl freeSurf
    
    QList<QPoint3f *> points ;
    
private:
    static int id_ ;
} ;


} ;

# endif
