/*****************************************************************************
 * Axel
 *****************************************************************************
 * QBoundingBox
 * 2006-01-18
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# include <QMessageBox>

# include <QAxel/QBoundingBox.h>
# include <QAxel/QPoint3f.h>

namespace QAxel {

int QBoundingBox::id_ = 0 ;

QBoundingBox::QBoundingBox(void)
{
    QString str ;
    setProperty("type", "BoundingBox") ;
    setProperty("name", "BoundingBox"+str.setNum(id_)) ;
    id_++ ;
    
    xmin_ = 0.0 ; xmax_ = 0.0 ;
    ymin_ = 0.0 ; ymax_ = 0.0 ;
    zmin_ = 0.0 ; zmax_ = 0.0 ;
    
    setObjectColor(new QColor(255,255,255)) ;
//	setObjectColor(new QColor(0,0,0)) ;
}

QBoundingBox::QBoundingBox(float xmin, float xmax, float ymin, float ymax, float zmin, float zmax) 
{
    frame_ = new ManipulatedFrame ;
    xmin_ = xmin ; xmax_ = xmax ;
    ymin_ = ymin ; ymax_ = ymax ;
    zmin_ = zmin ; zmax_ = zmax ;
    
    if(!isValid())
	QMessageBox::warning(0, "Axel", "BoundingBox coordinates are FALSE\n Problems WILL occur") ;
    
    QString str ;
    setProperty("type", "BoundingBox") ;
    setProperty("name", "BoundingBox"+str.setNum(id_)) ;
    id_++ ;
    
    setObjectColor(new QColor(255,255,255)) ;
//	setObjectColor(new QColor(0,0,0)) ;
}

void QBoundingBox::draw(void) 
{
    glDisable(GL_LIGHTING) ;
    glPushMatrix() ;
    glMultMatrixd(frame_->matrix()) ;
    glColor() ;
    glBegin(GL_LINE_LOOP) ;
    // bottom face
    glVertex3f(xmin_, ymin_, zmin_) ;
    glVertex3f(xmax_, ymin_, zmin_) ;
    glVertex3f(xmax_, ymin_, zmax_) ;
    glVertex3f(xmin_, ymin_, zmax_) ;
    glEnd() ;
    
    glBegin(GL_LINE_LOOP) ;
    // left face
    glVertex3f(xmin_, ymin_, zmin_) ;
    glVertex3f(xmin_, ymax_, zmin_) ;
    glVertex3f(xmin_, ymax_, zmax_) ;
    glVertex3f(xmin_, ymin_, zmax_) ;
    glEnd() ;
    
    glBegin(GL_LINE_LOOP) ;
    // top face
    glVertex3f(xmax_, ymax_, zmax_) ;
    glVertex3f(xmin_, ymax_, zmax_) ;
    glVertex3f(xmin_, ymax_, zmin_) ;
    glVertex3f(xmax_, ymax_, zmin_) ;
    glEnd() ;
    
    glBegin(GL_LINE_LOOP) ;
    // right face
    glVertex3f(xmax_, ymax_, zmax_) ;
    glVertex3f(xmax_, ymin_, zmax_) ;
    glVertex3f(xmax_, ymin_, zmin_) ;
    glVertex3f(xmax_, ymax_, zmin_) ;
    glEnd() ;
    
    glBegin(GL_LINE_LOOP) ;
    // front face
    glVertex3f(xmin_, ymin_, zmin_) ;
    glVertex3f(xmin_, ymax_, zmin_) ;
    glVertex3f(xmax_, ymax_, zmin_) ;
    glVertex3f(xmax_, ymin_, zmin_) ;
    glEnd() ;
    
    glBegin(GL_LINE_LOOP) ;
    // back face
    glVertex3f(xmin_, ymin_, zmax_) ;
    glVertex3f(xmin_, ymax_, zmax_) ;
    glVertex3f(xmax_, ymax_, zmax_) ;
    glVertex3f(xmax_, ymin_, zmax_) ;
    glEnd() ;
    glPopMatrix() ;	
    glEnable(GL_LIGHTING) ;
}

void QBoundingBox::drawWithNames(void) 
{
    // Since we do not want a BB to be an independant
    // object, we do not allow selection, therefor, manipulation
}

QString QBoundingBox::toString(void) 
{
    QString xmin ; xmin.setNum(xmin_) ;
    QString xmax ; xmax.setNum(xmax_) ;
    QString ymin ; ymin.setNum(ymin_) ;
    QString ymax ; ymax.setNum(ymax_) ;
    QString zmin ; zmin.setNum(zmin_) ;
    QString zmax ; zmax.setNum(zmax_) ;
    QString r = "[" 
	+ xmin + ", " + xmax + ", "
	+ ymin + ", " + ymax + ", "
	+ xmin + ", " + zmax + "]" ;
    return r ;
}
    
QString QBoundingBox::toDomain2D(void)
{
    QString xmin ; xmin.setNum(xmin_) ;
    QString xmax ; xmax.setNum(xmax_) ;
    QString ymin ; ymin.setNum(ymin_) ;
    QString ymax ; ymax.setNum(ymax_) ;
    QString r = xmin + " " + xmax + " " + ymin + " " + ymax ;
    return r ;	
}

QString QBoundingBox::toDomain3D(void)
{
    QString xmin ; xmin.setNum(xmin_) ;
    QString xmax ; xmax.setNum(xmax_) ;
    QString ymin ; ymin.setNum(ymin_) ;
    QString ymax ; ymax.setNum(ymax_) ;
    QString zmin ; zmin.setNum(zmin_) ;
    QString zmax ; zmax.setNum(zmax_) ;
    QString r = xmin + " " + xmax + " "
	+ ymin + " " + ymax + " "
	+ xmin + " " + zmax ;
    return r ;	
}

QPoint3f * QBoundingBox::center(void)
{
    float x = (xmin_+xmax_)/2 ;
    float y = (ymin_+ymax_)/2 ;
    float z = (zmin_+zmax_)/2 ;
    return new QPoint3f(x,y,z) ;
}

float QBoundingBox::size(void)
{
    return abs(xmax_-xmin_) ;
}

bool QBoundingBox::isNull()
{
    return xmax_ == xmin_ && ymax_ == ymin_ ;
}
    
bool QBoundingBox::isEmpty()
{ 
    return xmin_ > xmax_ || ymin_ > ymax_ ;
}

bool QBoundingBox::isValid()
{
    return xmin_ <= xmax_ && ymin_ <= ymax_ ;
}

bool QBoundingBox::contains(float x, float y, bool strict)
{
    return contains(new QPoint3f(x, y, 0.0), strict) ;
}

bool QBoundingBox::contains(float x, float y, float z, bool strict)
{
    return contains(new QPoint3f(x, y, z), strict) ;
}

bool QBoundingBox::contains(QPoint3f * p, bool strict)
{
    QBoundingBox * b = normalized();
    if (strict)
        return p->x() > b->xmin() && p->x() < b->xmax() &&
               p->y() > b->ymin() && p->y() < b->ymax() &&
               p->z() > b->zmin() && p->z() < b->zmax() ;
    else
        return p->x() >= b->xmin() && p->x() <= b->xmax() &&
               p->y() >= b->ymin() && p->y() <= b->ymax() &&
               p->z() >= b->zmin() && p->z() <= b->zmax() ;
}

bool QBoundingBox::contains(QBoundingBox * b, bool strict)
{
    if (isNull() || b->isNull())
        return false;
    QBoundingBox * b1 = normalized();
    QBoundingBox * b2 = b->normalized();
    if (strict)
        return b2->xmin() > b1->xmin() && b2->xmax() < b1->xmin()
	    && b2->ymin() > b1->ymin() && b2->ymax() < b1->ymin()
	    && b2->zmin() > b1->zmin() && b2->zmax() < b1->zmin() ;
    else
        return b2->xmin() >= b1->xmin() && b2->xmax() <= b1->xmin()
	    && b2->ymin() >= b1->ymin() && b2->ymax() <= b1->ymin()
	    && b2->zmin() >= b1->zmin() && b2->zmax() <= b1->zmin() ;
}

    
/****************************************************************************
 * Intersection test
 * CAUTION: in STRICT case, to be able to deal with 2D Bounding Boxes we keep
 * leq test for z coordinates.
 *
 * That is a problem for REAL 3D Bounding Boxes.
 * -> BoundingBox2D ?
 * -> BoundingBox3D ?
 ****************************************************************************/

bool QBoundingBox::intersects(QBoundingBox * b, bool strict)
{
    if (isNull() || b->isNull())
        return false;
    QBoundingBox * b1 = normalized();
    QBoundingBox * b2 = b->normalized();
    if(strict)
	return ((qMax(b1->xmin(), b2->xmin()) <  qMin(b1->xmax(), b2->xmax()) &&
		 qMax(b1->ymin(), b2->ymin()) <= qMin(b1->ymax(), b2->ymax()) &&
		 qMax(b1->zmin(), b2->zmin()) <= qMin(b1->zmax(), b2->zmax()))
	     || (qMax(b1->xmin(), b2->xmin()) <= qMin(b1->xmax(), b2->xmax()) &&
		 qMax(b1->ymin(), b2->ymin()) <  qMin(b1->ymax(), b2->ymax()) &&
		 qMax(b1->zmin(), b2->zmin()) <= qMin(b1->zmax(), b2->zmax()))) ;
    else
	return ( qMax(b1->xmin(), b2->xmin()) <= qMin(b1->xmax(), b2->xmax()) &&
		 qMax(b1->ymin(), b2->ymin()) <= qMin(b1->ymax(), b2->ymax()) &&
		 qMax(b1->zmin(), b2->zmin()) <= qMin(b1->zmax(), b2->zmax())) ;
}

bool QBoundingBox::unites(QBoundingBox * b, bool strict)
{
    if (isNull() || b->isNull())
	return false ;
    QBoundingBox * b1 = normalized();
    QBoundingBox * b2 = b->normalized();
    if(strict)
	return ( qMin(b1->xmin(), b2->xmin()) <  qMax(b1->xmax(), b2->xmax()) &&
		 qMin(b1->ymin(), b2->ymin()) <  qMax(b1->ymax(), b2->ymax()) &&
		 qMin(b1->zmin(), b2->zmin()) <= qMax(b1->zmax(), b2->zmax())) ;
    else
	return ( qMin(b1->xmin(), b2->xmin()) <= qMax(b1->xmax(), b2->xmax()) &&
		 qMin(b1->ymin(), b2->ymin()) <= qMax(b1->ymax(), b2->ymax()) &&
		 qMin(b1->zmin(), b2->zmin()) <= qMax(b1->zmax(), b2->zmax())) ;
}

QBoundingBox * QBoundingBox::normalized(void)
{
    if(isNull() || width() == 0 || height() == 0) return this ;
    QBoundingBox * b = new QBoundingBox() ;
    if (xmax_ < xmin_) {
        b->setXMin(xmax_) ;
        b->setXMax(xmin_) ;
    } else {
        b->setXMin(xmin_) ;
        b->setXMax(xmax_) ;
    }
    if (ymax_ < ymin_) {
        b->setYMin(ymax_) ;
        b->setYMax(ymin_) ;
    } else {
        b->setYMin(ymin_) ;
        b->setYMax(ymax_) ;
    }
    if (zmax_ < zmin_) {
        b->setZMin(zmax_) ;
        b->setZMax(zmin_) ;
    } else {
        b->setZMin(zmin_) ;
        b->setZMax(zmax_) ;
    }
    return b ;
}

QBoundingBox * QBoundingBox::intersect(QBoundingBox * b)
{
    if (isNull() || b->isNull())
        return new QBoundingBox ;
    QBoundingBox * b1 = normalized();
    QBoundingBox * b2 = b->normalized();
    QBoundingBox * tmp = new QBoundingBox ;
    tmp->setXMin(qMax(b1->xmin(), b2->xmin())) ;
    tmp->setXMax(qMin(b1->xmax(), b2->xmax())) ;
    tmp->setYMin(qMax(b1->ymin(), b2->ymin())) ;
    tmp->setYMax(qMin(b1->ymax(), b2->ymax())) ;
    return tmp->isEmpty() ? new QBoundingBox : tmp ;
}

QBoundingBox * QBoundingBox::intersected(QBoundingBox * b)
{
    return intersect(b) ;
}

QBoundingBox * QBoundingBox::unite(QBoundingBox * b)
{
    if (isNull())
        return b ;
    if (b->isNull())
        return this ;
    QBoundingBox * b1 = normalized();
    QBoundingBox * b2 = b->normalized();
    QBoundingBox * tmp = new QBoundingBox;
    tmp->setXMin(qMin(b1->xmin(), b2->xmin()));
    tmp->setXMax(qMax(b1->xmax(), b2->xmax()));
    tmp->setYMin(qMin(b1->ymin(), b2->ymin()));
    tmp->setYMax(qMax(b1->ymax(), b2->ymax()));
    return tmp;
}

QBoundingBox * QBoundingBox::united(QBoundingBox * b)
{
    return unite(b) ;
}

QList<QPoint3f *> QBoundingBox::corners(void)
{
    QList<QPoint3f *> points ;
    points << new QPoint3f(xmin_, ymin_, 0.0) ;
    points << new QPoint3f(xmax_, ymin_, 0.0) ;
    points << new QPoint3f(xmin_, ymax_, 0.0) ;
    points << new QPoint3f(xmax_, ymax_, 0.0) ;
    return points ;
}

std::ostream& operator << (std::ostream& stream, QBoundingBox & b)
{
    return stream << b.toString().toLatin1().data() ;
}

std::ostream& operator << (std::ostream& stream, QBoundingBox * b)
{
    return stream << b->toString().toLatin1().data() ;
}

float QBoundingBox::abs(float a)
{
    return (a<0?-a:a) ;
}

QBoundingBox * QBoundingBox::intersect(QBoundingBox * b1, QBoundingBox * b2)
{
    return b1->intersect(b2) ;
}

QBoundingBox * QBoundingBox::unite(QBoundingBox * b1, QBoundingBox * b2)
{
    return b1->unite(b2) ;
}

bool QBoundingBox::intersects(QBoundingBox * b1, QBoundingBox * b2)
{
    return b1->intersects(b2) ;
}

bool QBoundingBox::unites(QBoundingBox * b1, QBoundingBox * b2)
{
    return b1->unites(b2) ;
}

} ; // namespace QAxel
