/***************************************************************************** 
 *
 ***************************************************************************/

#ifndef QBOUNDINGSPHERE_H
#define QBOUNDINGSPHERE_H

/***************************************************************************** 
 * Includes
 ****************************************************************************/
#include <math.h>
#include <QAxel/QAxelObject.h>
#include <QGLViewer/qglviewer.h>

using namespace qglviewer ;

/*****************************************************************************
 * Are we using namespaces?
 ****************************************************************************/
namespace QAxel {

/****************************************************************************/
/*!
  \class QBoundingSphere QBoundingSphere.h
  \brief Class defining a sphere in 3D space, used for culling.

  This class is used for representing a bounding volume, which are are
  used for culling. A bounding sphere is a minimal sphere that totally
  encapsulates a rendered \link Shape shape \endlink. This sphere is
  then used for \e culling, i.e. determining if the shape is within the
  viewing frustum. If it is not it will not be rendered either. This can
  save a lot of time when much of the shapes are not visible.
  */
/****************************************************************************/
class QBoundingSphere : public QAxelObject
{
public:

  // constructors
	                QBoundingSphere      ();
	                QBoundingSphere      (const Vec & Center, const float Radius);

  // add a new bounding sphere to this
  void              add                 (const QBoundingSphere & BS);
  void              add                 (const Vec & V);
  void              add                 (const float X, const float Y, const float Z);
  
  // get the radius of the sphere
  float             radius              () const;

  //! get the center of the bounding sphere
  const Vec &    center              () const { return m_Center; };
  
  
protected:

  // the center of the sphere
  Vec            m_Center;
  
  //! the radius of the sphere
  float             m_Radius;
};

} ; // namespace QAxel

#endif // QBOUNDINGSPHERE_H


