/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QDisplayList
 * 2007-04-11
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :         
 *
 ****************************************************************************/

# ifndef QDISPLAYLIST_H
# define QDISPLAYLIST_H

# include <QAxel/QAxelObject.h>

namespace QAxel {

/****************************************************************************/
/*!
  \class QDisplayList QDisplayList.h
  \brief The class QDisplayList is an OpenGL display list.

  This class encapsulates the concept of an OpenGL display-list. It is
  used to optimize rendering of some specific part of the scene (even
  most of it).

  To use the \c QDisplayList first create one and add it to the proer
  place in the scene-graph. After this data must be filled into the
  list, and this is done between a \link begin() \endlink and \link
  end() \endlink -pair.

  Methods also exist for checking if the list is valid, i.e. if
  something has been recorded, and if the list is currently being
  recorded. If the list is rendered and is not valid or currenlt being
  recorded then nothing is done, otherwise the list is rendered
  normally.
*/
/****************************************************************************/
class QDisplayList : public QAxelObject
{
    Q_OBJECT

public:
     QDisplayList(QAxelObject * parent = NULL) ;
    ~QDisplayList(void) ;

    virtual void draw(void) ;
    virtual void drawWithNames(void) ;
    
    void begin() ;
    void end() ;
  
    bool isValid() const ;
    bool isRecording() const ;

    void invalidate() ;

private:
    GLuint m_ListId ;
    bool   m_Recording ;
    bool   m_Valid ;
} ;

} ; // namespace QAxel

# endif  // QDISPLAYLIST_H
