/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QEdge
 * 2006-03-25
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :         
 *
 ****************************************************************************/

# include <QAxel/QVertex.h>
# include <QAxel/QEdge.h>
# include <QAxel/QMesh.h>

using namespace qglviewer ;

namespace QAxel {

int QEdge::id_ = 0 ;

QEdge::QEdge(QVertex * source, QVertex * destination, QAxelObject * parent) : QAxelObject(parent)
{
    QString str ;
    setProperty("type", "Edge") ;
    setProperty("name", "Edge"+str.setNum(id_)) ;
    
    id_++ ;
    
    this->e_source = source ;
    this->e_destination = destination ;
    
    setObjectColor(new QColor(90,90,90)) ;        
}

void QEdge::setSource(QVertex * vertex) 
{
    this->e_source = vertex ;
}

void QEdge::setDestination(QVertex * vertex) 
{
    this->e_destination = vertex ;
}

QVertex * QEdge::source(void)
{
    return e_source ;
}

QVertex * QEdge::destination(void)
{
    return e_destination ;
}

QVertex * QEdge::middle(void)
{
    float x = (e_source->x() + e_destination->x()) / 2 ;
    float y = (e_source->y() + e_destination->y()) / 2 ;
    float z = (e_source->z() + e_destination->z()) / 2 ;
    return new QVertex(x, y, z) ;
}

void QEdge::draw(void) 
{
    glPushMatrix() ;
    glMultMatrixd(frame_->matrix()) ;
    if(m->mode() == QMesh::Edit) glColor() ;
    glBegin(GL_LINES) ;
    glVertex3f(m->vertices_[index1]->x(), m->vertices_[index1]->y(), m->vertices_[index1]->z()) ;
    glVertex3f(m->vertices_[index2]->x(), m->vertices_[index2]->y(), m->vertices_[index2]->z()) ;
    glEnd() ;
    glPopMatrix() ;
}

void QEdge::drawWithNames(void) 
{
    glPushName((GLuint)this);
    draw();
    glPopName();
}

// -------------- TO BE REMOVED ----------------    
/*!
 * This class allows to handle frame manipulation forwarding over incident vertices
 */
class EdgeFrameSetConstraint : public Constraint 
{
public:
    void clearSet();
    void addObjectToSet(QAxel::QAxelObject* o);
    
    QMesh * m ;
    QEdge * e ;
    
    virtual void constrainTranslation(Vec &translation, Frame *const) {
	m->vertices_[e->index1]->frame()->translate(translation);
	m->vertices_[e->index2]->frame()->translate(translation);
    }

    virtual void constrainRotation(Quaternion &rotation, Frame *const frame) {
	// A little bit of math. Easy to understand, hard to guess (tm).
	// rotation is expressed in the frame local coordinates system. Convert it back to world coordinates.
	const Vec worldAxis = frame->inverseTransformOf(rotation.axis());
	const Vec pos = frame->position();
	const float angle = rotation.angle();
	
	// First Vertex manipoulation
	{
	    Quaternion qObject(m->vertices_[e->index1]->frame()->transformOf(worldAxis), angle);
	    m->vertices_[e->index1]->frame()->rotate(qObject);
	    // Comment these lines only rotate the objects
	    Quaternion qWorld(worldAxis, angle);
	    // Rotation around frame world position (pos)
	    m->vertices_[e->index1]->frame()->setPosition(pos + qWorld.rotate(m->vertices_[e->index1]->frame()->position() - pos));
	}
	
	{
	    // Second Vertex manipoulation
	    Quaternion qObject(m->vertices_[e->index2]->frame()->transformOf(worldAxis), angle);
	    m->vertices_[e->index2]->frame()->rotate(qObject);
	    // Comment these lines only rotate the objects
	    Quaternion qWorld(worldAxis, angle);
	    // Rotation around frame world position (pos)
	    m->vertices_[e->index2]->frame()->setPosition(pos + qWorld.rotate(m->vertices_[e->index2]->frame()->position() - pos));
	}
    }
} ;
	
QEdge::QEdge(int index1, int index2, QMesh * m) 
{
    QString str ;
    setProperty("type", "Edge") ;
    setProperty("name", "Edge"+str.setNum(id_)) ;
    
    id_++ ;
    
    this->m = m ;
    this->index1 = index1 ;
    this->index2 = index2 ;
    
    setObjectColor(new QColor(90,90,90)) ;
    
    oldPos = this->frame()->position() ;
    
    EdgeFrameSetConstraint * c = new EdgeFrameSetConstraint ;
    c->m = m ; 
    c->e = this ;
    frame_->setConstraint(c) ;  
    
    connect(frame_, SIGNAL(modified()), this, SLOT(updateVertices())) ;
}

void QEdge::updateVertices(void) 
{                                                  
    QVertex * v1 = m->vertices_[index1] ;
    QVertex * v2 = m->vertices_[index2] ;
	
    Vec pos = this->frame()->position() ;    
    
    Vec trans = oldPos - pos ;
	
    v1->frame()->translate(trans) ;
    v2->frame()->translate(trans) ;
    
    oldPos = pos ;  
}
// --------------------------------------------
} ; // namespace QAxel
