/****************************************************************************
 * A x e l
 *****************************************************************************
 * QImplicitCurve2D.cpp
 * 2006-03-31
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# include <QMessageBox>

# include <QAxel/QMesh.h>
# include <QAxel/QImplicitCurve2D.h>
# include <QAxel/PluginManager.h>

# include <QSynaps/QSynapsInterface.h>

namespace QAxel {

class QBoundingBox ;

int QImplicitCurve2D::id_ = 0 ;

QImplicitCurve2D::QImplicitCurve2D(QString equation, 
				   QBoundingBox * boundingbox, 
				   QImplicitCurve::Method method, 
				   QImplicitCurve::Precision precision) : QImplicitCurve(boundingbox)
{
    QString str ;
    setProperty("type", "ImplicitCurve2D") ;
    setProperty("name", "icurve_"+str.setNum(id_++)) ;
    
    this->c_equation  = equation ;
    this->c_method    = method ;
    this->c_precision = precision ;
}

QImplicitCurve2D::QImplicitCurve2D(QString equation, 
				   double xmin, double xmax, 
				   double ymin, double ymax, 
				   QImplicitCurve::Method method, 
				   QImplicitCurve::Precision precision) : QImplicitCurve(xmin, xmax, ymin, ymax)
{
    QString str ;
    setProperty("type", "ImplicitCurve2D") ;
    setProperty("name", "icurve_"+str.setNum(id_)) ;
    
    this->c_equation  = equation ;
    this->c_method    = method ;
    this->c_precision = precision ;
}

void QImplicitCurve2D::draw(void)
{
    if(hidden()) return ;
    
    // We don't wanna be able to move an implicit curve so we do not push its frame's matrix

    glColor() ;

    if(bbdrawn_) boundingbox_->draw() ;

    if(c_dirty || !c_dlist->isValid()) {
	QSynapsInterface * iSynaps = dynamic_cast<QSynapsInterface *>(PluginManager::instance()->plugin("Synaps")) ;	
	if(iSynaps) {
	    c_dlist->begin() ;
	    if(c_precision == Absolute) iSynaps->Mesh(this)->drawWithoutColor() ;
	    else		        iSynaps->Mesh(this)->drawWithoutColor() ;
	    c_dlist->end() ;
	    c_dirty = false ;
	} else {
	    QMessageBox::information(0, "Axel","QImplicitCurve2D Computation : QSynaps plugin is not loaded","&Ok",0) ;
	}
    }
    c_dlist->draw() ;
}

void QImplicitCurve2D::drawWithNames(void)
{
    glPushName((GLuint)this) ;
    draw();
    glPopName() ;
}

} ; // namespace QAxel
