/****************************************************************************
 * A x e l
 *****************************************************************************
 * QImplicitCurve3D.cpp
 * 2007-05-22
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# include <QMessageBox>

# include <QAxel/QMesh.h>
# include <QAxel/QImplicitCurve3D.h>
# include <QAxel/PluginManager.h>

# include <QSynaps/QSynapsInterface.h>

namespace QAxel {

class QBoundingBox;

int QImplicitCurve3D::id_ = 0 ;

QImplicitCurve3D::QImplicitCurve3D(QString equation1, QString equation2, 
				   QBoundingBox * boundingbox, 
				   QImplicitCurve::Method method, 
				   QImplicitCurve::Precision precision) : QImplicitCurve(boundingbox)
{
    QString str ;
    setProperty("type", "ImplicitCurve3D") ;
    setProperty("name", "icurve_"+str.setNum(id_++)) ;
    
    this->c_equation1 = equation1 ;
    this->c_equation2 = equation2 ;
    this->c_method    = method ;
    this->c_precision = precision ;
}

QImplicitCurve3D::QImplicitCurve3D(QString equation1, QString equation2, 
				   double xmin, double xmax, 
				   double ymin, double ymax, 
				   double zmin, double zmax, 
				   QImplicitCurve::Method method, 
				   QImplicitCurve::Precision precision) : QImplicitCurve(xmin, xmax, 
											 ymin, ymax, 
											 zmin, zmax)
{
    QString str ;
    setProperty("type", "ImplicitCurve3D") ;
    setProperty("name", "icurve_"+str.setNum(id_)) ;
    
    this->c_equation1 = equation1 ;
    this->c_equation2 = equation2 ;
    this->c_method    = method ;
    this->c_precision = precision ;
}

void QImplicitCurve3D::draw(void)
{
    if(hidden()) return ;

    // We don't wanna be able to move an implicit curve so we do not push its frame's matrix

    glColor() ;

    if(c_dirty || !c_dlist->isValid()) {
	QSynapsInterface * iSynaps = dynamic_cast<QSynapsInterface *>(PluginManager::instance()->plugin("Synaps")) ;	
	if(iSynaps) {
	    c_dlist->begin() ;
	    if(c_precision == Absolute) iSynaps->Mesh(this)->drawWithoutColor() ;
	    else                        iSynaps->Mesh(this)->drawWithoutColor() ;
	    c_dlist->end() ;
	    c_dirty = false ;
	} else {
	    QMessageBox::information(0, "Axel","QImplicitCurve3D Computation : QSynaps plugin is not loaded","&Ok",0) ;
	}
    }
    c_dlist->draw() ;
}

void QImplicitCurve3D::drawWithNames(void)
{
    glPushName((GLuint)this) ;
    draw();
    glPopName() ;
}

} ; // namespace QAxel
