/****************************************************************************
 * A x e l
 *****************************************************************************
 * QImplicitSurface.cpp
 * 2007-05-22
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# include <QMessageBox>

# include <QAxel/QSingularity.h>
# include <QAxel/QImplicitSurface.h>
# include <QAxel/QMesh.h>
# include <QAxel/PluginManager.h>

# include <QSynaps/QSynapsInterface.h>

using namespace std ;

namespace QAxel {
		
int QImplicitSurface::id_ = 0 ;

QImplicitSurface::QImplicitSurface(QString equation, QBoundingBox * boundingbox) : QSurface(boundingbox) 
{
    QString str ;
    setProperty("type", "ImplicitSurface") ;
    setProperty("name", "ImplicitSurface"+str.setNum(id_)) ;
    id_++ ;
    
    this->draw_singularities = true ;
    this->equation_ = equation ;
}
    
QImplicitSurface::QImplicitSurface(QString equation,
				   double xmin, double xmax,
				   double ymin, double ymax,
				   double zmin, double zmax) : QSurface(xmin, xmax, 
									ymin, ymax, 
									zmin, zmax)
{
    QString str ;
    setProperty("type", "ImplicitSurface") ;
    setProperty("name", "ImplicitSurface"+str.setNum(id_)) ;
    id_++ ;
    
    this->draw_singularities = true ;
    this->equation_ = equation ;
}

QString QImplicitSurface::equation(void) 
{
    return equation_ ;
}

void QImplicitSurface::draw(void) 
{	
    if(hidden()) return ;

    // We don't wanna be able to move an implicit curve so we do not push its frame's matrix

    if(bbdrawn_) boundingbox_->draw() ;
    
    if(draw_singularities)
	for(int i = 0 ; i < singularities.size() ; i++)
	    singularities[i]->draw() ;

    glColor() ;

    if(s_dirty || !s_dlist->isValid()) {
	QSynapsInterface * iSynaps = dynamic_cast<QSynapsInterface *>(PluginManager::instance()->plugin("Synaps")) ;	
	if(iSynaps) {
	    s_dlist->begin() ;
	    glPushMatrix() ;
	    glMultMatrixd(frame_->matrix()) ;
	    iSynaps->Mesh(this)->drawWithoutColor() ;
	    glPopMatrix() ;
       	    s_dlist->end() ;
	    s_dirty = false ;
	} else {
	    QMessageBox::information(0, "Axel","QImplicitSurface Computation : QSynaps plugin is not loaded","&Ok",0) ;
	}
    }
    s_dlist->draw() ;
}

void QImplicitSurface::drawWithNames(void)
{
    glPushName((GLuint)this) ;
    draw();
    glPopName() ;
}

} ;
