/*****************************************************************************
 * Axel
 *****************************************************************************
 * QPoint3f
 * 2006-03-24
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 * Point's coordinates are the coordinates of its frame ! This way they are
 * always update with no problem ....	
 ****************************************************************************/

# include <QAxel/QPoint3f.h>

namespace QAxel {

int QPoint3f::id_ = 0 ;

QPoint3f::QPoint3f(float x, float y, float z)
{
    frame_->setPosition(x,y,z);
    QString str ;
    setProperty("type", "Point3f") ;
    setProperty("name", "point_"+str.setNum(id_)) ;
    id_++ ;
    setObjectColor(new QColor(0,255,0)) ;
    p_size = 0.03 ;
}
    
QPoint3f::~QPoint3f(void) 
{
    
}

float QPoint3f::x(void) { return frame_->position().x ; }
float QPoint3f::y(void) { return frame_->position().y ; }
float QPoint3f::z(void) { return frame_->position().z ; }
    
void QPoint3f::setx(float x)
{
    Vec v = frame_->position();
    frame_->setPosition(x,v.y,v.z);
}
    
void QPoint3f::sety(float y)
{
    Vec v = frame_->position();
    frame_->setPosition(v.x,y,v.z);
}

void QPoint3f::setz(float z)
{
    Vec v = frame_->position();
    frame_->setPosition(v.x,v.y,z);
}

void QPoint3f::setPosition(float x, float y, float z)
{
    frame_->setPosition(x,y,z) ;
}

void QPoint3f::setPosition(QPoint3f * p)
{
    frame_->setPosition(p->x(),p->y(),p->z()) ;
}

void QPoint3f::translate(void)
{
    frame_->translate(Vec(1,1,0)) ;
}    

float QPoint3f::size(void) { return p_size ; }
void QPoint3f::setSize(float size) { p_size = size ; }

void QPoint3f::draw(void)
{
    if(hidden()) return ;
    static GLUquadric* quad = gluNewQuadric() ;
    glPushMatrix() ;
    glMultMatrixd(frame_->matrix()) ;
    glColor() ;
    gluSphere(quad, p_size, 30, 15) ;
    glPopMatrix() ;
}
    
void QPoint3f::drawWithNames(void)
{
    glPushName((GLuint)this) ;
    draw();
    glPopName() ;
}
    
QString QPoint3f::toString(void)
{
    QString nx, ny, nz ;
    QString s = name() + "(" + nx.setNum(x()) + " " + ny.setNum(y()) + " " + nz.setNum(z()) + ")" ;
    return s ;
}

bool QPoint3f::equals(QPoint3f * p) 
{
    bool r = true ;
    if(p->x() != x()) r = false ;
    if(p->y() != y()) r = false ;
    if(p->z() != z()) r = false ;
    return r ;
}
    
bool QPoint3f::equals(QPoint3f * p, double tolerance)
{
    bool r = true ;
    if(fabs(p->x()-x()) > tolerance) r = false ;
    if(fabs(p->y()-y()) > tolerance) r = false ;
    if(fabs(p->z()-z()) > tolerance) r = false ;
    return r ;
}
    
std::ostream& operator << (std::ostream& stream, QPoint3f & p)
{
    return stream << p.toString().toLatin1().data() ;
}
    
std::ostream& operator << (std::ostream& stream, QPoint3f * p)
{
    return stream << p->toString().toLatin1().data() ;
}

} ; // namespace QAxel
