/*****************************************************************************
 * Axel
 *****************************************************************************
 * QPoint4f
 * 2007-04-18
 * Stephane Chau
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# ifndef QPOINT4F_H
# define QPOINT4F_H

# include <QList>

# include <QAxel/QAxelObject.h>

namespace QAxel {

/****************************************************************************/
/*!
  \class QPoint4f QPoint4f.h
  \brief The class QPoint4f provides ...

  More ...
*/
/****************************************************************************/

class QPoint4f : public QAxelObject 
{
    Q_OBJECT
public:
	QPoint4f(float s, float t, float u, float v) ;
	QPoint4f(int i, float var0, float var1, float var2, float var3) ; // constructor with shuffled variables
	~QPoint4f(void) ;

	float s(void){ return p_s ; }
    float t(void){ return p_t ; }
    float u(void){ return p_u ; }
    float v(void){ return p_v ; }

	void setS(float s){ p_s = s ; }
    void setT(float t){ p_t = t ; }
    void setU(float u){ p_u = u ; }
    void setV(float v){ p_v = v ; }
    
    void draw(void) ; 
    void drawWithNames(void) ;

    QString toString(void) ; 
    
    friend std::ostream& operator <<(std::ostream& stream, QPoint4f & r) ;
    friend std::ostream& operator <<(std::ostream& stream, QPoint4f * r) ;
    
    bool equals(QPoint4f * p) ;
    bool equals(QPoint4f * p, double tolerance) ;

public slots:
    float size(void) ;
    void setSize(float size) ;

/* The following functions are provided because QSA 1.2.1 does handle super methods. */
    void select(void) { ((QAxelObject *)this)->select() ; }
    QAxelObject * toObject(void) { return (QAxelObject *)this ; }
    
private :
    float p_size ;
    static int id_ ;
	float p_s ;
	float p_t ;
	float p_u ;
	float p_v ;
} ;

} ; // namespace QAxel

# endif
