/*****************************************************************************
 * Axel
 *****************************************************************************
 * RationalSurface.h
 * 2007-05-22
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# ifndef QRATIONALSURFACE_H
# define QRATIONALSURFACE_H

# include <QAxel/QParametricSurface.h>

namespace QAxel {

class QPoint3f ;
class QMesh ;
class QBoundingBox ;

/****************************************************************************/
/*!
  \class QRationalSurface QSurface.h
  \brief The class QRationalSurface provides ...

  More ...
*/
/****************************************************************************/
class QRationalSurface : public QParametricSurface 
{
    Q_OBJECT

public :
    QRationalSurface(QString px, QString py, QString pz, QString pw,
		     double umin = 0.0, double umax = 1.0,
		     double vmin = 0.0, double vmax = 1.0,
		     double grid_u = 50, double grid_v = 50,
		     QBoundingBox * boundingbox = NULL) ;
    void compute(void) ;
    void draw(void) ;
    void drawWithNames(void) ;

public slots:
    void setGrid(double g_u, double g_v) { grid_u = g_u ; grid_v = g_v ; }
    double gridU(void) { return grid_u ; }
    double gridV(void) { return grid_v ; }
    
    QString px(void) { return px_ ; }
    QString py(void) { return py_ ; }
    QString pz(void) { return pz_ ; }
    QString pw(void) { return pw_ ; }

    QPoint3f * eval(float u, float v) ;

    float umin(void) { return umin_ ; }
    float umax(void) { return umax_ ; }
    float vmin(void) { return vmin_ ; }
    float vmax(void) { return vmax_ ; }
    
    void setUMin(double u) { umin_ = u ; }
    void setUMax(double u) { umax_ = u ; }
    void setVMin(double v) { vmin_ = v ; }
    void setVMax(double v) { vmax_ = v ; }
    
    QAxelObject * toObject(void) { return (QAxelObject *)this ; }
    
private :
    QString px_ ;
    QString py_ ;
    QString pz_ ;
    QString pw_ ;
    double grid_u, grid_v ;
    double umin_, umax_ ;
    double vmin_, vmax_ ;
    static int id_ ;
} ;

} ;

# endif
