/*****************************************************************************
 * Axel
 *****************************************************************************
 * Surfaces in Axel
 * 2007-05-22
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# ifndef QSURFACE_H
# define QSURFACE_H

# include <QAxel/QAxelObject.h>

namespace QAxel {

class QDisplayList ;
class QBoundingBox ;

/****************************************************************************/
/*!
  \class QSurface QSurface.h
  \brief The class QSurface is an abstract class for surfaces in Axel.

  More ...
*/
/****************************************************************************/
class QSurface : public QAxelObject 
{
    Q_OBJECT
	
public:
     QSurface(void) ;
     QSurface(QBoundingBox * boundingbox) ;
     QSurface(double xmin, double xmax, 
	      double ymin, double ymax, 
	      double zmin = 0, double zmax = 0) ;
    ~QSurface(void) ;
		
    virtual void draw(void) = 0 ;
    virtual void drawWithNames(void) = 0 ;
	
    bool bbDrawn(void) { 
	return bbdrawn_ ; 
    }
    
    void setBBDrawn(bool b) { 
	bbdrawn_ = b ; 
    }

    QBoundingBox * boundingbox(void) { 
	return boundingbox_ ; 
    }

    void setBoundingBox(QBoundingBox * box) {
	boundingbox_ = box ;
    }

public slots:
    QAxelObject * toObject(void) { 
	return (QAxelObject *)this ; 
    }

protected:
    QBoundingBox * boundingbox_ ;
    bool bbdrawn_ ;

protected:
    bool           s_dirty ;
    QDisplayList * s_dlist ;
} ;

} ; // namespace QAxel

# endif
