/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QVertex
 * 2006-03-25
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :         
 *
 ****************************************************************************/

# include <iostream>

# include <QAxel/QVertex.h>
# include <QAxel/QMesh.h>

namespace QAxel {

int QVertex::id_ = 0 ;

QVertex::QVertex(QAxelObject * parent) : QAxelObject(parent) 
{
    setObjectColor(new QColor(255,128,0)) ;  
    
    QString str ;
    setProperty("type", "Vertex") ;
    setProperty("name", "Vertex"+str.setNum(id_)) ;
    
    id_++ ;	
}

QVertex::QVertex(float x, float y, float z, QAxelObject * parent) : QAxelObject(parent)
{
    frame_->setPosition(x,y,z) ;
    
    QString str ;
    setProperty("type", "Vertex") ;
    setProperty("name", "Vertex"+str.setNum(id_)) ;
    
    id_++ ;	

    setObjectColor(new QColor(255,128,0)) ;  
}                                             

QVertex::~QVertex(void)
{

}

float QVertex::x(void) 
{
    if(parent()) 
	return parent()->frame()->position().x + frame()->position().x ;
    else
	return frame()->position().x ;
}

float QVertex::y(void) 
{ 
    if(parent()) 
	return parent()->frame()->position().y + frame()->position().y ;
    else
	return frame()->position().y ;
}

float QVertex::z(void) 
{ 
    if(parent()) 
	return parent()->frame()->position().z + frame()->position().z ;
    else
	return frame()->position().z ;
}

QString QVertex::toString(void)
{
    QString text ; QString tmp ;
    text += name() + "(" + tmp.setNum(x()) + ", " + tmp.setNum(y()) + ", " + tmp.setNum(z()) + ")" ;
    return text ;
}

std::ostream& operator << (std::ostream& stream, QVertex & v) 
{ 
    return stream << v.toString().toLatin1().data() ; 
}

std::ostream& operator << (std::ostream& stream, QVertex * v) 
{ 
    return stream << v->toString().toLatin1().data() ; 
}

// -------------- TO BE REMOVED ----------------
QVertex::QVertex(int index, float x, float y, float z, QMesh * m) : QAxelObject(0)
{
    frame_->setPosition(x,y,z) ;
    mesh_ = m ;	                       
    
    QString str ;
    setProperty("type", "Vertex") ;
    setProperty("name", "Vertex"+str.setNum(id_)) ;
    
    id_++ ;	
    setObjectColor(new QColor(255,128,0)) ;  
    
    m->glvertices_[3*index+0] = x ;	
    m->glvertices_[3*index+1] = y ;
    m->glvertices_[3*index+2] = z ;
    
    index_ = index ;
    
    connect(frame_, SIGNAL(modified()), this, SLOT(updatePosition())) ;
}                                             

void QVertex::updatePosition(void) 
{
    mesh_->glvertices_[3*index_+0] = x() ;
    mesh_->glvertices_[3*index_+1] = y() ;
    mesh_->glvertices_[3*index_+2] = z() ;
}
                                                     
int QVertex::index(void) { return index_ ; }
void QVertex::setIndex(int index) { index_ = index ; }

QMesh * QVertex::mesh(void) { return mesh_ ; }

void QVertex::setNormal(float x, float y, float z) 
{ 
    normal_.x = x ;
    normal_.y = y ;
    normal_.z = z ;
    mesh_->glnormals_[3*index_+0] = x ;
    mesh_->glnormals_[3*index_+1] = y ;
    mesh_->glnormals_[3*index_+2] = z ;
}     

void QVertex::addToNormal(float x, float y, float z) 
{
    normal_.x += x ;
    normal_.y += y ;
    normal_.z += z ;
    mesh_->glnormals_[3*index_+0] += x ;
    mesh_->glnormals_[3*index_+1] += y ;
    mesh_->glnormals_[3*index_+2] += z ;
}

void QVertex::setColor(float r, float g, float b) 
{ 
    setObjectColor(new QColor((int)(r*255),(int)(g*255),(int)(b*255))) ; 
}
                      
void QVertex::setUVMapping(float u, float v) 
{
    this->u = u ;
    this->v = v ;
}

void QVertex::draw(void) 
{
    glPushMatrix() ;
    glMultMatrixd(frame_->matrix()) ;
    glColor() ;
    glBegin(GL_POINTS) ;
    glVertex3i(0,0,0) ;
    glEnd() ;
    glPopMatrix() ;
}   

void QVertex::drawWithNames(void) 
{      
    glPushName((GLuint)this);
    draw();
    glPopName();
}
// ---------------------------------------------

} ; // namespace QAxel
