/*****************************************************************************
 * Axel
 *****************************************************************************
 * QBoundingBox4D4D
 * 2007-04-18
 * Stephane Chau
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# include <QMessageBox>

# include <QData/QBoundingBox4D.h>

namespace QData {

int QBoundingBox4D::id_ = 0 ;

QBoundingBox4D::QBoundingBox4D(void)
{
    QString str ;
    setProperty("type", "BoundingBox4D") ;
    setProperty("name", "BoundingBox4D"+str.setNum(id_)) ;
    id_++ ;
    
    b_smin = 0 ; b_smax = 1 ;
    b_tmin = 0 ; b_tmax = 1 ;
    b_umin = 0 ; b_umax = 1 ;
    b_vmin = 0 ; b_vmax = 1 ;
    
    setObjectColor(new QColor(255,255,255)) ;
//	setObjectColor(new QColor(0,0,0)) ;
}

QBoundingBox4D::QBoundingBox4D(float smin, float smax, float tmin, float tmax, float umin, float umax, float vmin, float vmax)
{
	b_smin = smin ; b_smax = smax ;
    b_tmin = tmin ; b_tmax = tmax ;
	b_umin = umin ; b_umax = umax ;
	b_vmin = vmin ; b_vmax = vmax ;
    
    if(!isValid())
	QMessageBox::warning(0, "Axel", "BoundingBox4D coordinates are FALSE\n Problems WILL occur") ;
    
    QString str ;
    setProperty("type", "BoundingBox4D") ;
    setProperty("name", "BoundingBox4D"+str.setNum(id_)) ;
    id_++ ;
    
    setObjectColor(new QColor(255,255,255)) ;
//	setObjectColor(new QColor(0,0,0)) ;
}

void QBoundingBox4D::draw(void) // 
{
    // glDisable(GL_LIGHTING) ;
    // glPushMatrix() ;
    // glMultMatrixd(frame_->matrix()) ;
    // glColor() ;
    // glBegin(GL_LINE_LOOP) ;
    // // bottom face
    // glVertex3f(xmin_, ymin_, zmin_) ;
    // glVertex3f(xmax_, ymin_, zmin_) ;
    // glVertex3f(xmax_, ymin_, zmax_) ;
    // glVertex3f(xmin_, ymin_, zmax_) ;
    // glEnd() ;
    // 
    // glBegin(GL_LINE_LOOP) ;
    // // left face
    // glVertex3f(xmin_, ymin_, zmin_) ;
    // glVertex3f(xmin_, ymax_, zmin_) ;
    // glVertex3f(xmin_, ymax_, zmax_) ;
    // glVertex3f(xmin_, ymin_, zmax_) ;
    // glEnd() ;
    // 
    // glBegin(GL_LINE_LOOP) ;
    // // top face
    // glVertex3f(xmax_, ymax_, zmax_) ;
    // glVertex3f(xmin_, ymax_, zmax_) ;
    // glVertex3f(xmin_, ymax_, zmin_) ;
    // glVertex3f(xmax_, ymax_, zmin_) ;
    // glEnd() ;
    // 
    // glBegin(GL_LINE_LOOP) ;
    // // right face
    // glVertex3f(xmax_, ymax_, zmax_) ;
    // glVertex3f(xmax_, ymin_, zmax_) ;
    // glVertex3f(xmax_, ymin_, zmin_) ;
    // glVertex3f(xmax_, ymax_, zmin_) ;
    // glEnd() ;
    // 
    // glBegin(GL_LINE_LOOP) ;
    // // front face
    // glVertex3f(xmin_, ymin_, zmin_) ;
    // glVertex3f(xmin_, ymax_, zmin_) ;
    // glVertex3f(xmax_, ymax_, zmin_) ;
    // glVertex3f(xmax_, ymin_, zmin_) ;
    // glEnd() ;
    // 
    // glBegin(GL_LINE_LOOP) ;
    // // back face
    // glVertex3f(xmin_, ymin_, zmax_) ;
    // glVertex3f(xmin_, ymax_, zmax_) ;
    // glVertex3f(xmax_, ymax_, zmax_) ;
    // glVertex3f(xmax_, ymin_, zmax_) ;
    // glEnd() ;
    // glPopMatrix() ;	
    // glEnable(GL_LIGHTING) ;
}

void QBoundingBox4D::drawWithNames(void) 
{
    // Since we do not want a BB to be an independant
    // object, we do not allow selection, therefor, manipulation
}

QString QBoundingBox4D::toString(void) 
{
    QString smin ; smin.setNum(b_smin) ;
    QString smax ; smax.setNum(b_smax) ;
    QString tmin ; tmin.setNum(b_tmin) ;
    QString tmax ; tmax.setNum(b_tmax) ;
    QString umin ; umin.setNum(b_umin) ;
    QString umax ; umax.setNum(b_umax) ;
    QString vmin ; vmin.setNum(b_vmin) ;
    QString vmax ; vmax.setNum(b_vmax) ;

    QString r = "[" 
	+ smin + ", " + smax + ", "
	+ tmin + ", " + tmax + ", "
	+ umin + ", " + umax + ", "
	+ vmin + ", " + vmax + "]" ;
    return r ;
}
    
float QBoundingBox4D::size(void)
{
    return abs(b_smax-b_smin) ;
}

QPoint4f * QBoundingBox4D::center(void)
{
    float s = (b_smin+b_smax)/2 ;
    float t = (b_tmin+b_tmax)/2 ;
    float u = (b_umin+b_umax)/2 ;
    float v = (b_vmin+b_vmax)/2 ;
    return new QPoint4f(s,t,u,v) ;
}

// QList<QPoint4f *> QBoundingBox::corners(void)
// {
//     QList<QPoint4f *> points ;
//     points << new QPoint3f(xmin_, ymin_, 0.0) ;
//     points << new QPoint3f(xmax_, ymin_, 0.0) ;
//     points << new QPoint3f(xmin_, ymax_, 0.0) ;
//     points << new QPoint3f(xmax_, ymax_, 0.0) ;
//     return points ;
// }

bool QBoundingBox4D::isNull()
{
    return b_smax == b_smin && b_tmax == b_tmin && b_umax == b_umin && b_vmax == b_vmin ;
}
    
bool QBoundingBox4D::isEmpty()
{ 
    return b_smax < b_smin || b_tmax < b_tmin || b_umax < b_umin || b_vmax < b_vmin ;
}

bool QBoundingBox4D::isValid()
{
    return b_smin <= b_smax && b_tmin <= b_tmax && b_umin <= b_umax && b_vmin <= b_vmax ;
}

    
/****************************************************************************
 * Intersection test
 * CAUTION: in STRICT case, to be able to deal with 2D Bounding Boxes we keep
 * leq test for z coordinates.
 *
 * That is a problem for REAL 3D Bounding Boxes.
 * -> BoundingBox2D ?
 * -> BoundingBox3D ?
 ****************************************************************************/

bool QBoundingBox4D::intersects(QBoundingBox4D * b, bool strict)
{
    if (isNull() || b->isNull())
        return false;

    QBoundingBox4D * b1 = normalized();
    QBoundingBox4D * b2 = b->normalized();
	bool inequal = ( qMax(b1->smin(), b2->smin()) <= qMin(b1->smax(), b2->smax()) &&
					qMax(b1->tmin(), b2->tmin()) <= qMin(b1->tmax(), b2->tmax()) &&
					qMax(b1->umin(), b2->umin()) <= qMin(b1->umax(), b2->umax()) &&
					qMax(b1->vmin(), b2->vmin()) <= qMin(b1->vmax(), b2->vmax()) ) ;
	bool inequalStrict = inequal && 
							( qMax(b1->smin(), b2->smin()) < qMin(b1->smax(), b2->smax()) ||
								qMax(b1->tmin(), b2->tmin()) < qMin(b1->tmax(), b2->tmax()) ||
								qMax(b1->umin(), b2->umin()) < qMin(b1->umax(), b2->umax()) ||
								qMax(b1->vmin(), b2->vmin()) < qMin(b1->vmax(), b2->vmax()) ) ;
	return strict ? inequalStrict : inequal ;
}

bool QBoundingBox4D::unites(QBoundingBox4D * b, bool strict)
{
  Q_UNUSED(b) ;
  Q_UNUSED(strict) ;
	//     if (isNull() || b->isNull())
	// return false ;
	//     QBoundingBox4D * b1 = normalized();
	//     QBoundingBox4D * b2 = b->normalized();
	//     if(strict)
	// return ( qMin(b1->xmin(), b2->xmin()) <  qMax(b1->xmax(), b2->xmax()) &&
	// 	 qMin(b1->ymin(), b2->ymin()) <  qMax(b1->ymax(), b2->ymax()) &&
	// 	 qMin(b1->zmin(), b2->zmin()) <= qMax(b1->zmax(), b2->zmax())) ;
	//     else
	// return ( qMin(b1->xmin(), b2->xmin()) <= qMax(b1->xmax(), b2->xmax()) &&
	// 	 qMin(b1->ymin(), b2->ymin()) <= qMax(b1->ymax(), b2->ymax()) &&
	// 	 qMin(b1->zmin(), b2->zmin()) <= qMax(b1->zmax(), b2->zmax())) ;
	return 0 ;
}

QBoundingBox4D * QBoundingBox4D::normalized(void)
{
    if(isNull()) return this ;
    QBoundingBox4D * b = new QBoundingBox4D() ;
    if (b_smax < b_smin) {
        b->setSMin(b_smax) ;
        b->setSMax(b_smin) ;
    } else {
        b->setSMin(b_smin) ;
        b->setSMax(b_smax) ;
    }
    if (b_tmax < b_tmin) {
        b->setTMin(b_tmax) ;
        b->setTMax(b_tmin) ;
    } else {
        b->setTMin(b_tmin) ;
        b->setTMax(b_tmax) ;
    }
    if (b_umax < b_umin) {
        b->setUMin(b_umax) ;
        b->setUMax(b_umin) ;
    } else {
        b->setUMin(b_umin) ;
        b->setUMax(b_umax) ;
    }
    if (b_vmax < b_vmin) {
        b->setVMin(b_vmax) ;
        b->setVMax(b_vmin) ;
    } else {
        b->setVMin(b_vmin) ;
        b->setVMax(b_vmax) ;
    }
    return b ;
}

QBoundingBox4D * QBoundingBox4D::intersect(QBoundingBox4D * b)
{
    if (isNull() || b->isNull())
        return new QBoundingBox4D ;
    QBoundingBox4D * b1 = normalized();
    QBoundingBox4D * b2 = b->normalized();
    QBoundingBox4D * tmp = new QBoundingBox4D ;
    tmp->setSMin(qMax(b1->smin(), b2->smin())) ;
    tmp->setSMax(qMin(b1->smax(), b2->smax())) ;
    tmp->setTMin(qMax(b1->tmin(), b2->tmin())) ;
    tmp->setTMax(qMin(b1->tmax(), b2->tmax())) ;
    tmp->setUMin(qMax(b1->umin(), b2->umin())) ;
    tmp->setUMax(qMin(b1->umax(), b2->umax())) ;
    tmp->setVMin(qMax(b1->vmin(), b2->vmin())) ;
    tmp->setVMax(qMin(b1->vmax(), b2->vmax())) ;

    return tmp->isEmpty() ? new QBoundingBox4D : tmp ; 
	// return tmp ;
}

QBoundingBox4D * QBoundingBox4D::intersected(QBoundingBox4D * b)
{
    return intersect(b) ;
}

QBoundingBox4D * QBoundingBox4D::unite(QBoundingBox4D * b)
{
    if (isNull())
        return b ;
    if (b->isNull())
        return this ;
    QBoundingBox4D * b1 = normalized();
    QBoundingBox4D * b2 = b->normalized();
    QBoundingBox4D * tmp = new QBoundingBox4D;
    tmp->setSMin(qMin(b1->smin(), b2->smin()));
    tmp->setSMax(qMax(b1->smax(), b2->smax()));
    tmp->setTMin(qMin(b1->tmin(), b2->tmin()));
    tmp->setTMax(qMax(b1->tmax(), b2->tmax()));
    tmp->setUMin(qMin(b1->umin(), b2->umin()));
    tmp->setUMax(qMax(b1->umax(), b2->umax()));
    tmp->setVMin(qMin(b1->vmin(), b2->vmin()));
    tmp->setVMax(qMax(b1->vmax(), b2->vmax()));
    return tmp;
}

QBoundingBox4D * QBoundingBox4D::united(QBoundingBox4D * b)
{
    return unite(b) ;
}


std::ostream& operator << (std::ostream& stream, QBoundingBox4D & b)
{
    return stream << b.toString().toLatin1().data() ;
}

std::ostream& operator << (std::ostream& stream, QBoundingBox4D * b)
{
    return stream << b->toString().toLatin1().data() ;
}

float QBoundingBox4D::abs(float a)
{
    return (a<0?-a:a) ;
}

QBoundingBox4D * QBoundingBox4D::intersect(QBoundingBox4D * b1, QBoundingBox4D * b2)
{
    return b1->intersect(b2) ;
}

QBoundingBox4D * QBoundingBox4D::unite(QBoundingBox4D * b1, QBoundingBox4D * b2)
{
    return b1->unite(b2) ;
}

bool QBoundingBox4D::intersects(QBoundingBox4D * b1, QBoundingBox4D * b2)
{
    return b1->intersects(b2) ;
}

bool QBoundingBox4D::unites(QBoundingBox4D * b1, QBoundingBox4D * b2)
{
    return b1->unites(b2) ;
}

} ; // namespace QAxel
