/*****************************************************************************
 * Axel
 *****************************************************************************
 * QBoundingBox4D
 * 2007-04-18
 * Stephane Chau
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# ifndef QBOUNDINGBOX4D_H
# define QBOUNDINGBOX4D_H

# include <QAxel/QAxelObject.h>
# include <QAxel/QPoint4f.h>

using namespace QAxel ;

namespace QData {

/****************************************************************************/
/*!
  \class QBoundingBox QBoundingBox4D.h

  More ...
*/
/****************************************************************************/
class QBoundingBox4D : public QAxelObject 
{
    Q_OBJECT
			
public:
    QBoundingBox4D(void) ;
    QBoundingBox4D(float smin, float smax, float tmin=0, float tmax=0, float umin=0, float umax=0, float vmin=0, float vmax=0) ;
	
    float smin(void) { return b_smin ; }
    float smax(void) { return b_smax ; }
    float tmin(void) { return b_tmin ; }
    float tmax(void) { return b_tmax ; }
    float umin(void) { return b_umin ; }
    float umax(void) { return b_umax ; }
    float vmin(void) { return b_vmin ; }
    float vmax(void) { return b_vmax ; }
	
    void setSMin(float s) { b_smin = s ; }
    void setSMax(float s) { b_smax = s ; }
    void setTMin(float t) { b_tmin = t ; }
    void setTMax(float t) { b_tmax = t ; }
    void setUMin(float u) { b_umin = u ; }
    void setUMax(float u) { b_umax = u ; }
    void setVMin(float v) { b_vmin = v ; }
    void setVMax(float v) { b_vmax = v ; }
	
	
    void draw(void) ;
    void drawWithNames(void) ;
    QString toString(void) ; 
   
    float abs(float a) ;

    float size(void) ;
	QPoint4f * center(void) ;
	// QList<QPoint4f *> corners(void) ;

    bool isNull(void) ;
    bool isEmpty(void) ;
    bool isValid(void) ;

    bool intersects(QBoundingBox4D * b, bool strict = true) ;
    bool unites(QBoundingBox4D * b, bool strict = true) ;

    
    QBoundingBox4D * normalized(void) ;
    QBoundingBox4D * intersect(QBoundingBox4D * b) ;
    QBoundingBox4D * intersected(QBoundingBox4D * b) ;
    QBoundingBox4D * unite(QBoundingBox4D * b) ;
    QBoundingBox4D * united(QBoundingBox4D * b) ;
    
    friend std::ostream& operator <<(std::ostream& stream, QBoundingBox4D & b) ;
    friend std::ostream& operator <<(std::ostream& stream, QBoundingBox4D * b) ;
    
public:
    static QBoundingBox4D * intersect(QBoundingBox4D * b1, QBoundingBox4D * b2) ;
    static QBoundingBox4D * unite    (QBoundingBox4D * b1, QBoundingBox4D * b2) ;

    static bool intersects(QBoundingBox4D * b1, QBoundingBox4D * b2) ;
    static bool unites    (QBoundingBox4D * b1, QBoundingBox4D * b2) ;

protected:	
    static int id_ ;
    float b_smin, b_smax ;
    float b_tmin, b_tmax ;
    float b_umin, b_umax ;
    float b_vmin, b_vmax ;
} ;

} ; // namespace QAxel

# endif
