/*****************************************************************************
 * Axel
 *****************************************************************************
 * QCell
 * 2007-01-10
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * TODO : What if two objects intersect on the border of the cell ?
 ****************************************************************************/

# include <QAxel/QPoint3f.h>

# include <QData/QCell.h>

namespace QData {

QCell::QCell(void) : QBoundingBox()
{	
	QString str ;
	setProperty("type", "Cell") ;
	setProperty("name", "cell"+str.setNum(id_)) ;
	id_++ ;	
}

QCell::QCell(QBoundingBox * bb) : QBoundingBox(bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax(), bb->zmin(), bb->zmax())
{
	QString str ;
	setProperty("type", "Cell") ;
	setProperty("name", "cell"+str.setNum(id_)) ;
	id_++ ;
}

QCell::QCell(float xmin, float xmax, float ymin, float ymax, float zmin, float zmax) : QBoundingBox(xmin, xmax, ymin, ymax, zmin, zmax)
{
	QString str ;
	setProperty("type", "Cell") ;
	setProperty("name", "cell"+str.setNum(id_)) ;
	id_++ ;
}

QList<QCell *> QCell::subdivide2D(QPoint3f * center)
{
	QList<QCell *> l ;
	
	QCell * NW = new QCell(xmin_, center->x(), center->y(), ymax_) ; l << NW ;
	QCell * NE = new QCell(center->x(), xmax_, center->y(), ymax_) ; l << NE ;
	QCell * SW = new QCell(xmin_, center->x(), ymin_, center->y()) ; l << SW ;
	QCell * SE = new QCell(center->x(), xmax_, ymin_, center->y()) ; l << SE ;
	
	foreach(QPoint3f * p, n_intersections) {
		if(p->x() <= center->x()) NW->pushNorthIntersection(p) ;
		if(p->x() >  center->x()) NE->pushNorthIntersection(p) ;
	}
	
	foreach(QPoint3f * p, s_intersections) {
		if(p->x() <= center->x()) SW->pushSouthIntersection(p) ;
		if(p->x() >  center->x()) SE->pushSouthIntersection(p) ;
	}

	foreach(QPoint3f * p, w_intersections) {
		if(p->y() <= center->y()) SW->pushWestIntersection(p) ;
		if(p->y() >  center->y()) NW->pushWestIntersection(p) ;
	}
	
	foreach(QPoint3f * p, e_intersections) {
		if(p->y() <= center->y()) SE->pushEastIntersection(p) ;
		if(p->y() >  center->y()) NE->pushEastIntersection(p) ;
	}

	return l ;
}

QList<QCell *> QCell::subdivide3D(QPoint3f * center)
{
	QList<QCell *> l ;

	QCell * FNW = new QCell(xmin_, center->x(), center->y(), ymax_, zmin_, center->z()) ; l << FNW ;
	QCell * FNE = new QCell(center->x(), xmax_, center->y(), ymax_, zmin_, center->z()) ; l << FNE ;
	QCell * FSW = new QCell(xmin_, center->x(), ymin_, center->y(), zmin_, center->z()) ; l << FSW ;
	QCell * FSE = new QCell(center->x(), xmax_, ymin_, center->y(), zmin_, center->z()) ; l << FSE ;

	QCell * BNW = new QCell(xmin_, center->x(), center->y(), ymax_, center->z(), zmax_) ; l << BNW ;
	QCell * BNE = new QCell(center->x(), xmax_, center->y(), ymax_, center->z(), zmax_) ; l << BNE ;
	QCell * BSW = new QCell(xmin_, center->x(), ymin_, center->y(), center->z(), zmax_) ; l << BSW ;
	QCell * BSE = new QCell(center->x(), xmax_, ymin_, center->y(), center->z(), zmax_) ; l << BSE ;
	
	// In 3D Intersections are on faces, not on edges so ...
	
	return l ;
}

float QCell::abs(float a) { return (a<0)?-a:a ; }

/*
QBoundingBox * QBoundingBox::normalized(void)
{
    if(isNull() || width() == 0 || height() == 0) return this ;
    QBoundingBox * b = new QBoundingBox() ;
    if (xmax_ < xmin_) {
        b->setXMin(xmax_) ;
        b->setXMax(xmin_) ;
    } else {
        b->setXMin(xmin_) ;
        b->setXMax(xmax_) ;
    }
    if (ymax_ < ymin_) {
        b->setYMin(ymax_) ;
        b->setYMax(ymin_) ;
    } else {
        b->setYMin(ymin_) ;
        b->setYMax(ymax_) ;
    }
    if (zmax_ < zmin_) {
        b->setZMin(zmax_) ;
        b->setZMax(zmin_) ;
    } else {
        b->setZMin(zmin_) ;
        b->setZMax(zmax_) ;
    }
    return b ;
}

QCell * QCell::intersect(QCell * c)
{
    if (isNull() || b->isNull())
        return new QCell ;
    QCell * c1 = normalized();
    QCell * c2 = c->normalized();
    float xmin = qMax(b1->xmin(), b2->xmin()) ;
    float xmax = qMin(b1->xmax(), b2->xmax()) ;
    float ymin = qMax(b1->ymin(), b2->ymin()) ;
    float ymax = qMin(b1->ymax(), b2->ymax()) ;
    float zmin = qMax(b1->zmin(), b2->zmin()) ;
    float zmax = qMin(b1->zmax(), b2->zmax()) ;
    QCell * tmp = new QCell(xmin, xmax, ymin, ymax, zmin, zmax) ;

    if (tmp->isEmpty()) return new QCell ;
    
    // Inherit intersections of objects with the border of the cell
	if(c1->westIntersections().size() > 0 && c1->westIntersections().first()->x() == xmin) tmp->setWestIntersections(c1->westIntersections()) ;
	if(c2->westIntersections().size() > 0 && c2->westIntersections().first()->x() == xmin) tmp->setWestIntersections(c2->westIntersections()) ;
	if(c1->eastIntersections().size() > 0 && c1->eastIntersections().first()->x() == xmax) tmp->setEastIntersections(c1->eastIntersections()) ;
	if(c2->eastIntersections().size() > 0 && c2->eastIntersections().first()->x() == xmax) tmp->setEastIntersections(c2->eastIntersections()) ;
	if(c1->southIntersections().size() > 0 && c1->southIntersections().first()->y() == ymin) tmp->setSouthIntersections(c1->southIntersections()) ;
	if(c2->southIntersections().size() > 0 && c2->southIntersections().first()->y() == ymin) tmp->setSouthIntersections(c2->southIntersections()) ;
	if(c1->northIntersections().size() > 0 && c1->northIntersections().first()->y() == ymax) tmp->setNorthIntersections(c1->northIntersections()) ;
	if(c2->northIntersections().size() > 0 && c2->northIntersections().first()->y() == ymax) tmp->setNorthIntersections(c2->northIntersections()) ;
	
	// Inherit singularities of objects
	foreach(QPoint3f * s, c1->singularPoints()) if(s->x() >= xmin && s->x() <= xmax && s->y() >= ymin && s->y() <= ymax) tmp->pushSingularPoint(s) ;
	foreach(QPoint3f * s, c2->singularPoints()) if(s->x() >= xmin && s->x() <= xmax && s->y() >= ymin && s->y() <= ymax) tmp->pushSingularPoint(s) ;
	
	// Inherit intersections of objects
	foreach(QPoint3f * s, c1->intersectionPoints()) if(s->x() >= xmin && s->x() <= xmax && s->y() >= ymin && s->y() <= ymax) tmp->pushIntersectionPoint(s) ;
	foreach(QPoint3f * s, c2->intersectionPoints()) if(s->x() >= xmin && s->x() <= xmax && s->y() >= ymin && s->y() <= ymax) tmp->pushIntersectionPoint(s) ;
	
    return tmp ;
}

QCell * QCell::intersected(QCell * c)
{
    return intersect(c) ;
}

QCell * QCell::unite(QCell * c)
{
    if (isNull())
        return c ;
    if (c->isNull())
        return this ;
    QCell * b1 = normalized() ;
    QCell * b2 = b->normalized() ;
    QCell * tmp = new QCell ;
    tmp->setXMin(qMin(c1->xmin(), c2->xmin()));
    tmp->setXMax(qMax(c1->xmax(), c2->xmax()));
    tmp->setYMin(qMin(c1->ymin(), c2->ymin()));
    tmp->setYMax(qMax(c1->ymax(), c2->ymax()));
    return tmp;
}

QBoundingBox * QBoundingBox::united(QBoundingBox * b)
{
	return unite(b) ;
}

*/

/********************************************************/

QCell * QCell::intersection(QCell * c1, QCell * c2)
{
	// Is the intersection non empty ?
	if(c1->xmin() < c1->xmax() && c1->xmax() <= c2->xmin() && c2->xmin() < c2->xmax()) return NULL ;
	if(c2->xmin() < c2->xmax() && c2->xmax() <= c1->xmin() && c1->xmin() < c1->xmax()) return NULL ;
	if(c1->ymin() < c1->ymax() && c1->ymax() <= c2->ymin() && c2->ymin() < c2->ymax()) return NULL ;
	if(c2->ymin() < c2->ymax() && c2->ymax() <= c1->ymin() && c1->ymin() < c1->ymax()) return NULL ;
	
	// Compute cell size
	float xmin = (c1->xmin()<c2->xmin()) ? c1->xmin() : c2->xmin() ;
	float xmax = (c1->xmax()<c2->xmax()) ? c1->xmax() : c2->xmax() ;
	float ymin = (c1->ymin()<c2->ymin()) ? c1->ymin() : c2->ymin() ;
	float ymax = (c1->ymax()<c2->ymax()) ? c1->ymax() : c2->ymax() ;
	float zmin = (c1->zmin()<c2->zmin()) ? c1->zmin() : c2->zmin() ;
	float zmax = (c1->zmax()<c2->zmax()) ? c1->zmax() : c2->zmax() ;
	QCell * c = new QCell(xmin, xmax, ymin, ymax, zmin, zmax) ;
	
	// Inherit intersections of objects with the border of the cell
	if(c1->westIntersections().size() > 0 && c1->westIntersections().first()->x() == xmin) c->setWestIntersections(c1->westIntersections()) ;
	if(c2->westIntersections().size() > 0 && c2->westIntersections().first()->x() == xmin) c->setWestIntersections(c2->westIntersections()) ;
	if(c1->eastIntersections().size() > 0 && c1->eastIntersections().first()->x() == xmax) c->setEastIntersections(c1->eastIntersections()) ;
	if(c2->eastIntersections().size() > 0 && c2->eastIntersections().first()->x() == xmax) c->setEastIntersections(c2->eastIntersections()) ;
	if(c1->southIntersections().size() > 0 && c1->southIntersections().first()->y() == ymin) c->setSouthIntersections(c1->southIntersections()) ;
	if(c2->southIntersections().size() > 0 && c2->southIntersections().first()->y() == ymin) c->setSouthIntersections(c2->southIntersections()) ;
	if(c1->northIntersections().size() > 0 && c1->northIntersections().first()->y() == ymax) c->setNorthIntersections(c1->northIntersections()) ;
	if(c2->northIntersections().size() > 0 && c2->northIntersections().first()->y() == ymax) c->setNorthIntersections(c2->northIntersections()) ;
	
	// Inherit singularities of objects
	foreach(QPoint3f * s, c1->singularPoints()) if(s->x() >= xmin && s->x() <= xmax && s->y() >= ymin && s->y() <= ymax) c->pushSingularPoint(s) ;
	foreach(QPoint3f * s, c2->singularPoints()) if(s->x() >= xmin && s->x() <= xmax && s->y() >= ymin && s->y() <= ymax) c->pushSingularPoint(s) ;
	
	// Inherit (x&y)trem points
	foreach(QPoint3f * s, c1->xtremPoints())    if(s->x() >= xmin && s->x() <= xmax && s->y() >= ymin && s->y() <= ymax) c->pushSingularPoint(s) ;
	foreach(QPoint3f * s, c1->ytremPoints())    if(s->x() >= xmin && s->x() <= xmax && s->y() >= ymin && s->y() <= ymax) c->pushSingularPoint(s) ;

	// Inherit intersections of objects
	// foreach(QPoint3f * s, c1->intersectionPoints()) if(s->x() >= xmin && s->x() <= xmax && s->y() >= ymin && s->y() <= ymax) c->pushIntersectionPoint(s) ;
	// foreach(QPoint3f * s, c2->intersectionPoints()) if(s->x() >= xmin && s->x() <= xmax && s->y() >= ymin && s->y() <= ymax) c->pushIntersectionPoint(s) ;
	
	return c ;
}

int QCell::id_ = 0 ;

} ; // namespace QData
