/*****************************************************************************
 * A x e l                                                                   *
 ******************************************************************************
 * QRegionEdge                                                               *
 * 2006-10-16                                                                *
 * Julien Wintz                                                              *
 ******************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis                   *
 ******************************************************************************
 * Comments :                                                                *
 *****************************************************************************/

# ifndef QREGIONEDGE_H
# define QREGIONEDGE_H

# include <QAxel/QAxelObject.h>
# include <QAxel/QBoundingBox.h>
# include <QData/QRegionVertex.h>

using namespace QAxel ;

namespace QData {

/****************************************************************************/
/*!
  \class QRegionEdge QRegionEdge.h
  \brief The class QRegionEdge provides ...

  More ...
*/
/****************************************************************************/
class QRegionEdge : public QObject
{
    Q_OBJECT
    Q_ENUMS(MonotonyCode)

public:
    QRegionEdge(QRegionVertex * source, QRegionVertex * dest, QAxelObject * object) ;
    ~QRegionEdge(void) ;

    void draw(void) {}
    void drawWithNames(void) {}

    enum MonotonyCode { 
	Undefined,   // (?,?)
	PlusPlus,    // (+,+) 
	PlusMinus,   // (+,-)
	MinusPlus,   // (-,+)
	MinusMinus,  // (-,-)
	NullPlus,    // (0,+)
	NullMinus,   // (0,-)
	PlusNull,    // (+,0)
	MinusNull,   // (-,0)
	NullNull     // (0,0)
    } ;

public slots:    
    bool contains(QRegionVertex * v) ;
    QRegionVertex * other(QRegionVertex * v) ;
    
    QAxelObject * object(void) ;
    QRegionVertex * source(void) ;
    QRegionVertex * dest(void) ;
    
    QString type(void) ;
    QString toString(void) ;
    
    QList<QRegionVertex *> commonVerticesOf(QRegionEdge * e) ;
    QList<QRegionVertex *> commonVerticesWith(QRegionEdge * e) ;
    
    void replace(QRegionVertex * p1, QRegionVertex * p2) ;
    void switchVertices(void) ;

    bool isCellBorder(void) ;
    bool equals(QRegionEdge * e) ;
    bool alignedWith(QRegionEdge * e) ;
    
    bool mergesWith(QRegionEdge * e) ;
    QList<QRegionVertex *> mergeWith(QRegionEdge * e) ;
	
    QBoundingBox * box(void) ;

    MonotonyCode code(void) ;
    QString codeString(void) ;

public:
    friend std::ostream& operator << (std::ostream& stream, QRegionEdge & e) ;
    friend std::ostream& operator << (std::ostream& stream, QRegionEdge * e) ;
    
private:
    QRegionVertex * e_source ;
    QRegionVertex * e_dest ;
    QAxelObject   * e_object ;
    QBoundingBox  * e_box ;
} ;

} ; // namespace QData

# endif // QREGIONEDGE_H
