/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QTreeNode
 * 2006-07-20
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

/*!***************************************************************************
  This is the constructor for the node, doing some stuff of initialization.
*****************************************************************************/
template <class Object, class Cell> QTreeNode<Object, Cell>::QTreeNode(NODE_TYPE nodeType, QTreeNode<Object, Cell> * theParent, Cell &theCell)
{  
    cell = theCell;

    type = nodeType; 
 
    parent = theParent;

    F_NEchild = NULL; 
    F_NWchild = NULL; 
    F_SEchild = NULL; 
    F_SWchild = NULL; 
	 
    B_NEchild = NULL; 
    B_NWchild = NULL; 
    B_SEchild = NULL; 
    B_SWchild = NULL; 

    depth = theParent->depth+1;

    switch( nodeType )
    { 
    case F_NE: theParent->SetF_NEchild(this); 
	break; 
    case F_NW: theParent->SetF_NWchild(this); 
	break; 
    case F_SW: theParent->SetF_SWchild(this); 
	break; 
    case F_SE: theParent->SetF_SEchild(this); 
	break;
    case B_NE: theParent->SetB_NEchild(this); 
	break; 
    case B_NW: theParent->SetB_NWchild(this);
	break; 
    case B_SW: theParent->SetB_SWchild(this);
	break; 
    case B_SE: theParent->SetB_SEchild(this); 
	break; 
       
    default: std::cerr << "Error : the node's type isn't appropriate \n";
	break;
    }
}

template <class Object, class Cell> QTreeNode<Object, Cell>::QTreeNode(QTreeNode<Object, Cell> * left, QTreeNode<Object, Cell> * right, Cell cell)
{
    this->parent = NULL ;
    this->cell   = cell ;

    left->type  = F_NW ; left->parent =  this ; F_NWchild = left ;
    right->type = F_NE ; right->parent = this ; F_NEchild = right ;

    F_NWchild = left  ; F_SWchild = NULL ;
    F_NEchild = right ; F_SEchild = NULL ;
    B_NEchild = NULL  ; B_NWchild = NULL ; 
    B_SEchild = NULL  ; B_SWchild = NULL ;

    depth = 0 ; left->depth++ ; right->depth++ ;
}

template<class Object, class Cell> QTreeNode<Object, Cell>::QTreeNode(Object object, Cell cell)
{ 
    this->type   = F_NE ;
    this->parent = NULL ;
    this->cell   = cell ;
    this->objects << object ;

    F_NEchild = NULL ; F_NWchild = NULL ; F_SEchild = NULL ; F_SWchild = NULL;
    B_NEchild = NULL ; B_NWchild = NULL ; B_SEchild = NULL ; B_SWchild = NULL;

    depth = 0;
    index = 0;
} 

/*! This is the constructor for the node, doing some stuff of 
  initialization. This constructor must only be called by the
  constructor of Octree. */
template<class Object, class Cell> QTreeNode<Object, Cell>::QTreeNode()
{ 
    type = F_NE;
    parent = NULL; 

    F_NEchild = NULL;F_NWchild = NULL;F_SEchild = NULL;F_SWchild = NULL;
    B_NEchild = NULL;B_NWchild = NULL;B_SEchild = NULL;B_SWchild = NULL;

    depth = 0;

    index = 0;
} 

/************************************************ 
 *           Some additional functions          * 
 ************************************************/ 

/*! Check to see if current node is a leaf or not, remember to check all. */ 
template<class Object, class Cell>
bool QTreeNode<Object, Cell>::isLeaf()
{
    if((F_NEchild == NULL) && (F_NWchild == NULL) &&
       (F_SEchild == NULL) && (F_SWchild == NULL) && 
       (B_NEchild == NULL) && (B_NWchild == NULL) &&
       (B_SEchild == NULL) && (B_SWchild == NULL) )
	return true; 
    return false; 
} 


/**************************************** 
 *     DUE PARTS		        * 
 ****************************************/ 
 
//--------------- DUE SOUTH--------------------- 

template <class Object, class Cell>
void QTreeNode<Object, Cell>::getAllSouthNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs)
{ 
    //get all south neighb of the current node 
    QTreeNode<Object, Cell>* tmpNode = getSouthNeighborNode(); 
    if(tmpNode != NULL) 
	tmpNode->getNorthBoundary(nbrs); 
}

/*! This is for the child to call. */
template<class Object, class Cell>
QTreeNode<Object, Cell>* QTreeNode<Object, Cell>::getSouthNeighborNode()
{
    //get one special node on the south side 
    if(parent == NULL) return NULL; 
    switch(type)
    { 
    case F_NE:{return (parent->GetF_SEchild());} 
    case F_NW:{return (parent->GetF_SWchild());} 
    case F_SW:
    {
	QTreeNode<Object, Cell>* tmp = parent->getSouthNeighborNode(); 
	if(tmp == NULL)
	    return NULL; 
	if(tmp->isLeaf())
	    return tmp; 
	else
	    return (tmp->GetF_NWchild());
    } 
    case F_SE:
    { 
	QTreeNode<Object, Cell>* tmp = parent->getSouthNeighborNode(); 
	if(tmp == NULL)
	    return NULL; 
	if(tmp->isLeaf())
	    return tmp; 
	else
	    return (tmp->GetF_NEchild());
    }		 
    case B_NE:{return (parent->GetB_SEchild());} 
    case B_NW:{return (parent->GetB_SWchild());} 
    case B_SW:
    { 
	QTreeNode<Object, Cell>* tmp = parent->getSouthNeighborNode(); 
	if(tmp == NULL)
	    return NULL; 
	if(tmp->isLeaf())
	    return tmp; 
	else
	    return (tmp->GetB_NWchild());
    } 
    case B_SE:
    { 
	QTreeNode<Object, Cell>* tmp = parent->getSouthNeighborNode(); 
	if(tmp == NULL)
	    return NULL; 
	if(tmp->isLeaf())
	    return tmp; 
	else 
	    return (tmp->GetB_NEchild());
    }		 
    default: 
	return NULL; 
    } 
} 

/*! Get all the node on the north side. */
template<class Object, class Cell>
void QTreeNode<Object, Cell>::getNorthBoundary(std::vector< QTreeNode<Object, Cell>*>& nbrs)
{ 
    if(isLeaf())
	nbrs.push_back(this); 
    else
    { 
	if ( F_NWchild != NULL )
	    F_NWchild->getNorthBoundary(nbrs); 
	if ( F_NEchild != NULL )
	    F_NEchild->getNorthBoundary(nbrs); 
	if ( B_NWchild != NULL )
	    B_NWchild->getNorthBoundary(nbrs); 
	if ( B_NEchild != NULL )
	    B_NEchild->getNorthBoundary(nbrs); 
    } 
} 
 
//---------------DUE NORTH--------------------- 

/*! Comment is the similar for the SOUTH part above. */
template<class Object, class Cell>
void QTreeNode<Object, Cell>::getAllNorthNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs)
{ 
    QTreeNode<Object, Cell>* tmpNode = getNorthNeighborNode(); 
    if(tmpNode != NULL) 
	tmpNode->getSouthBoundary(nbrs); 
} 
 
/*! This is for the child to call. */
template<class Object, class Cell>
QTreeNode<Object, Cell>* QTreeNode<Object, Cell>::getNorthNeighborNode()
{
    if(parent == NULL)
	return NULL; 
    switch(type)
    {
    case F_SE:{return (parent->GetF_NEchild());} 
    case F_SW:{return (parent->GetF_NWchild());} 
    case F_NW:
    { 
	QTreeNode<Object, Cell>* tmp = parent->getNorthNeighborNode(); 
	if(tmp == NULL)
	    return NULL; 
	if(tmp->isLeaf())
	    return tmp; 
	else 
	    return (tmp->GetF_SWchild());
    } 
    case F_NE:
    { 
	QTreeNode<Object, Cell>* tmp = parent->getNorthNeighborNode(); 
	if(tmp == NULL)
	    return NULL; 
	if(tmp->isLeaf())
	    return tmp; 
	else 
	    return (tmp->GetF_SEchild());
    }
    case B_SE:{return (parent->GetB_NEchild());} 
    case B_SW:{return (parent->GetB_NWchild());} 
    case B_NW:
    { 
	QTreeNode<Object, Cell>* tmp = parent->getNorthNeighborNode(); 
	if(tmp == NULL)
	    return NULL; 
	if(tmp->isLeaf())
	    return tmp; 
	else 
	    return (tmp->GetB_SWchild());
    } 
    case B_NE:
    { 
	QTreeNode<Object, Cell>* tmp = parent->getNorthNeighborNode(); 
	if(tmp == NULL)
	    return NULL; 
	if(tmp->isLeaf())
	    return tmp; 
	else 
	    return (tmp->GetB_SEchild());
    } 
    default: 
	return NULL; 
    } 
} 

/*! Get all the node on the north side. */
template <class Object, class Cell>
void QTreeNode<Object, Cell>::getSouthBoundary(std::vector< QTreeNode<Object, Cell>*>& nbrs)
{ 
    if(isLeaf())
	nbrs.push_back(this); 
    else
    { 
	if ( F_SWchild != NULL )
	    F_SWchild->getSouthBoundary(nbrs); 
	if ( F_SEchild != NULL )
	    F_SEchild->getSouthBoundary(nbrs); 
	if ( B_SWchild != NULL )
	    B_SWchild->getSouthBoundary(nbrs); 
	if ( B_SEchild != NULL )
	    B_SEchild->getSouthBoundary(nbrs); 
    } 
} 

//---------------DUE WEST--------------------- 

/*! Comment is the similar for the SOUTH part above. */
template<class Object, class Cell>
void QTreeNode<Object, Cell>::getAllWestNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs)
{ 
    QTreeNode<Object, Cell>* tmpNode = getWestNeighborNode(); 
    if(tmpNode != NULL) 
	tmpNode->getEastBoundary(nbrs); 
} 

/*! This is for the child to call. */
template <class Object, class Cell>
QTreeNode<Object, Cell>* QTreeNode<Object, Cell>::getWestNeighborNode()
{
    if(parent == NULL) 
	return NULL; 
    switch(type)
    { 
    case F_SE:{return (parent->GetF_SWchild());} 
    case F_SW:
    { 
	QTreeNode<Object, Cell>* tmp = parent->getWestNeighborNode(); 
	if(tmp == NULL)
	    return NULL; 
	if(tmp->isLeaf())
	    return tmp; 
	else 
	    return (tmp->GetF_SEchild());
    } 
    case F_NW:
    { 
	QTreeNode<Object, Cell>* tmp = parent->getWestNeighborNode(); 
	if(tmp == NULL)
	    return NULL; 
	if(tmp->isLeaf())
	    return tmp; 
	else 
	    return (tmp->GetF_NEchild());
    } 
    case F_NE:{return (parent->GetF_NWchild());} 
    case B_SE:{return (parent->GetB_SWchild());} 
    case B_SW:
    { 
	QTreeNode<Object, Cell>* tmp = parent->getWestNeighborNode(); 
	if(tmp == NULL)
	    return NULL; 
	if(tmp->isLeaf())
	    return tmp; 
	else 
	    return (tmp->GetB_SEchild());
    } 
    case B_NW:
    { 
	QTreeNode<Object, Cell>* tmp = parent->getWestNeighborNode(); 
	if(tmp == NULL)
	    return NULL; 
	if(tmp->isLeaf())
	    return tmp; 
	else 
	    return (tmp->GetB_NEchild());
    } 
    case B_NE:{return (parent->GetB_NWchild());} 
    default: 
	return NULL; 
    } 
} 

template<class Object, class Cell>
void QTreeNode<Object, Cell>::getEastBoundary(std::vector< QTreeNode<Object, Cell>*>& nbrs)
{ 
    if(isLeaf())
	nbrs.push_back(this); 
    else
    { 
	if ( F_NEchild != NULL )
	    F_NEchild->getEastBoundary(nbrs);
	if ( F_SEchild != NULL )
	    F_SEchild->getEastBoundary(nbrs); 
	if ( B_NEchild != NULL )		 
	    B_NEchild->getEastBoundary(nbrs); 
	if ( B_SEchild != NULL )
	    B_SEchild->getEastBoundary(nbrs); 
    } 
} 

//-------- DUE EAST-------------------- 

/*! Comment is the similar for the SOUTH part above. */ 
template <class Object, class Cell>
void QTreeNode<Object, Cell>::getAllEastNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs)
{ 
    QTreeNode<Object, Cell>* tmpNode = getEastNeighborNode(); 
    if(tmpNode != NULL) 
	tmpNode->getWestBoundary(nbrs); 
} 

/*! This is for the child to call. */
template<class Object, class Cell>
QTreeNode<Object, Cell>* QTreeNode<Object, Cell>::getEastNeighborNode()
{
    if(parent == NULL)
	return NULL; 
    switch(type)
    { 
    case F_SE:
    { 
	QTreeNode<Object, Cell>* tmp = parent->getEastNeighborNode(); 
	if(tmp == NULL)
	    return NULL; 
	if(tmp->isLeaf())
	    return tmp; 
	else 
	    return (tmp->GetF_SWchild());
    }
    case F_SW:{return (parent->GetF_SEchild());} 
    case F_NW:{return (parent->GetF_NEchild());} 
    case F_NE:
    { 
	QTreeNode<Object, Cell>* tmp = parent->getEastNeighborNode(); 
	if(tmp == NULL)
	    return NULL; 
	if(tmp->isLeaf())
	    return tmp; 
	else 
	    return (tmp->GetF_NWchild());
    }
    case B_SE:
    { 
	QTreeNode<Object, Cell>* tmp = parent->getEastNeighborNode(); 
	if(tmp == NULL)
	    return NULL; 
	if(tmp->isLeaf())
	    return tmp; 
	else 
	    return (tmp->GetB_SWchild());
    }
    case B_SW:{return (parent->GetB_SEchild());} 
    case B_NW:{return (parent->GetB_NEchild());} 
    case B_NE:
    { 
	QTreeNode<Object, Cell>* tmp = parent->getEastNeighborNode(); 
	if(tmp == NULL)
	    return NULL; 
	if(tmp->isLeaf())
	    return tmp; 
	else 
	    return (tmp->GetB_NWchild());
    } 
    default: 
	return NULL; 
    } 
}  

template<class Object, class Cell>
void QTreeNode<Object, Cell>::getWestBoundary(std::vector< QTreeNode<Object, Cell>*>& nbrs)
{ 
    if(isLeaf())
	nbrs.push_back(this); 
    else
    { 
	if ( F_NWchild != NULL )
	    F_NWchild->getWestBoundary(nbrs); 
	if ( F_SWchild != NULL )
	    F_SWchild->getWestBoundary(nbrs); 
	if ( B_NWchild != NULL )		 
	    B_NWchild->getWestBoundary(nbrs); 
	if ( B_SWchild != NULL )
	    B_SWchild->getWestBoundary(nbrs); 
    }
} 

//-------- DUE FRONT (from EAST) -------------------- 

/*! Comment is the similar for the SOUTH part above. */
template<class Object, class Cell>
void QTreeNode<Object, Cell>::getAllFrontNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs)
{ 
    QTreeNode<Object, Cell>* tmpNode = getFrontNeighborNode(); 
    if(tmpNode != NULL) 
	tmpNode->getBackBoundary(nbrs); 
} 

/*! This is for the child to call. */
template <class Object, class Cell>
QTreeNode<Object, Cell>* QTreeNode<Object, Cell>::getFrontNeighborNode()
{
    if(parent == NULL)
	return NULL; 
    switch(type)
    { 
    case F_SE:
    { 
	QTreeNode<Object, Cell>* tmp = parent->getFrontNeighborNode(); 
	if(tmp == NULL)
	    return NULL; 
	if(tmp->isLeaf()) 
	    return tmp; 
	else 
	    return (tmp->GetB_SEchild());
    }
    case F_SW:
    { 
	QTreeNode<Object, Cell>* tmp = parent->getFrontNeighborNode(); 
	if(tmp == NULL)
	    return NULL; 
	if(tmp->isLeaf()) 
	    return tmp; 
	else 
	    return (tmp->GetB_SWchild());
    }
    case F_NW:
    { 
	QTreeNode<Object, Cell>* tmp = parent->getFrontNeighborNode(); 
	if(tmp == NULL)
	    return NULL; 
	if(tmp->isLeaf())
	    return tmp; 
	else 
	    return (tmp->GetB_NWchild());
    } 
    case F_NE:
    { 
	QTreeNode<Object, Cell>* tmp = parent->getFrontNeighborNode(); 
	if(tmp == NULL)
	    return NULL; 
	if(tmp->isLeaf()) 
	    return tmp; 
	else 
	    return (tmp->GetB_NEchild());
    }		 
    case B_SE:{return (parent->GetF_SEchild());} 
    case B_SW:{return (parent->GetF_SWchild());} 
    case B_NW:{return (parent->GetF_NWchild());} 
    case B_NE:{return (parent->GetF_NEchild());} 
    default: 
	return NULL; 
    } 
}  

template<class Object, class Cell>
void QTreeNode<Object, Cell>::getBackBoundary(std::vector< QTreeNode<Object, Cell>*>& nbrs)
{ 
    if(isLeaf())
	nbrs.push_back(this); 
    else
    { 
	if ( B_NWchild != NULL )
	    B_NWchild->getBackBoundary(nbrs); 
	if ( B_SWchild != NULL )
	    B_SWchild->getBackBoundary(nbrs); 
	if ( B_NEchild != NULL )		 
	    B_NEchild->getBackBoundary(nbrs); 
	if ( B_SEchild != NULL )
	    B_SEchild->getBackBoundary(nbrs); 
    } 
} 
 
//-------- DUE BACK -------------------- 

/*! Comment is the similar for the SOUTH part above. */
template<class Object, class Cell>
void QTreeNode<Object, Cell>::getAllBackNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs)
{ 
    QTreeNode<Object, Cell>* tmpNode = getBackNeighborNode(); 
    if(tmpNode != NULL) 
	tmpNode->getFrontBoundary(nbrs); 
} 

/*! This is for the child to call. */ 
template<class Object, class Cell>
QTreeNode<Object, Cell>* QTreeNode<Object, Cell>::getBackNeighborNode()
{
    if(parent == NULL)
	return NULL; 
    switch(type)
    { 
    case B_SE:
    { 
	QTreeNode<Object, Cell>* tmp = parent->getBackNeighborNode(); 
	if(tmp == NULL)
	    return NULL; 
	if(tmp->isLeaf())
	    return tmp; 
	else 
	    return (tmp->GetF_SEchild());
    }
    case B_SW:
    { 
	QTreeNode<Object, Cell>* tmp = parent->getBackNeighborNode(); 
	if(tmp == NULL)
	    return NULL; 
	if(tmp->isLeaf())
	    return tmp; 
	else 
	    return (tmp->GetF_SWchild());
    }
    case B_NW:
    { 
	QTreeNode<Object, Cell>* tmp = parent->getBackNeighborNode(); 
	if(tmp == NULL)
	    return NULL; 
	if(tmp->isLeaf())
	    return tmp; 
	else 
	    return (tmp->GetF_NWchild());
    }
    case B_NE:
    { 
	QTreeNode<Object, Cell>* tmp = parent->getBackNeighborNode(); 
	if(tmp == NULL)
	    return NULL; 
	if(tmp->isLeaf())
	    return tmp; 
	else 
	    return (tmp->GetF_NEchild());
    }		 
    case F_SE:{return (parent->GetB_SEchild());} 
    case F_SW:{return (parent->GetB_SWchild());} 
    case F_NW:{return (parent->GetB_NWchild());} 
    case F_NE:{return (parent->GetB_NEchild());} 
    default: 
	return NULL; 
    } 
}  
 
template<class Object, class Cell>
void QTreeNode<Object, Cell>::getFrontBoundary(std::vector< QTreeNode<Object, Cell>*>& nbrs)
{ 
    if(isLeaf())
	nbrs.push_back(this); 
    else
    { 
	if ( F_NWchild != NULL )
	    F_NWchild->getFrontBoundary(nbrs); 
	if ( F_SWchild != NULL )
	    F_SWchild->getFrontBoundary(nbrs); 
	if ( F_NEchild != NULL )	 
	    F_NEchild->getFrontBoundary(nbrs); 
	if ( F_SEchild != NULL )
	    F_SEchild->getFrontBoundary(nbrs); 
    } 
} 
 

/************************************************ 
 *             CORNER-LINE PARTS                * 
 ************************************************/ 
/*! The line means inner line, these routine will  
  get the nodes , say, on the NW edge of the current node. */
template <class Object, class Cell>
void QTreeNode<Object, Cell>::getNWLine(std::vector< QTreeNode<Object, Cell>*>& line)
{ 
    if(isLeaf()) 
	line.push_back(this); 
    else
    { 
	if ( F_NWchild!=NULL )
	    F_NWchild->getNWLine(line);
	if (B_NWchild!=NULL )
	    B_NWchild->getNWLine(line); 
    } 
} 

/*! The line means inner line, these routine will  
  get the nodes , say, on the NE edge of the current node. */
template <class Object, class Cell>
void QTreeNode<Object, Cell>::getNELine(std::vector< QTreeNode<Object, Cell>*>& line)
{ 
    if (isLeaf())
	line.push_back(this); 
    else
    {
	if ( F_NEchild!=NULL )
	    F_NEchild->getNELine(line);
	if (  B_NEchild!=NULL )
	    B_NEchild->getNELine(line); 
    } 
} 

/*! The line means inner line, these routine will  
  get the nodes , say, on the SW edge of the current node. */
template <class Object, class Cell>
void QTreeNode<Object, Cell>::getSWLine(std::vector< QTreeNode<Object, Cell>*>& line)
{ 
    if(isLeaf())
	line.push_back(this); 
    else
    { 
	if (F_SWchild!=NULL )
	    F_SWchild->getSWLine(line); 
	if (B_SWchild!=NULL )
	    B_SWchild->getSWLine(line); 
    } 
} 

/*! The line means inner line, these routine will  
  get the nodes , say, on the SE edge of the current node. */
template <class Object, class Cell>
void QTreeNode<Object, Cell>::getSELine(std::vector< QTreeNode<Object, Cell>*>& line)
{ 
    if(isLeaf())
	line.push_back(this); 
    else
    { 
	if (F_SEchild!=NULL )
	    F_SEchild->getSELine(line); 
	if (B_SEchild!=NULL )
	    B_SEchild->getSELine(line); 
    } 
} 


template <class Object, class Cell>
void QTreeNode<Object, Cell>::getF_NLine(std::vector< QTreeNode<Object, Cell>*>& line)
{ 
    if(isLeaf())
	line.push_back(this); 
    else
    {
	if (F_NEchild!=NULL )
	    F_NEchild->getF_NLine(line); 
	if (F_NWchild!=NULL )
	    F_NWchild->getF_NLine(line); 
    } 
} 


template <class Object, class Cell>
void QTreeNode<Object, Cell>::getF_SLine(std::vector< QTreeNode<Object, Cell>*>& line)
{ 
    if(isLeaf()) 
	line.push_back(this); 
    else
    { 
	if (F_SEchild!=NULL )
	    F_SEchild->getF_SLine(line); 
	if (F_SWchild!=NULL )
	    F_SWchild->getF_SLine(line); 
    } 
}


template <class Object, class Cell>
void QTreeNode<Object, Cell>::getF_ELine(std::vector< QTreeNode<Object, Cell>*>& line)
{ 
    if(isLeaf()) 
	line.push_back(this); 
    else
    {
	if (F_NEchild!=NULL )
	    F_NEchild->getF_ELine(line); 
	if (F_SEchild!=NULL )
	    F_SEchild->getF_ELine(line); 
    } 
} 


template <class Object, class Cell>
void QTreeNode<Object, Cell>::getF_WLine(std::vector< QTreeNode<Object, Cell>*>& line)
{ 
    if(isLeaf())
	line.push_back(this); 
    else
    { 
	if (F_NWchild!=NULL )
	    F_NWchild->getF_WLine(line); 
	if (F_SWchild!=NULL )
	    F_SWchild->getF_WLine(line); 
    } 
} 


template <class Object, class Cell> 
void QTreeNode<Object, Cell>::getB_NLine(std::vector< QTreeNode<Object, Cell>*>& line)
{ 
    if(isLeaf()) 
	line.push_back(this); 
    else
    { 
	if (B_NEchild!=NULL )
	    B_NEchild->getB_NLine(line); 
	if (B_NWchild!=NULL )
	    B_NWchild->getB_NLine(line); 
    } 
} 


template <class Object, class Cell>
void QTreeNode<Object, Cell>::getB_SLine(std::vector< QTreeNode<Object, Cell>*>& line)
{ 
    if(isLeaf()) 
	line.push_back(this); 
    else
    { 
	if (B_SEchild!=NULL )
	    B_SEchild->getB_SLine(line); 
	if (B_SWchild!=NULL )
	    B_SWchild->getB_SLine(line); 
    }
} 


template <class Object, class Cell>
void QTreeNode<Object, Cell>::getB_ELine(std::vector< QTreeNode<Object, Cell>*>& line)
{ 
    if(isLeaf())
	line.push_back(this); 
    else
    { 
	if (B_NEchild!=NULL )
	    B_NEchild->getB_ELine(line); 
	if (B_SEchild!=NULL )
	    B_SEchild->getB_ELine(line); 
    } 
} 


template <class Object, class Cell>
void QTreeNode<Object, Cell>::getB_WLine(std::vector< QTreeNode<Object, Cell>*>& line)
{ 
    if(isLeaf()) 
	line.push_back(this); 
    else
    { 
	if (B_NWchild!=NULL )
	    B_NWchild->getB_WLine(line); 
	if (B_SWchild!=NULL )
	    B_SWchild->getB_WLine(line); 
    } 
} 
 

template <class Object, class Cell>
void QTreeNode<Object, Cell>::getNWNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs)
{ 
    //get the west and the north node (equivalent or greater) 
    QTreeNode<Object, Cell>* WestNode = getWestNeighborNode();
    if(WestNode == NULL)
	return; 
    QTreeNode<Object, Cell>* NorthNode = getNorthNeighborNode();
    if(NorthNode == NULL) 
	return; 
  
    //get the north of the west 
    QTreeNode<Object, Cell>* NorthOfWestNode = WestNode->getNorthNeighborNode(); 
    if(NorthOfWestNode == NULL)
	return; 
    //get the west of the north 
    QTreeNode<Object, Cell>* WestOfNorthNode = NorthNode->getWestNeighborNode();
    if(WestOfNorthNode == NULL)
	return; 
  
    //get the south east edges of both nodes 
    std::vector< QTreeNode<Object, Cell>*> SELine1; 
    NorthOfWestNode->getSELine(SELine1);
    if(SELine1.size() == 0)
	return; 
    std::vector< QTreeNode<Object, Cell>*> SELine2; 
    WestOfNorthNode->getSELine(SELine2);
    if(SELine2.size() == 0)
	return; 
  
    for(int i = 0; i < SELine1.size(); i++) 
	for(int j =0; j < SELine2.size(); j++)
	    if(SELine1[i] == SELine2[j])
		nbrs.push_back(SELine1[i]); 
} 


template <class Object, class Cell>
void QTreeNode<Object, Cell>::getNENeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs)
{ 
    //get the East and the north node (equivalent or greater) 
    QTreeNode<Object, Cell>* EastNode = getEastNeighborNode();
    if(EastNode == NULL)
	return; 
    QTreeNode<Object, Cell>* NorthNode = getNorthNeighborNode();
    if(NorthNode == NULL)
	return; 
 
    //get the north of the East 
    QTreeNode<Object, Cell>* NorthOfEastNode = EastNode->getNorthNeighborNode();
    if(NorthOfEastNode == NULL)
	return; 
    //get the East of thre north 
    QTreeNode<Object, Cell>* EastOfNorthNode = NorthNode->getEastNeighborNode();
    if(EastOfNorthNode == NULL) 
	return; 
	 
    //get the south east edges of both nodes 
    std::vector< QTreeNode<Object, Cell>*> SWLine1; 
    NorthOfEastNode->getSWLine(SWLine1);
    if(SWLine1.size() == 0)
	return; 
    std::vector< QTreeNode<Object, Cell>*> SWLine2; 
    EastOfNorthNode->getSWLine(SWLine2);
    if(SWLine2.size() == 0)
	return; 
	 
    for(int i = 0; i < SWLine1.size(); i++)
	for(int j =0; j < SWLine2.size(); j++)
	    if(SWLine1[i] == SWLine2[j])
		nbrs.push_back(SWLine1[i]);
} 


template <class Object, class Cell>
void QTreeNode<Object, Cell>::getSWNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs)
{ 
    //get the west and the South node (equivalent or greater) 
    QTreeNode<Object, Cell>* WestNode = getWestNeighborNode();
    if(WestNode == NULL)
	return; 
    QTreeNode<Object, Cell>* SouthNode = getSouthNeighborNode();
    if(SouthNode == NULL)
	return; 
    //get the South of the west 
    QTreeNode<Object, Cell>* SouthOfWestNode = WestNode->getSouthNeighborNode();
    if(SouthOfWestNode == NULL) 
	return; 
    //get the west of the South 
    QTreeNode<Object, Cell>* WestOfSouthNode = SouthNode->getWestNeighborNode();
    if(WestOfSouthNode == NULL)
	return; 
  
    //get the south east edges of both nodes 
    std::vector< QTreeNode<Object, Cell>*> NELine1; 
    SouthOfWestNode->getNELine(NELine1); 
    if(NELine1.empty())
	return;
    if(NELine1.size() == 0)
	return; 
   
    std::vector< QTreeNode<Object, Cell>*> NELine2;
    WestOfSouthNode->getNELine(NELine2);

    if(NELine2.empty())
	return;
    if(NELine2.size() == 0)
	return; 
   
    for(int i = 0; i < NELine1.size(); i++)
	for(int j =0; j < NELine2.size(); j++)
	    if(NELine1[i] == NELine2[j])
		nbrs.push_back(NELine1[i]); 
} 


template <class Object, class Cell>
void QTreeNode<Object, Cell>::getSENeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs)
{ 
    //get the East and the South node (equivalent or greater) 
    QTreeNode<Object, Cell>* EastNode = getEastNeighborNode();
    if(EastNode == NULL)
	return; 
    QTreeNode<Object, Cell>* SouthNode = getSouthNeighborNode();
    if(SouthNode == NULL)
	return; 
 
    //get the South of the East 
    QTreeNode<Object, Cell>* SouthOfEastNode = EastNode->getSouthNeighborNode(); 
    if(SouthOfEastNode == NULL)
	return; 
    //get the East of thre South 
    QTreeNode<Object, Cell>* EastOfSouthNode = SouthNode->getEastNeighborNode();
    if(EastOfSouthNode == NULL)
	return; 
	 
    //get the south east edges of both nodes 
    std::vector< QTreeNode<Object, Cell>*> NWLine1; 
    SouthOfEastNode->getNWLine(NWLine1);
    if(NWLine1.size() == 0)
	return; 
    std::vector< QTreeNode<Object, Cell>*> NWLine2; 
    EastOfSouthNode->getNWLine(NWLine2);
    if(NWLine2.size() == 0)
	return; 
	 
    for(int i = 0; i < NWLine1.size(); i++)
	for(int j =0; j < NWLine2.size(); j++)
	    if(NWLine1[i] == NWLine2[j])
		nbrs.push_back(NWLine1[i]); 
}


template <class Object, class Cell>
void QTreeNode<Object, Cell>::getF_NNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs)
{ 
    //get the Front and the North node (equivalent or greater) 
    QTreeNode<Object, Cell>* FrontNode = getFrontNeighborNode();
    if(FrontNode == NULL)
	return; 
    QTreeNode<Object, Cell>* NorthNode = getNorthNeighborNode();
    if(NorthNode == NULL)
	return; 
 
    //get the Front of the North 
    QTreeNode<Object, Cell>* FrontOfNorthNode = NorthNode->getFrontNeighborNode();
    if(FrontOfNorthNode == NULL)
	return; 
    //get the North of the Front 
    QTreeNode<Object, Cell>* NorthOfFrontNode = FrontNode->getNorthNeighborNode();
    if(NorthOfFrontNode == NULL)
	return; 
	 
    //get the south east edges of both nodes 
    std::vector< QTreeNode<Object, Cell>*> B_SLine1; 
    FrontOfNorthNode->getB_SLine(B_SLine1);
    if(B_SLine1.size() == 0)
	return; 
    std::vector< QTreeNode<Object, Cell>*> B_SLine2; 
    NorthOfFrontNode->getB_SLine(B_SLine2);
    if(B_SLine2.size() == 0)
	return; 
	 
    for(int i = 0; i < B_SLine1.size(); i++)
	for(int j =0; j < B_SLine2.size(); j++)
	    if(B_SLine1[i] == B_SLine2[j])
		nbrs.push_back(B_SLine1[i]); 
} 


template <class Object, class Cell>
void QTreeNode<Object, Cell>::getF_SNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs)
{ 
    //get the Front and the South node (equivalent or greater) 
    QTreeNode<Object, Cell>* FrontNode = getFrontNeighborNode();
    if(FrontNode == NULL)
	return; 
    QTreeNode<Object, Cell>* SouthNode = getSouthNeighborNode();
    if(SouthNode == NULL)
	return; 
 
    //get the Front of the South 
    QTreeNode<Object, Cell>* FrontOfSouthNode = SouthNode->getFrontNeighborNode();
    if(FrontOfSouthNode == NULL)
	return; 
    //get the South of the Front 
    QTreeNode<Object, Cell>* SouthOfFrontNode = FrontNode->getSouthNeighborNode();
    if(SouthOfFrontNode == NULL)
	return; 
	 
    //get the south east edges of both nodes 
    std::vector< QTreeNode<Object, Cell>*> B_NLine1; 
    FrontOfSouthNode->getB_NLine(B_NLine1);
    if(B_NLine1.size() == 0)
	return; 
    std::vector< QTreeNode<Object, Cell>*> B_NLine2; 
    SouthOfFrontNode->getB_NLine(B_NLine2);
    if(B_NLine2.size() == 0)
	return; 
	 
    for(int i = 0; i < B_NLine1.size(); i++)
	for(int j =0; j < B_NLine2.size(); j++)
	    if(B_NLine1[i] == B_NLine2[j])
		nbrs.push_back(B_NLine1[i]); 
} 


template <class Object, class Cell>
void QTreeNode<Object, Cell>::getF_ENeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs)
{ 
    //get the Front and the East node (equivalent or greater) 
    QTreeNode<Object, Cell>* FrontNode = getFrontNeighborNode(); 
    if(FrontNode == NULL)
	return; 
    QTreeNode<Object, Cell>* EastNode = getEastNeighborNode();
    if(EastNode == NULL)
	return; 
 
    //get the Front of the East 
    QTreeNode<Object, Cell>* FrontOfEastNode = EastNode->getFrontNeighborNode();
    if(FrontOfEastNode == NULL)
	return; 
    //get the East of the Front 
    QTreeNode<Object, Cell>* EastOfFrontNode = FrontNode->getEastNeighborNode(); 
    if(EastOfFrontNode == NULL) 
	return; 
	 
    //get the East east edges of both nodes 
    std::vector< QTreeNode<Object, Cell>*> B_WLine1; 
    FrontOfEastNode->getB_WLine(B_WLine1);
    if(B_WLine1.size() == 0) 
	return; 
    std::vector< QTreeNode<Object, Cell>*> B_WLine2; 
    EastOfFrontNode->getB_WLine(B_WLine2); 
    if(B_WLine2.size() == 0)
	return; 
	 
    for(int i = 0; i < B_WLine1.size(); i++)
	for(int j =0; j < B_WLine2.size(); j++)
	    if(B_WLine1[i] == B_WLine2[j])
		nbrs.push_back(B_WLine1[i]); 
} 


template <class Object, class Cell>
void QTreeNode<Object, Cell>::getF_WNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs)
{ 
    //get the Front and the West node (equivalent or greater) 
    QTreeNode<Object, Cell>* FrontNode = getFrontNeighborNode();
    if(FrontNode == NULL) 
	return; 
    QTreeNode<Object, Cell>* WestNode = getWestNeighborNode();
    if(WestNode == NULL) 
	return; 
 
    //get the Front of the West 
    QTreeNode<Object, Cell>* FrontOfWestNode = WestNode->getFrontNeighborNode();
    if(FrontOfWestNode == NULL)
	return; 
    //get the West of the Front 
    QTreeNode<Object, Cell>* WestOfFrontNode = FrontNode->getWestNeighborNode();
    if(WestOfFrontNode == NULL)
	return; 
	 
    //get the West West edges of both nodes 
    std::vector< QTreeNode<Object, Cell>*> B_ELine1; 
    FrontOfWestNode->getB_ELine(B_ELine1);
    if(B_ELine1.size() == 0)
	return; 
    std::vector< QTreeNode<Object, Cell>*> B_ELine2; 
    WestOfFrontNode->getB_ELine(B_ELine2);
    if(B_ELine2.size() == 0)
	return; 
	 
    for(int i = 0; i < B_ELine1.size(); i++)
	for(int j =0; j < B_ELine2.size(); j++)
	    if(B_ELine1[i] == B_ELine2[j])
		nbrs.push_back(B_ELine1[i]); 
} 


template <class Object, class Cell>
void QTreeNode<Object, Cell>::getB_NNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs)
{ 
    //get the Back and the North node (equivalent or greater) 
    QTreeNode<Object, Cell>* BackNode = getBackNeighborNode();
    if(BackNode == NULL)
	return; 
    QTreeNode<Object, Cell>* NorthNode = getNorthNeighborNode();
    if(NorthNode == NULL)
	return; 
 
    //get the Back of the North 
    QTreeNode<Object, Cell>* BackOfNorthNode = NorthNode->getBackNeighborNode();
    if(BackOfNorthNode == NULL)
	return; 
    //get the North of the Back 
    QTreeNode<Object, Cell>* NorthOfBackNode = BackNode->getNorthNeighborNode(); 
    if(NorthOfBackNode == NULL) 
	return; 
	 
    //get the south east edges of both nodes 
    std::vector< QTreeNode<Object, Cell>*> F_SLine1; 
    BackOfNorthNode->getF_SLine(F_SLine1);
    if(F_SLine1.size() == 0)
	return; 
    std::vector< QTreeNode<Object, Cell>*> F_SLine2; 
    NorthOfBackNode->getF_SLine(F_SLine2);
    if(F_SLine2.size() == 0) 
	return; 
	 
    for(int i = 0; i < F_SLine1.size(); i++)
	for(int j =0; j < F_SLine2.size(); j++) 
	    if(F_SLine1[i] == F_SLine2[j])
		nbrs.push_back(F_SLine1[i]); 
} 


template <class Object, class Cell>
void QTreeNode<Object, Cell>::getB_SNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs)
{ 
    //get the Back and the South node (equivalent or greater) 
    QTreeNode<Object, Cell>* BackNode = getBackNeighborNode();
    if(BackNode == NULL)
	return; 
    QTreeNode<Object, Cell>* SouthNode = getSouthNeighborNode();
    if(SouthNode == NULL)
	return; 
 
    //get the Back of the South 
    QTreeNode<Object, Cell>* BackOfSouthNode = SouthNode->getBackNeighborNode(); 
    if(BackOfSouthNode == NULL)
	return; 
    //get the South of the Back 
    QTreeNode<Object, Cell>* SouthOfBackNode = BackNode->getSouthNeighborNode();
    if(SouthOfBackNode == NULL)
	return; 
	 
    //get the south east edges of both nodes 
    std::vector< QTreeNode<Object, Cell>*> F_NLine1; 
    BackOfSouthNode->getF_NLine(F_NLine1);
    if(F_NLine1.size() == 0)
	return; 
    std::vector< QTreeNode<Object, Cell>*> F_NLine2; 
    SouthOfBackNode->getF_NLine(F_NLine2);
    if(F_NLine2.size() == 0)
	return; 
	 
    for(int i = 0; i < F_NLine1.size(); i++) 
	for(int j =0; j < F_NLine2.size(); j++) 
	    if(F_NLine1[i] == F_NLine2[j])
		nbrs.push_back(F_NLine1[i]); 
} 


template <class Object, class Cell>
void QTreeNode<Object, Cell>::getB_ENeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs)
{ 
    //get the Back and the East node (equivalent or greater) 
    QTreeNode<Object, Cell>* BackNode = getBackNeighborNode();
    if(BackNode == NULL)
	return; 
    QTreeNode<Object, Cell>* EastNode = getEastNeighborNode();
    if(EastNode == NULL)
	return; 
 
    //get the Back of the East 
    QTreeNode<Object, Cell>* BackOfEastNode = EastNode->getBackNeighborNode();
    if(BackOfEastNode == NULL)
	return; 
    //get the East of the Back 
    QTreeNode<Object, Cell>* EastOfBackNode = BackNode->getEastNeighborNode(); 
    if(EastOfBackNode == NULL)
	return; 
	 
    //get the East east edges of both nodes 
    std::vector< QTreeNode<Object, Cell>*> F_WLine1; 
    BackOfEastNode->getF_WLine(F_WLine1);
    if(F_WLine1.size() == 0) 
	return; 
    std::vector< QTreeNode<Object, Cell>*> F_WLine2; 
    EastOfBackNode->getF_WLine(F_WLine2); 
    if(F_WLine2.size() == 0) 
	return; 
	 
    for(int i = 0; i < F_WLine1.size(); i++)
	for(int j =0; j < F_WLine2.size(); j++)
	    if(F_WLine1[i] == F_WLine2[j])
		nbrs.push_back(F_WLine1[i]); 
} 


template <class Object, class Cell>
void QTreeNode<Object, Cell>::getB_WNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs)
{ 
    //get the Back and the West node (equivalent or greater) 
    QTreeNode<Object, Cell>* BackNode = getBackNeighborNode();
    if(BackNode == NULL)
	return; 
    QTreeNode<Object, Cell>* WestNode = getWestNeighborNode();
    if(WestNode == NULL)
	return; 
 
    //get the Back of the West 
    QTreeNode<Object, Cell>* BackOfWestNode = WestNode->getBackNeighborNode();
    if(BackOfWestNode == NULL)
	return; 
    //get the West of the Back 
    QTreeNode<Object, Cell>* WestOfBackNode = BackNode->getWestNeighborNode();
    if(WestOfBackNode == NULL) 
	return; 
	 
    //get the West east edges of both nodes 
    std::vector< QTreeNode<Object, Cell>*> F_ELine1; 
    BackOfWestNode->getF_ELine(F_ELine1);
    if(F_ELine1.size() == 0) 
	return; 
    std::vector< QTreeNode<Object, Cell>*> F_ELine2; 
    WestOfBackNode->getF_ELine(F_ELine2); 
    if(F_ELine2.size() == 0) 
	return; 
	 
    for(int i = 0; i < F_ELine1.size(); i++)
	for(int j =0; j < F_ELine2.size(); j++)
	    if(F_ELine1[i] == F_ELine2[j])
		nbrs.push_back(F_ELine1[i]); 
} 
 
/*!********************************************** 
 *             TRUE CORNER PARTS                * 
 ************************************************/ 
/*! This part is to get the 8 boxes each of them  
  share only one vertex with the current node. */
 
/*! Get the most Front NW child of the current node
  - corner means the corner inside current node. */
template <class Object, class Cell>
QTreeNode<Object, Cell>* QTreeNode<Object, Cell>::getF_NWCorner()
{//return the node on the Front NW corner 
    if(isLeaf())
	return this; 
    else
	return F_NWchild->getF_NWCorner(); 
} 

/*! Get the most NE child of the current node. */
template <class Object, class Cell>
QTreeNode<Object, Cell>* QTreeNode<Object, Cell>::getF_NECorner()
{ 
    if(isLeaf())
	return this; 
    else
	return F_NEchild->getF_NECorner(); 
} 

/*! Get the most SW child of the current node. */ 
template <class Object, class Cell>
QTreeNode<Object, Cell>* QTreeNode<Object, Cell>::getF_SWCorner()
{ 
    if(isLeaf())
	return this; 
    else
	return F_SWchild->getF_SWCorner(); 
}
 
/*! Get the most SE child of the current node. */
template <class Object, class Cell>
QTreeNode<Object, Cell>* QTreeNode<Object, Cell>::getF_SECorner()
{ 
    if(isLeaf())
	return this; 
    else
	return F_SEchild->getF_SECorner(); 
} 
 
/*! get the most Front NW child of the current node
  - corner means the corner inside current node. */
template <class Object, class Cell>
QTreeNode<Object, Cell>* QTreeNode<Object, Cell>::getB_NWCorner()
{//return the node on the Front NW corner 
    if(isLeaf()) 
	return this; 
    else 
	return B_NWchild->getB_NWCorner(); 
}
 
/*! Get the most NE child of the current node. */
template <class Object, class Cell>
QTreeNode<Object, Cell>* QTreeNode<Object, Cell>::getB_NECorner()
{ 
    if(isLeaf()) 
	return this; 
    else
	return B_NEchild->getB_NECorner(); 
}

/*! Get the most SW child of the current node. */
template <class Object, class Cell>
QTreeNode<Object, Cell>* QTreeNode<Object, Cell>::getB_SWCorner()
{ 
    if(isLeaf())
	return this; 
    else
	return B_SWchild->getB_SWCorner(); 
} 

/*! Get the most SE child of the current node.*/
template <class Object, class Cell>
QTreeNode<Object, Cell>* QTreeNode<Object, Cell>::getB_SECorner()
{ 
    if(isLeaf())
	return this; 
    else
	return B_SEchild->getB_SECorner(); 
} 
 
// There are two possibilities :

/*!**** FRONT **********/ 
//------ F_NE Corner------------ 

template <class Object, class Cell>
QTreeNode<Object, Cell>* QTreeNode<Object, Cell>::F_NENeighbor()
{ 
    QTreeNode<Object, Cell>* NorthNode = getNorthNeighborNode();
    if(NorthNode == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* EastNode = getEastNeighborNode(); 
    if(EastNode == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* FrontNode = getFrontNeighborNode();
    if(FrontNode == NULL) 
	return NULL; 
    //get the north of front, east of north, and front of east 
    QTreeNode<Object, Cell>* NorthOfFront = FrontNode->getNorthNeighborNode();
    if(NorthOfFront == NULL) 
	return NULL; 
    QTreeNode<Object, Cell>* EastOfNorth = NorthNode->getEastNeighborNode(); 
    if(EastOfNorth == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* FrontOfEast = EastNode->getFrontNeighborNode(); 
    if(FrontOfEast == NULL) 
	return NULL; 
    //get the corner box 
    QTreeNode<Object, Cell>* EastOfNorthOfFront = NorthOfFront->getEastNeighborNode();
    if(EastOfNorthOfFront == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* FrontOfEastOfNorth = EastOfNorth->getFrontNeighborNode();
    if(FrontOfEastOfNorth == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* NorthOfFrontOfEast = FrontOfEast->getNorthNeighborNode();
    if(NorthOfFrontOfEast == NULL) 
	return NULL; 
    //get the corner of the corner 
    QTreeNode<Object, Cell>* Cor1 = EastOfNorthOfFront->getB_SWCorner();
    if(Cor1 == NULL) 
	return NULL; 
    QTreeNode<Object, Cell>* Cor2 = FrontOfEastOfNorth->getB_SWCorner();
    if(Cor2 == NULL) 
	return NULL; 
    QTreeNode<Object, Cell>* Cor3 = NorthOfFrontOfEast->getB_SWCorner();
    if(Cor3 == NULL)
	return NULL; 

    if((Cor1 == Cor2) && (Cor2 == Cor3))
	return Cor1; 
    else
	return NULL; 
} 
 
//------ F_NW Corner------------ 

template <class Object, class Cell>
QTreeNode<Object, Cell>* QTreeNode<Object, Cell>::F_NWNeighbor()
{ 
    QTreeNode<Object, Cell>* NorthNode = getNorthNeighborNode();
    if(NorthNode == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* WestNode = getWestNeighborNode();
    if(WestNode == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* FrontNode = getFrontNeighborNode();
    if(FrontNode == NULL)
	return NULL; 
    //get the north of front, West of north, and front of West 
    QTreeNode<Object, Cell>* NorthOfFront = FrontNode->getNorthNeighborNode();
    if(NorthOfFront == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* WestOfNorth = NorthNode->getWestNeighborNode();
    if(WestOfNorth == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* FrontOfWest = WestNode->getFrontNeighborNode();
    if(FrontOfWest == NULL) 
	return NULL; 
    //get the corner box 
    QTreeNode<Object, Cell>* WestOfNorthOfFront = NorthOfFront->getWestNeighborNode();
    if(WestOfNorthOfFront == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* FrontOfWestOfNorth = WestOfNorth->getFrontNeighborNode();
    if(FrontOfWestOfNorth == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* NorthOfFrontOfWest = FrontOfWest->getNorthNeighborNode();
    if(NorthOfFrontOfWest == NULL)
	return NULL; 
    //get the corner of the corner 
    QTreeNode<Object, Cell>* Cor1 = WestOfNorthOfFront->getB_SECorner();
    if(Cor1 == NULL) 
	return NULL; 
    QTreeNode<Object, Cell>* Cor2 = FrontOfWestOfNorth->getB_SECorner();
    if(Cor2 == NULL) 
	return NULL; 
    QTreeNode<Object, Cell>* Cor3 = NorthOfFrontOfWest->getB_SECorner();
    if(Cor3 == NULL)
	return NULL; 

    if((Cor1 == Cor2) && (Cor2 == Cor3))
	return Cor1; 
    else 
	return NULL; 
} 
 
//------ F_SW Corner------------ 

template <class Object, class Cell>
QTreeNode<Object, Cell>* QTreeNode<Object, Cell>::F_SWNeighbor()
{ 
    QTreeNode<Object, Cell>* SouthNode = getSouthNeighborNode();
    if(SouthNode == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* WestNode = getWestNeighborNode();
    if(WestNode == NULL) 
	return NULL; 
    QTreeNode<Object, Cell>* FrontNode = getFrontNeighborNode();
    if(FrontNode == NULL) 
	return NULL; 
    //get the South of front, West of South, and front of West 
    QTreeNode<Object, Cell>* SouthOfFront = FrontNode->getSouthNeighborNode(); 
    if(SouthOfFront == NULL) 
	return NULL; 
    QTreeNode<Object, Cell>* WestOfSouth = SouthNode->getWestNeighborNode(); 
    if(WestOfSouth == NULL) 
	return NULL; 
    QTreeNode<Object, Cell>* FrontOfWest = WestNode->getFrontNeighborNode();
    if(FrontOfWest == NULL) 
	return NULL; 
    //get the corner box 
    QTreeNode<Object, Cell>* WestOfSouthOfFront = SouthOfFront->getWestNeighborNode();
    if(WestOfSouthOfFront == NULL) 
	return NULL; 
    QTreeNode<Object, Cell>* FrontOfWestOfSouth = WestOfSouth->getFrontNeighborNode(); 
    if(FrontOfWestOfSouth == NULL) 
	return NULL; 
    QTreeNode<Object, Cell>* SouthOfFrontOfWest = FrontOfWest->getSouthNeighborNode();
    if(SouthOfFrontOfWest == NULL) 
	return NULL; 
    //get the corner of the corner 
    QTreeNode<Object, Cell>* Cor1 = WestOfSouthOfFront->getB_NECorner();
    if(Cor1 == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* Cor2 = FrontOfWestOfSouth->getB_NECorner();
    if(Cor2 == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* Cor3 = SouthOfFrontOfWest->getB_NECorner();
    if(Cor3 == NULL) 
	return NULL; 
  
    if((Cor1 == Cor2) && (Cor2 == Cor3))
	return Cor1; 
    else
	return NULL; 
} 
 
//------ F_SE Corner------------ 

template <class Object, class Cell>
QTreeNode<Object, Cell>* QTreeNode<Object, Cell>::F_SENeighbor()
{ 
    QTreeNode<Object, Cell>* SouthNode = getSouthNeighborNode();
    if(SouthNode == NULL) 
	return NULL; 
    QTreeNode<Object, Cell>* EastNode = getEastNeighborNode(); 
    if(EastNode == NULL) 
	return NULL; 
    QTreeNode<Object, Cell>* FrontNode = getFrontNeighborNode(); 
    if(FrontNode == NULL)
	return NULL; 
    //get the South of front, East of South, and front of East 
    QTreeNode<Object, Cell>* SouthOfFront = FrontNode->getSouthNeighborNode();
    if(SouthOfFront == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* EastOfSouth = SouthNode->getEastNeighborNode(); 
    if(EastOfSouth == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* FrontOfEast = EastNode->getFrontNeighborNode(); 
    if(FrontOfEast == NULL)
	return NULL; 
    //get the corner box 
    QTreeNode<Object, Cell>* EastOfSouthOfFront = SouthOfFront->getEastNeighborNode();
    if(EastOfSouthOfFront == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* FrontOfEastOfSouth = EastOfSouth->getFrontNeighborNode();
    if(FrontOfEastOfSouth == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* SouthOfFrontOfEast = FrontOfEast->getSouthNeighborNode(); 
    if(SouthOfFrontOfEast == NULL) 
	return NULL; 
    //get the corner of the corner 
    QTreeNode<Object, Cell>* Cor1 = EastOfSouthOfFront->getB_NWCorner();
    if(Cor1 == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* Cor2 = FrontOfEastOfSouth->getB_NWCorner();
    if(Cor2 == NULL) 
	return NULL; 
    QTreeNode<Object, Cell>* Cor3 = SouthOfFrontOfEast->getB_NWCorner();
    if(Cor3 == NULL)
	return NULL; 
   
    if((Cor1 == Cor2) && (Cor2 == Cor3))
	return Cor1; 
    else 
	return NULL; 
} 
 
/*!**** BACK **********/ 
//------ B_NE Corner------------ 

template <class Object, class Cell>
QTreeNode<Object, Cell>* QTreeNode<Object, Cell>::B_NENeighbor()
{ 
    QTreeNode<Object, Cell>* NorthNode = getNorthNeighborNode(); 
    if(NorthNode == NULL) 
	return NULL; 
    QTreeNode<Object, Cell>* EastNode = getEastNeighborNode();
    if(EastNode == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* BackNode = getBackNeighborNode();
    if(BackNode == NULL) 
	return NULL; 
    //get the north of Back, east of north, and Back of east 
    QTreeNode<Object, Cell>* NorthOfBack = BackNode->getNorthNeighborNode();
    if(NorthOfBack == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* EastOfNorth = NorthNode->getEastNeighborNode();
    if(EastOfNorth == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* BackOfEast = EastNode->getBackNeighborNode(); 
    if(BackOfEast == NULL) 
	return NULL; 
    //get the corner box 
    QTreeNode<Object, Cell>* EastOfNorthOfBack = NorthOfBack->getEastNeighborNode();
    if(EastOfNorthOfBack == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* BackOfEastOfNorth = EastOfNorth->getBackNeighborNode();
    if(BackOfEastOfNorth == NULL) 
	return NULL; 
    QTreeNode<Object, Cell>* NorthOfBackOfEast = BackOfEast->getNorthNeighborNode();
    if(NorthOfBackOfEast == NULL) 
	return NULL; 
    //get the corner of the corner 
    QTreeNode<Object, Cell>* Cor1 = EastOfNorthOfBack->getF_SWCorner();
    if(Cor1 == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* Cor2 = BackOfEastOfNorth->getF_SWCorner();
    if(Cor2 == NULL) 
	return NULL; 
    QTreeNode<Object, Cell>* Cor3 = NorthOfBackOfEast->getF_SWCorner();
    if(Cor3 == NULL) 
	return NULL; 
  
    if((Cor1 == Cor2) && (Cor2 == Cor3))
	return Cor1; 
    else
	return NULL; 
} 
 
//------ B_NW Corner------------ 

template <class Object, class Cell>
QTreeNode<Object, Cell>* QTreeNode<Object, Cell>::B_NWNeighbor()
{ 
    QTreeNode<Object, Cell>* NorthNode = getNorthNeighborNode(); 
    if(NorthNode == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* WestNode = getWestNeighborNode();
    if(WestNode == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* BackNode = getBackNeighborNode();
    if(BackNode == NULL)
	return NULL; 
    //get the north of Back, West of north, and Back of West 
    QTreeNode<Object, Cell>* NorthOfBack = BackNode->getNorthNeighborNode();
    if(NorthOfBack == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* WestOfNorth = NorthNode->getWestNeighborNode();
    if(WestOfNorth == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* BackOfWest = WestNode->getBackNeighborNode(); 
    if(BackOfWest == NULL) 
	return NULL; 
    //get the corner box 
    QTreeNode<Object, Cell>* WestOfNorthOfBack = NorthOfBack->getWestNeighborNode();
    if(WestOfNorthOfBack == NULL) 
	return NULL; 
    QTreeNode<Object, Cell>* BackOfWestOfNorth = WestOfNorth->getBackNeighborNode();
    if(BackOfWestOfNorth == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* NorthOfBackOfWest = BackOfWest->getNorthNeighborNode(); 
    if(NorthOfBackOfWest == NULL) 
	return NULL; 
    //get the corner of the corner 
    QTreeNode<Object, Cell>* Cor1 = WestOfNorthOfBack->getF_SECorner();
    if(Cor1 == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* Cor2 = BackOfWestOfNorth->getF_SECorner();
    if(Cor2 == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* Cor3 = NorthOfBackOfWest->getF_SECorner();
    if(Cor3 == NULL)
	return NULL; 
  
    if((Cor1 == Cor2) && (Cor2 == Cor3))
	return Cor1; 
    else 
	return NULL; 
} 

//------ B_SW Corner------------ 

template <class Object, class Cell>
QTreeNode<Object, Cell>* QTreeNode<Object, Cell>::B_SWNeighbor()
{ 
    QTreeNode<Object, Cell>* SouthNode = getSouthNeighborNode();
    if(SouthNode == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* WestNode = getWestNeighborNode();
    if(WestNode == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* BackNode = getBackNeighborNode(); 
    if(BackNode == NULL)
	return NULL; 
    //get the South of Back, West of South, and Back of West 
    QTreeNode<Object, Cell>* SouthOfBack = BackNode->getSouthNeighborNode();
    if(SouthOfBack == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* WestOfSouth = SouthNode->getWestNeighborNode();
    if(WestOfSouth == NULL) 
	return NULL; 
    QTreeNode<Object, Cell>* BackOfWest = WestNode->getBackNeighborNode();
    if(BackOfWest == NULL)
	return NULL; 
    //get the corner box 
    QTreeNode<Object, Cell>* WestOfSouthOfBack = SouthOfBack->getWestNeighborNode();
    if(WestOfSouthOfBack == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* BackOfWestOfSouth = WestOfSouth->getBackNeighborNode();
    if(BackOfWestOfSouth == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* SouthOfBackOfWest = BackOfWest->getSouthNeighborNode(); 
    if(SouthOfBackOfWest == NULL)
	return NULL; 
    //get the corner of the corner 
    QTreeNode<Object, Cell>* Cor1 = WestOfSouthOfBack->getF_NECorner();
    if(Cor1 == NULL) 
	return NULL; 
    QTreeNode<Object, Cell>* Cor2 = BackOfWestOfSouth->getF_NECorner();
    if(Cor2 == NULL) 
	return NULL; 
    QTreeNode<Object, Cell>* Cor3 = SouthOfBackOfWest->getF_NECorner();
    if(Cor3 == NULL) 
	return NULL; 
  
    if((Cor1 == Cor2) && (Cor2 == Cor3))
	return Cor1; 
    else 
	return NULL; 
} 
 
//------ B_SE Corner------------ 

template <class Object, class Cell>
QTreeNode<Object, Cell>* QTreeNode<Object, Cell>::B_SENeighbor()
{ 
    QTreeNode<Object, Cell>* SouthNode = getSouthNeighborNode();
    if(SouthNode == NULL) 
	return NULL; 
    QTreeNode<Object, Cell>* EastNode = getEastNeighborNode();
    if(EastNode == NULL) 
	return NULL; 
    QTreeNode<Object, Cell>* BackNode = getBackNeighborNode(); 
    if(BackNode == NULL)
	return NULL; 
    //get the South of Back, East of South, and Back of East 
    QTreeNode<Object, Cell>* SouthOfBack = BackNode->getSouthNeighborNode();
    if(SouthOfBack == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* EastOfSouth = SouthNode->getEastNeighborNode();
    if(EastOfSouth == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* BackOfEast = EastNode->getBackNeighborNode();
    if(BackOfEast == NULL)
	return NULL; 
    //get the corner box 
    QTreeNode<Object, Cell>* EastOfSouthOfBack = SouthOfBack->getEastNeighborNode();
    if(EastOfSouthOfBack == NULL) 
	return NULL; 
    QTreeNode<Object, Cell>* BackOfEastOfSouth = EastOfSouth->getBackNeighborNode();
    if(BackOfEastOfSouth == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* SouthOfBackOfEast = BackOfEast->getSouthNeighborNode();
    if(SouthOfBackOfEast == NULL) 
	return NULL; 
    //get the corner of the corner 
    QTreeNode<Object, Cell>* Cor1 = EastOfSouthOfBack->getF_NWCorner();
    if(Cor1 == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* Cor2 = BackOfEastOfSouth->getF_NWCorner();
    if(Cor2 == NULL)
	return NULL; 
    QTreeNode<Object, Cell>* Cor3 = SouthOfBackOfEast->getF_NWCorner();
    if(Cor3 == NULL)
	return NULL; 
  
    if((Cor1 == Cor2) && (Cor2 == Cor3))
	return Cor1; 
    else
	return NULL; 
} 
 
/************************************************** 
 *        THE TOP FUNCTION in Neighbor finding    * 
 **************************************************/ 

/*! 
  PURPOSES: get all the neighbor of this node; 
  RECEIVES:  
  RETURNS:  the list of neighbor store in the vector nbrs (vector of pointers)	 
  REMARKS: 
  USING THIS FUNCTION to get all the neighbor of  
  the current node, including S, W, N , E neighbor  
  as well as NW, SE, SW,... (exterior corners) 
*/ 
template <class Object, class Cell>
void QTreeNode<Object, Cell>::getAllNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs)
{ 
    // get the neighbors on the four due directions 
    getAllSouthNeighbors(nbrs); 
    getAllNorthNeighbors(nbrs); 
    getAllWestNeighbors(nbrs); 
    getAllEastNeighbors(nbrs); 
    getAllFrontNeighbors(nbrs); 
    getAllBackNeighbors(nbrs); 

    //get the nbr on the edges 
    getNWNeighbors(nbrs); 
    getNENeighbors(nbrs); 
    getSWNeighbors(nbrs); 
    getSENeighbors(nbrs); 
    getF_NNeighbors(nbrs); 
    getF_SNeighbors(nbrs); 
    getF_ENeighbors(nbrs); 
    getF_WNeighbors(nbrs); 
    getB_NNeighbors(nbrs); 
    getB_SNeighbors(nbrs); 
    getB_ENeighbors(nbrs); 
    getB_WNeighbors(nbrs); 
	 
    //get the neighbor on the corner 
    QTreeNode<Object, Cell>* F_NE = F_NENeighbor();//------ NE Corner------------ 
    if(F_NE != NULL) 
	nbrs.push_back(F_NE); 
    QTreeNode<Object, Cell>* F_NW = F_NWNeighbor(); 
    if(F_NW != NULL)
	nbrs.push_back(F_NW); 
    QTreeNode<Object, Cell>* F_SW = F_SWNeighbor(); 
    if(F_SW != NULL)
	nbrs.push_back(F_SW); 
    QTreeNode<Object, Cell>* F_SE = F_SENeighbor(); 
    if(F_SE != NULL)
	nbrs.push_back(F_SE); 
    QTreeNode<Object, Cell>* B_NE = B_NENeighbor(); 
    if(B_NE != NULL)
	nbrs.push_back(B_NE); 
    QTreeNode<Object, Cell>* B_NW = B_NWNeighbor(); 
    if(B_NW != NULL)
	nbrs.push_back(B_NW); 
    QTreeNode<Object, Cell>* B_SW = B_SWNeighbor(); 
    if(B_SW != NULL) 
	nbrs.push_back(B_SW); 
    QTreeNode<Object, Cell>* B_SE = B_SENeighbor(); 
    if(B_SE != NULL)
	nbrs.push_back(B_SE); 
} 

/*! Get the neighbors on the four due directions. */
template <class Object, class Cell>
void QTreeNode<Object, Cell>::getAllFaceNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs)
{ 
    getAllSouthNeighbors( nbrs );
    getAllNorthNeighbors( nbrs );
    getAllWestNeighbors ( nbrs );
    getAllEastNeighbors ( nbrs );
    getAllFrontNeighbors( nbrs );
    getAllBackNeighbors ( nbrs );
}

/*! Get the neighbors on the edges. */
template <class Object, class Cell>
void QTreeNode<Object, Cell>::getAllEdgeNeighbors(std::vector< QTreeNode<Object, Cell> * > &nbrs) 
{
    getNWNeighbors ( nbrs );
    getNENeighbors ( nbrs );
    getSWNeighbors ( nbrs );
    getSENeighbors ( nbrs );
    getF_NNeighbors( nbrs );
    getF_SNeighbors( nbrs );
    getF_ENeighbors( nbrs );
    getF_WNeighbors( nbrs );
    getB_NNeighbors( nbrs );
    getB_SNeighbors( nbrs );
    getB_ENeighbors( nbrs );
    getB_WNeighbors( nbrs );
}

/*! Return true if the current node is smaller than its neighbors,
  forgery if not (it is has to say has a major level in the octree). */
template <class Object, class Cell>
bool QTreeNode<Object, Cell>::isSmallerThanNeighbors( std::vector< QTreeNode<Object, Cell> * > &nbrs )
{  
    // If a node has no neighbors.
    if ( nbrs.empty() )
	return true;
  
    // Si un voisin est plus petit.
    for ( int i = 0 ; i < nbrs.size() ; i++ )
	if ( nbrs[i]->depth > depth) 
	    return false;

    return true;
}
