/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QTreeNode
 * 2007-04-06
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# ifndef QTREENODE_H
# define QTREENODE_H

# include <iostream>
# include <vector>

# include <QData/QGraph.h>
# include <QData/QGraphNode.h>
# include <QData/QGraphEdge.h>
# include <QData/QRegion3f.h>

namespace QData {
  
enum FACE_TYPE{B,F,W,E,S,N};
enum NODE_TYPE{B_SW, F_SW, B_SE, F_SE, B_NW, F_NW, B_NE, F_NE};
enum EDGE_TYPE{B_E, B_W, B_S, B_N, F_E, F_W, F_S, F_N, S_E, N_E, S_W, N_W};  

/****************************************************************************/
/*!
  \class QTreeNode QTreeNode.h
  \brief A generic node class for all trees in Axel 
*/
/****************************************************************************/
template <class Object, class Cell> class QTreeNode
{

public:
    
    Cell cell;
    Object object(void) { return objects.first() ; }
    QList<Object> objects ;

    NODE_TYPE type;
    
    QTreeNode* parent;   
 
    QTreeNode* F_NEchild; 
    QTreeNode* F_NWchild;
    QTreeNode* F_SEchild;
    QTreeNode* F_SWchild;

    QTreeNode* B_NEchild;
    QTreeNode* B_NWchild; 
    QTreeNode* B_SEchild; 
    QTreeNode* B_SWchild; 
    
    int depth;    
    int index;
    
protected :
    
    QTreeNode(QTreeNode& node);
    
public:

    QTreeNode(QTreeNode<Object, Cell> * left, QTreeNode<Object, Cell> * right, Cell cell) ;
    QTreeNode(NODE_TYPE nodeType, QTreeNode<Object, Cell>* parent, Cell & cell);
    QTreeNode(Object object, Cell cell) ;
    QTreeNode();
    
    void  SetCell(Cell &c) { cell = c; }
    
    void  SetParent(QTreeNode* n){parent = n;}
    
    void  SetF_NEchild( QTreeNode<Object, Cell>* n){F_NEchild = n;}
    void  SetF_NWchild( QTreeNode<Object, Cell>* n){F_NWchild = n;}
    void  SetF_SEchild( QTreeNode<Object, Cell>* n){F_SEchild = n;}
    void  SetF_SWchild( QTreeNode<Object, Cell>* n){F_SWchild = n;} 
    
    void  SetB_NEchild( QTreeNode<Object, Cell>* n){B_NEchild = n;} 
    void  SetB_NWchild( QTreeNode<Object, Cell>* n){B_NWchild = n;} 
    void  SetB_SEchild( QTreeNode<Object, Cell>* n){B_SEchild = n;} 
    void  SetB_SWchild( QTreeNode<Object, Cell>* n){B_SWchild = n;} 
    
    Cell&          GetCell() {return cell;}
    
    QTreeNode<Object, Cell>*    GetF_NEchild(){return(F_NEchild);}
    QTreeNode<Object, Cell>*    GetF_NWchild(){return(F_NWchild);}
    QTreeNode<Object, Cell>*    GetF_SEchild(){return(F_SEchild);}
    QTreeNode<Object, Cell>*    GetF_SWchild(){return(F_SWchild);}
    
    QTreeNode<Object, Cell>*    GetB_NEchild(){return(B_NEchild);} 
    QTreeNode<Object, Cell>*    GetB_NWchild(){return(B_NWchild);} 
    QTreeNode<Object, Cell>*    GetB_SEchild(){return(B_SEchild);} 
    QTreeNode<Object, Cell>*    GetB_SWchild(){return(B_SWchild);}

    QTreeNode<Object, Cell>*    left(void)  { return F_NWchild ; }
    QTreeNode<Object, Cell>*    right(void) { return F_NEchild ; }
    
    bool isLeaf();
    
    void getAllFaceNeighbors(std::vector< QTreeNode<Object, Cell> * > &nbrs); 
    void getAllNeighbors(std::vector< QTreeNode<Object, Cell> * > &nbrs);
    void getAllEdgeNeighbors(std::vector< QTreeNode<Object, Cell> * > &nbrs);
    
    //---------------DUE SOUTH---------------------

    void getAllSouthNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs);
    QTreeNode<Object, Cell>* getSouthNeighborNode();
    void getNorthBoundary(std::vector< QTreeNode<Object, Cell>*>& nbrs);
    
    //---------------DUE NORTH---------------------
    
    void getAllNorthNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs);
    QTreeNode<Object, Cell>* getNorthNeighborNode();
    void getSouthBoundary(std::vector< QTreeNode<Object, Cell>*>& nbrs);

    //---------------DUE WEST---------------------

    void getAllWestNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs);
    QTreeNode<Object, Cell>* getWestNeighborNode();
    void getEastBoundary(std::vector< QTreeNode<Object, Cell>*>& nbrs);
    
    //--------DUE EAST--------------------
    
    void getWestBoundary(std::vector< QTreeNode<Object, Cell>*>& nbrs);
    void getAllEastNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs);
    QTreeNode<Object, Cell>* getEastNeighborNode(); 

    //---------------DUE FRONT--------------------- 

    void getAllFrontNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs); 
    QTreeNode<Object, Cell>* getFrontNeighborNode(); 
    void getBackBoundary(std::vector< QTreeNode<Object, Cell>*>& nbrs); 

    //---------------DUE BACK--------------------- 

    void getAllBackNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs); 
    QTreeNode<Object, Cell>* getBackNeighborNode(); 
    void getFrontBoundary(std::vector< QTreeNode<Object, Cell>*>& nbrs); 

    /************************************************ 
     *             CORNER-LINE PARTS                * 
     ************************************************/ 
    /*! Supporting routine */
    void getNWLine(std::vector< QTreeNode<Object, Cell>*>& line); 
    void getNELine(std::vector< QTreeNode<Object, Cell>*>& line); 
    void getSWLine(std::vector< QTreeNode<Object, Cell>*>& line); 
    void getSELine(std::vector< QTreeNode<Object, Cell>*>& line); 
    void getF_NLine(std::vector< QTreeNode<Object, Cell>*>& line); 
    void getF_SLine(std::vector< QTreeNode<Object, Cell>*>& line); 
    void getF_ELine(std::vector< QTreeNode<Object, Cell>*>& line); 
    void getF_WLine(std::vector< QTreeNode<Object, Cell>*>& line); 
    void getB_NLine(std::vector< QTreeNode<Object, Cell>*>& line); 
    void getB_SLine(std::vector< QTreeNode<Object, Cell>*>& line); 
    void getB_ELine(std::vector< QTreeNode<Object, Cell>*>& line); 
    void getB_WLine(std::vector< QTreeNode<Object, Cell>*>& line); 
	 
    /*! The neighbors on the edge */
    void getNWNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs); 
    void getNENeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs); 
    void getSWNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs); 
    void getSENeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs); 
    void getF_NNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs); 
    void getF_SNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs); 
    void getF_ENeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs); 
    void getF_WNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs); 
    void getB_NNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs); 
    void getB_SNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs); 
    void getB_ENeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs); 
    void getB_WNeighbors(std::vector< QTreeNode<Object, Cell>*>& nbrs); 

    /************************************************ 
     *             TRUE CORNER PARTS                * 
     ************************************************/ 
    /*! Supprorting routine */
    QTreeNode<Object, Cell>* getF_NWCorner(); 
    QTreeNode<Object, Cell>* getF_NECorner(); 
    QTreeNode<Object, Cell>* getF_SWCorner(); 
    QTreeNode<Object, Cell>* getF_SECorner(); 
    
    QTreeNode<Object, Cell>* getB_NWCorner(); 
    QTreeNode<Object, Cell>* getB_NECorner(); 
    QTreeNode<Object, Cell>* getB_SWCorner(); 
    QTreeNode<Object, Cell>* getB_SECorner(); 
 
    /*! Here come the box on the vertex */
    QTreeNode<Object, Cell>* F_NENeighbor();
    QTreeNode<Object, Cell>* F_NWNeighbor(); 
    QTreeNode<Object, Cell>* F_SWNeighbor(); 
    QTreeNode<Object, Cell>* F_SENeighbor(); 
 
    QTreeNode<Object, Cell>* B_NENeighbor();
    QTreeNode<Object, Cell>* B_NWNeighbor(); 
    QTreeNode<Object, Cell>* B_SWNeighbor(); 
    QTreeNode<Object, Cell>* B_SENeighbor(); 

    bool isSmallerThanNeighbors( std::vector< QTreeNode<Object, Cell> * > &nbrs );
};
    
# include "QTreeNode.cpp"

} ; // namespace QData

# endif // QTREENODE_H
