/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QTreeNode4D
 * 2007-05-09
 * Stephane CHAU
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# ifndef QTREENODE4D_H
# define QTREENODE4D_H

# include <iostream>
# include <vector>

# include <QData/QGraph.h>
# include <QData/QGraphNode.h>
# include <QData/QGraphEdge.h>
# include <QData/QRegion3f.h>

namespace QData {
  
enum FACE_TYPE{SMIN,SMAX,TMIN,TMAX,UMIN,UMAX,VMIN,VMAX};
enum NODE_TYPE{LLLL, LLLR, LLRL, LLRR, LRLL, LRLR, LRRL, LRRR, RLLL, RLLR, RLRL, RLRR, RRLL, RRLR, RRRL, RRRR};
// enum EDGE_TYPE{B_E, B_W, B_S, B_N, F_E, F_W, F_S, F_N, S_E, N_E, S_W, N_W};  

// enum FACE_TYPE{B,F,W,E,S,N};
// enum NODE_TYPE{B_SW, F_SW, B_SE, F_SE, B_NW, F_NW, B_NE, F_NE};
// enum EDGE_TYPE{B_E, B_W, B_S, B_N, F_E, F_W, F_S, F_N, S_E, N_E, S_W, N_W};  


/****************************************************************************/
/*!
  \class QTreeNode4D QTreeNode4D.h
  \brief A generic node class for all trees in Axel 
*/
/****************************************************************************/
template <class Object, class Cell> class QTreeNode4D
{

public:
    
    Cell cell;
    Object object(void) { return objects.first() ; }
    QList<Object> objects ;

    NODE_TYPE type;
    
    QTreeNode4D* parent;   

	QTreeNode4D* LLLLchild;
    QTreeNode4D* LLLRchild;
    QTreeNode4D* LLRLchild;
    QTreeNode4D* LLRRchild;
    QTreeNode4D* LRLLchild;
    QTreeNode4D* LRLRchild;
    QTreeNode4D* LRRLchild;
    QTreeNode4D* LRRRchild;
    QTreeNode4D* RLLLchild;
    QTreeNode4D* RLLRchild;
    QTreeNode4D* RLRLchild;
    QTreeNode4D* RLRRchild;
    QTreeNode4D* RRLLchild;
    QTreeNode4D* RRLRchild;
    QTreeNode4D* RRRLchild;
    QTreeNode4D* RRRRchild;

    
    int depth;    
    int index;
    
protected :
    
    QTreeNode4D(QTreeNode4D& node);
    
public:

    // QTreeNode4D(QTreeNode4D<Object, Cell> * left, QTreeNode4D<Object, Cell> * right, Cell cell) ;
    QTreeNode4D(NODE_TYPE nodeType, QTreeNode4D<Object, Cell>* parent, Cell & cell);
    QTreeNode4D(int nodeType, QTreeNode4D<Object, Cell>* parent, Cell & cell);
    QTreeNode4D(Object object, Cell cell) ;
    QTreeNode4D(void);
    
    void SetCell(Cell &c) { cell = c; }

    void SetParent(QTreeNode4D* n){parent = n;}
    
	void SetLLLLchild(QTreeNode4D<Object, Cell> * n){ LLLLchild = n ; }
	void SetLLLRchild(QTreeNode4D<Object, Cell> * n){ LLLRchild = n ; }
	void SetLLRLchild(QTreeNode4D<Object, Cell> * n){ LLRLchild = n ; }
	void SetLLRRchild(QTreeNode4D<Object, Cell> * n){ LLRRchild = n ; }
	void SetLRLLchild(QTreeNode4D<Object, Cell> * n){ LRLLchild = n ; }
	void SetLRLRchild(QTreeNode4D<Object, Cell> * n){ LRLRchild = n ; }
	void SetLRRLchild(QTreeNode4D<Object, Cell> * n){ LRRLchild = n ; }
	void SetLRRRchild(QTreeNode4D<Object, Cell> * n){ LRRRchild = n ; }
	void SetRLLLchild(QTreeNode4D<Object, Cell> * n){ RLLLchild = n ; }
	void SetRLLRchild(QTreeNode4D<Object, Cell> * n){ RLLRchild = n ; }
	void SetRLRLchild(QTreeNode4D<Object, Cell> * n){ RLRLchild = n ; }
	void SetRLRRchild(QTreeNode4D<Object, Cell> * n){ RLRRchild = n ; }
	void SetRRLLchild(QTreeNode4D<Object, Cell> * n){ RRLLchild = n ; }
	void SetRRLRchild(QTreeNode4D<Object, Cell> * n){ RRLRchild = n ; }
	void SetRRRLchild(QTreeNode4D<Object, Cell> * n){ RRRLchild = n ; }
	void SetRRRRchild(QTreeNode4D<Object, Cell> * n){ RRRRchild = n ; }
	void SetChild(int i, QTreeNode4D<Object, Cell> * n);
	
    Cell& GetCell(void) {return cell;}

	QTreeNode4D<Object, Cell>* GetLLLLchild(void){ return(LLLLchild) ; }
	QTreeNode4D<Object, Cell>* GetLLLRchild(void){ return(LLLRchild) ; }
	QTreeNode4D<Object, Cell>* GetLLRLchild(void){ return(LLRLchild) ; }
	QTreeNode4D<Object, Cell>* GetLLRRchild(void){ return(LLRRchild) ; }
	QTreeNode4D<Object, Cell>* GetLRLLchild(void){ return(LRLLchild) ; }
	QTreeNode4D<Object, Cell>* GetLRLRchild(void){ return(LRLRchild) ; }
	QTreeNode4D<Object, Cell>* GetLRRLchild(void){ return(LRRLchild) ; }
	QTreeNode4D<Object, Cell>* GetLRRRchild(void){ return(LRRRchild) ; }
	QTreeNode4D<Object, Cell>* GetRLLLchild(void){ return(RLLLchild) ; }
	QTreeNode4D<Object, Cell>* GetRLLRchild(void){ return(RLLRchild) ; }
	QTreeNode4D<Object, Cell>* GetRLRLchild(void){ return(RLRLchild) ; }
	QTreeNode4D<Object, Cell>* GetRLRRchild(void){ return(RLRRchild) ; }
	QTreeNode4D<Object, Cell>* GetRRLLchild(void){ return(RRLLchild) ; }
	QTreeNode4D<Object, Cell>* GetRRLRchild(void){ return(RRLRchild) ; }
	QTreeNode4D<Object, Cell>* GetRRRLchild(void){ return(RRRLchild) ; }
	QTreeNode4D<Object, Cell>* GetRRRRchild(void){ return(RRRRchild) ; }
	QTreeNode4D<Object, Cell>* GetChild(int i);
	
    bool isLeaf(void);
    
    // void getAllFaceNeighbors(std::vector< QTreeNode4D<Object, Cell> * > &nbrs); 
    // void getAllNeighbors(std::vector< QTreeNode4D<Object, Cell> * > &nbrs);
    // void getAllEdgeNeighbors(std::vector< QTreeNode4D<Object, Cell> * > &nbrs);
    
    //---------------DUE SMIN---------------------

    void getAllSminNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs);
    QTreeNode4D<Object, Cell>* getSminNeighborNode(void);
    void getSmaxBoundary(std::vector< QTreeNode4D<Object, Cell>*>& nbrs);
    
    //---------------DUE SMAX---------------------

    void getAllSmaxNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs);
    QTreeNode4D<Object, Cell>* getSmaxNeighborNode(void);
    void getSminBoundary(std::vector< QTreeNode4D<Object, Cell>*>& nbrs);

    //---------------DUE TMIN---------------------

    void getAllTminNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs);
    QTreeNode4D<Object, Cell>* getTminNeighborNode(void);
    void getTmaxBoundary(std::vector< QTreeNode4D<Object, Cell>*>& nbrs);
    
    //---------------DUE TMAX---------------------

    void getAllTmaxNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs);
    QTreeNode4D<Object, Cell>* getTmaxNeighborNode(void);
    void getTminBoundary(std::vector< QTreeNode4D<Object, Cell>*>& nbrs);

    //---------------DUE UMIN---------------------

    void getAllUminNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs);
    QTreeNode4D<Object, Cell>* getUminNeighborNode(void);
    void getUmaxBoundary(std::vector< QTreeNode4D<Object, Cell>*>& nbrs);
    
    //---------------DUE UMAX---------------------

    void getAllUmaxNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs);
    QTreeNode4D<Object, Cell>* getUmaxNeighborNode(void);
    void getUminBoundary(std::vector< QTreeNode4D<Object, Cell>*>& nbrs);

    //---------------DUE VMIN---------------------

    void getAllVminNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs);
    QTreeNode4D<Object, Cell>* getVminNeighborNode(void);
    void getVmaxBoundary(std::vector< QTreeNode4D<Object, Cell>*>& nbrs);
    
    //---------------DUE VMAX---------------------

    void getAllVmaxNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs);
    QTreeNode4D<Object, Cell>* getVmaxNeighborNode(void);
    void getVminBoundary(std::vector< QTreeNode4D<Object, Cell>*>& nbrs);


    /************************************************ 
     *             CORNER-LINE PARTS                * 
     ************************************************/ 
    /*! Supporting routine */
    // void getNWLine(std::vector< QTreeNode4D<Object, Cell>*>& line); 
    // void getNELine(std::vector< QTreeNode4D<Object, Cell>*>& line); 
    // void getSWLine(std::vector< QTreeNode4D<Object, Cell>*>& line); 
    // void getSELine(std::vector< QTreeNode4D<Object, Cell>*>& line); 
    // void getF_NLine(std::vector< QTreeNode4D<Object, Cell>*>& line); 
    // void getF_SLine(std::vector< QTreeNode4D<Object, Cell>*>& line); 
    // void getF_ELine(std::vector< QTreeNode4D<Object, Cell>*>& line); 
    // void getF_WLine(std::vector< QTreeNode4D<Object, Cell>*>& line); 
    // void getB_NLine(std::vector< QTreeNode4D<Object, Cell>*>& line); 
    // void getB_SLine(std::vector< QTreeNode4D<Object, Cell>*>& line); 
    // void getB_ELine(std::vector< QTreeNode4D<Object, Cell>*>& line); 
    // void getB_WLine(std::vector< QTreeNode4D<Object, Cell>*>& line); 
	 
    /*! The neighbors on the edge */

    // void getNWNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs); 
    // void getNENeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs); 
    // void getSWNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs); 
    // void getSENeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs); 
    // void getF_NNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs); 
    // void getF_SNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs); 
    // void getF_ENeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs); 
    // void getF_WNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs); 
    // void getB_NNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs); 
    // void getB_SNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs); 
    // void getB_ENeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs); 
    // void getB_WNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs); 

    /************************************************ 
     *             TRUE CORNER PARTS                * 
     ************************************************/ 
    /*! Supprorting routine */

    // QTreeNode4D<Object, Cell>* getF_NWCorner(); 
    // QTreeNode4D<Object, Cell>* getF_NECorner(); 
    // QTreeNode4D<Object, Cell>* getF_SWCorner(); 
    // QTreeNode4D<Object, Cell>* getF_SECorner(); 
    // 
    // QTreeNode4D<Object, Cell>* getB_NWCorner(); 
    // QTreeNode4D<Object, Cell>* getB_NECorner(); 
    // QTreeNode4D<Object, Cell>* getB_SWCorner(); 
    // QTreeNode4D<Object, Cell>* getB_SECorner(); 
    //  
    // /*! Here come the box on the vertex */
    // QTreeNode4D<Object, Cell>* F_NENeighbor();
    // QTreeNode4D<Object, Cell>* F_NWNeighbor(); 
    // QTreeNode4D<Object, Cell>* F_SWNeighbor(); 
    // QTreeNode4D<Object, Cell>* F_SENeighbor(); 
    //  
    // QTreeNode4D<Object, Cell>* B_NENeighbor();
    // QTreeNode4D<Object, Cell>* B_NWNeighbor(); 
    // QTreeNode4D<Object, Cell>* B_SWNeighbor(); 
    // QTreeNode4D<Object, Cell>* B_SENeighbor(); 

    // bool isSmallerThanNeighbors( std::vector< QTreeNode4D<Object, Cell> * > &nbrs );
};
    
# include "QTreeNode4D.cpp"

} ; // namespace QData

# endif // QTREENODE4D_H
