/*****************************************************************************
 * Axel
 *****************************************************************************
 * MainWindow
 * 2005-10-18
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# ifndef MAINWINDOW_H
# define MAINWINDOW_H

# include <QMainWindow>
# include <QGLViewer/qglviewer.h>

using namespace qglviewer;

class QFrame ;
class QAction ;
class QActionGroup ;
class QUndoStack ;
class QPixmap ;
class QLabel ;
class QToolBar ;

class QStackedWidget ;
class QCameraModeWidget;
class QSelectedObjectsWidget;
class QAlgorithmWidget;
class QDimensionModeWidget;

class Viewer ;
class SampleSelector ;
class ObjectToolDock ;
class ObjectPropertiesDock ;
class ToolPropertiesDock ;
class SceneInspectorDock ;
class ScriptingDock ;
class HistoryDock ;

class MainWindow : public QMainWindow 
{
    Q_OBJECT

    friend class Viewer;
    
public:
    MainWindow(void) ;
    Viewer * viewer(void) ;
    QUndoStack * stack(void) ;
    QFrame * deamonsFrame(void) ;
    QCameraModeWidget * cameraModeWidget(void);
    QAlgorithmWidget * algorithmWidget(void) ;
    
    QMenu * importMenu(void) {
	return importMenu_ ;
    }

    QMenu * exportMenu(void) {
	return exportMenu_ ;
    }
    
public slots:
    void fileNew(void) ;
    void fileOpen(QString s = NULL) ;
    void fileSave(void) ;
    void fileSaveAs(void) ;
    void fileClose(void) ;
    void documentWasModified();
    void importVectSlot(QString s = NULL) ;
    void importOffSlot(QString s = NULL) ;
    void importGcmlSlot(QString s = NULL);
    void importG2CurveSlot(QString s = NULL) ;
    void importG2SurfaceSlot(QString s = NULL) ;
    void exportVectSlot(void) ;
    void exportOffSlot(void) ;  
    void exportGcmlSlot(void) ;  
    void showPreferencesDialog(void) ;	
    void saveSnapshot(void) ;
    void help(void) ;
    
    void setWireDisplaySlot(bool checked) ;
    void setLitWireDisplaySlot(bool checked) ;
    void setFlatDisplaySlot(bool checked) ;
    void setSmoothDisplaySlot(bool checked) ;
        
    void setVertexEditionMode(bool checked) ;
    void setEdgeEditionMode(bool checked) ;
    void setFaceEditionMode(bool checked) ;
    
    void showAboutDialog(void) ;
    void aboutPlugins(void) ;
    void checkForUpdates(void) ;
    
    void displayMessage(QString title, QString text, QPixmap image = QPixmap()) ;
    void displayAxisMessage(bool on) ;
    void displayAxisOnMessage(void) ;
    void displayAxisOffMessage(void) ;
    void displayOrthographicModeMessage(void) ;
    void displayPerspectiveModeMessage(void) ;
    void display2dModeMessage(void) ;
    void display3dModeMessage(void) ;
    void displayFullScreenMessage(bool on) ;
    void displayFullScreenOnMessage(void) ;
    void displayFullScreenOffMessage(void) ;
    
    //************** AF SandBox
    void addObject(const Vec &v);
    //*************************
    
    ScriptingDock * scriptingDock(void) { return scriptingDock_ ; }
    ToolPropertiesDock * toolPropertiesDock(void) { return toolPropertiesDock_ ; }

    void loadImplicitCurvesArrangementDock(void) ;    
    void loadSampleSelector(void) ;
    void unloadSampleSelector(QString) ;
	
protected:
    void closeEvent(QCloseEvent *event) ;
    
private:
    void loadPlugins(void) ;
    void createViewer(void) ;
    void createDocks(void) ;
    void createActions(void) ;
    void createMenu(void) ;
    void createToolBars(void) ;
    void setStatusBar(void) ;

    bool maybeSave(void) ;
    void readSettings(void) ;
    void writeSettings(void) ;

public:

    void setCurrentFile(const QString &fileName);
    QString strippedName(const QString &fullFileName);
    MainWindow *findMainWindow(const QString &fileName);

private :

    bool isUntitled ;
    QString curFile_;
    QStackedWidget * centerArea_ ;
    Viewer * viewer_ ;
    //SampleSelector * sampleSelector_ ;

    QUndoStack * undoStack_ ;

    ObjectToolDock * objectToolDock_ ;
    ObjectPropertiesDock * objectPropertiesDock_ ;
    ToolPropertiesDock * toolPropertiesDock_ ;
    HistoryDock * historyDock_ ;
    SceneInspectorDock * sceneInspectorDock_ ;
    ScriptingDock * scriptingDock_ ;

    QTabWidget * helpWidget_ ;
    
    QMenu * fileMenu_ ;
    QAction * fileNewAction_ ;
    QAction * fileOpenAction_ ;
    QAction * fileSaveAction_ ;
    QAction * fileSaveAsAction_ ;
    QAction * fileCloseAction_ ;
    QMenu * importMenu_ ;
    QMenu * exportMenu_ ;
    QAction * importVectAction_ ;
    QAction * importOffAction_ ;
    QAction * importG2CurveAction_ ;
    QAction * importG2SurfaceAction_ ;
    QAction * importGcmlAction_ ;
    QAction * exportVectAction_ ;
    QAction * exportOffAction_ ;
    QAction * exportGcmlAction_ ;
    QAction * saveSnaphotAction_ ;
    QAction * showPreferencesDialogAction_ ;
    QAction * quitAction_ ;
    
    QMenu * editMenu_ ;
    QAction * undoAction_ ;
    QAction * redoAction_ ;
    
    QMenu * selectionMenu_ ;
    QAction * selectAllAction_ ;
    QAction * unselectAllAction_ ;
    QAction * deleteSelectionAction_ ;
    QAction * toggleEditionMode_ ;
    QActionGroup * selectionActionGroup_ ;
    QAction * toggleVertexSelectionAction_ ;
    QAction * toggleEdgeSelectionAction_ ;
    QAction * toggleFaceSelectionAction_ ;
    
    QMenu * displayMenu_ ;
    QAction * toggleAxisAction_ ;
    QAction * toggleXYGridAction_ ;
    QAction * toggleXZGridAction_ ;
    QAction * toggleYZGridAction_ ;
    QActionGroup * displayActionGroup_ ;
    QAction * toggleWireDisplayAction_ ;
    QAction * toggleLitWireDisplayAction_ ;
    QAction * toggleFlatDisplayAction_ ;
    QAction * toggleSmoothDisplayAction_ ;
    QActionGroup * cameraModeGroupAction_ ;
    QActionGroup * dimensionModeGroupAction_ ;
    QAction * perspectiveCameraModeAction_ ;
    QAction * orthographicCameraModeAction_ ;
    QAction * twoDimensionModeAction_ ;
    QAction * threeDimensionModeAction_ ;

    QMenu * renderMenu_ ;
    QAction * yafrayRenderAction_ ;
    QAction * surfRenderAction_ ;
    
    QMenu * windowMenu_ ;
    QAction * toggleObjectToolDockAction_ ;
    QAction * toggleObjectPropertiesDockAction_ ;
    QAction * toggleToolPropertiesDockAction_ ;
    QAction * toggleSceneInspectorDockAction_ ;
    QAction * toggleScriptingDockAction_ ;
    QAction * toggleFullScreenAction_ ;
    QAction * toggleHistoryDockAction_ ;
    
    QMenu * scriptMenu_ ;
    QAction * loadScriptAction_ ;
    QAction * saveScriptAction_ ;
    QAction * saveAsScriptAction_ ;
    QAction * runScriptAction_ ;
    
    QMenu * helpMenu_ ;
    QAction * searchSampleAction_ ;
    QAction * showHelpDialogAction_ ;
    QAction * aboutPluginsAction_ ;
    QAction * showAboutDialogAction_ ;
    QAction * checkForUpdatesAction_ ;

    QToolBar * fileToolBar ;
    QToolBar * editToolBar ;
    QToolBar * displayToolBar ;
    
    QFrame * deamonsFrame_ ;
    QDimensionModeWidget * dimensionModeWidget_ ;
    QAlgorithmWidget * algorithmWidget_;
    QSelectedObjectsWidget * selectedObjectsWidget_ ;
    QCameraModeWidget * cameraModeWidget_;
       
} ;

# endif
