/*****************************************************************************
 * A x e l
 *****************************************************************************
 * ObjectToolDock
 * 2006-09-28
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# include <QIcon>
# include <QTreeWidget>
# include <QTreeWidgetItem>
# include <QtGui/QHeaderView>

# include <QGui/ObjectToolDock.h>
# include <QGui/SheetDelegate.h>

ObjectToolDock::ObjectToolDock(const QString & title, QWidget * parent) : QDockWidget(title, parent) 
{
	/***********************************************************************************************
	 * Objects
	 **********************************************************************************************/
   	tree = new QTreeWidget(this);
	tree->setColumnCount(1) ;
	
	objects = new QTreeWidgetItem(tree) ;
	objects->setText(0, "Objects") ;                       
	
	point = new QTreeWidgetItem(objects) ;
	point->setText(0, "Point") ;
	point->setIcon(0, QIcon(QPixmap(":images/objects/point.png"))) ;
	
	plane = new QTreeWidgetItem(objects) ;
	plane->setText(0, "Plane") ;
	plane->setIcon(0, QIcon(QPixmap(":images/objects/plane.png"))) ;
	
	curve = new QTreeWidgetItem(objects) ;
	curve->setText(0, "Curve") ;
	curve->setIcon(0, QIcon(QPixmap(":images/objects/parametric_curve.png"))) ;
	
	surface = new QTreeWidgetItem(objects) ;
	surface->setText(0, "Surface") ;            
	surface->setIcon(0, QIcon(QPixmap(":images/objects/parametric_surface.png"))) ;
	
	/***********************************************************************************************
	 * Intersections
	 **********************************************************************************************/
	
	intersections = new QTreeWidgetItem(tree) ;
	intersections->setText(0, "Intersections") ;

	icintersection = new QTreeWidgetItem(intersections) ;
	icintersection->setText(0, "Implicit curves") ;
	icintersection->setIcon(0, QIcon(QPixmap(":images/objects/point.png"))) ;
	
	pcintersection = new QTreeWidgetItem(intersections) ;
	pcintersection->setText(0, "Parametric curves") ;
	pcintersection->setIcon(0, QIcon(QPixmap(":images/objects/point.png"))) ;
	
	isintersection = new QTreeWidgetItem(intersections) ;
	isintersection->setText(0, "Implicit surfaces") ;
	isintersection->setIcon(0, QIcon(QPixmap(":images/objects/parametric_curve.png"))) ;
	
	psintersection = new QTreeWidgetItem(intersections) ;
	psintersection->setText(0, "Parametric surfaces") ;
	psintersection->setIcon(0, QIcon(QPixmap(":images/objects/parametric_curve.png"))) ;
	
	/***********************************************************************************************
	 * Interpolations
	 **********************************************************************************************/
	
	interpolations = new QTreeWidgetItem(tree) ;
	interpolations->setText(0, "Interpolation") ;
	
	bscinterpolation = new QTreeWidgetItem(interpolations) ;
	bscinterpolation->setText(0, "BSplineCurve") ;
	bscinterpolation->setIcon(0, QIcon(QPixmap(":images/objects/parametric_curve.png"))) ;

	/***********************************************************************************************
	 * Appoximations
	 **********************************************************************************************/
	
	approximations = new QTreeWidgetItem(tree) ;
	approximations->setText(0, "Approximation") ;
	
	bscapproximation = new QTreeWidgetItem(approximations) ;
	bscapproximation->setText(0, "BSplineCurve") ;
	bscapproximation->setIcon(0, QIcon(QPixmap(":images/objects/parametric_curve.png"))) ;

	bssapproximation = new QTreeWidgetItem(approximations) ;
	bssapproximation->setText(0, "BSplineSurface") ;
	bssapproximation->setIcon(0, QIcon(QPixmap(":images/objects/parametric_surface.png"))) ;
	/***********************************************************************************************
	 * Joins
	 **********************************************************************************************/
	
	joins = new QTreeWidgetItem(tree) ;
	joins->setText(0, "Join") ;
	
	bscjoin = new QTreeWidgetItem(joins) ;
	bscjoin->setText(0, "BSplineCurve") ;
	bscjoin->setIcon(0, QIcon(QPixmap(":images/objects/parametric_curve.png"))) ;
	
	/***********************************************************************************************
	 * Set-theoretic Modeling
	 **********************************************************************************************/
	
	settheoretic = new QTreeWidgetItem(tree) ;
	settheoretic->setText(0, "Set Theoretic Modeling") ;
	
	bunion = new QTreeWidgetItem(settheoretic) ;
	bunion->setText(0, "Union") ;
	bunion->setIcon(0, QIcon(QPixmap(":images/objects/boolean_minus.png"))) ;
	
	bintersection = new QTreeWidgetItem(settheoretic) ;
	bintersection->setText(0, "Intersection") ;
	bintersection->setIcon(0, QIcon(QPixmap(":images/objects/boolean_minus.png"))) ;
	
	bdifference = new QTreeWidgetItem(settheoretic) ;
	bdifference->setText(0, "Difference") ;
	bdifference->setIcon(0, QIcon(QPixmap(":images/objects/boolean_minus.png"))) ;
                
	curves_arrangement = new QTreeWidgetItem(settheoretic) ;
	curves_arrangement->setText(0, "Curves arrangement") ;
	curves_arrangement->setIcon(0, QIcon(QPixmap(":images/objects/arrangement.png"))) ;
	
	surfaces_arrangement = new QTreeWidgetItem(settheoretic) ;
	surfaces_arrangement->setText(0, "Surfaces arrangement") ;
	surfaces_arrangement->setIcon(0, QIcon(QPixmap(":images/objects/arrangement.png"))) ;


	
	differential_calculus = new QTreeWidgetItem(tree) ;
	differential_calculus->setText(0, "Differential Calculus") ;

	critical_points = new QTreeWidgetItem(differential_calculus) ;
	critical_points->setText(0, "Critical points") ;
	critical_points->setIcon(0, QIcon(QPixmap(":images/objects/point.png"))) ;
	
	first_derivative_curve = new QTreeWidgetItem(differential_calculus) ;
	first_derivative_curve->setText(0, "First derivative curve") ;
	first_derivative_curve->setIcon(0, QIcon(QPixmap(":images/objects/parametric_curve.png"))) ;
	
	second_derivative_curve = new QTreeWidgetItem(differential_calculus) ;
	second_derivative_curve->setText(0, "Second derivative curve") ;
	second_derivative_curve->setIcon(0, QIcon(QPixmap(":images/objects/parametric_curve.png"))) ;
	
	jacobi_curve = new QTreeWidgetItem(differential_calculus) ;
	jacobi_curve->setText(0, "Jacobi curve") ;
	jacobi_curve->setIcon(0, QIcon(QPixmap(":images/objects/parametric_curve.png"))) ;
	
	hessian_curve = new QTreeWidgetItem(differential_calculus) ;
	hessian_curve->setText(0, "Hessian curve") ;
	hessian_curve->setIcon(0, QIcon(QPixmap(":images/objects/parametric_curve.png"))) ;

	graphed_surface = new QTreeWidgetItem(differential_calculus) ;
	graphed_surface->setText(0, "Graphed surface") ;
	graphed_surface->setIcon(0, QIcon(QPixmap(":images/objects/parametric_surface.png"))) ;
	
	/***********************************************************************************************
	 * Constraints Modeling
	 **********************************************************************************************/
	
	constraints = new QTreeWidgetItem(tree) ;
	constraints->setText(0, "Constraints") ;                       

	incidenceConstraint = new QTreeWidgetItem(constraints) ;
	incidenceConstraint->setText(0, "Incidence Constraint") ;
	incidenceConstraint->setIcon(0, QIcon(QPixmap(":images/objects/incidenceConstraint.png"))) ;

	distanceConstraint = new QTreeWidgetItem(constraints) ;
	distanceConstraint->setText(0, "Distance Constraint") ;
	distanceConstraint->setIcon(0, QIcon(QPixmap(":images/objects/distanceConstraint.png"))) ;

	angleConstraint = new QTreeWidgetItem(constraints) ;
	angleConstraint->setText(0, "Angle Constraint") ;
	angleConstraint->setIcon(0, QIcon(QPixmap(":images/objects/angleConstraint.png"))) ;
	
	expandItem(objects) ;
	expandItem(intersections) ;
	expandItem(interpolations) ;
	expandItem(approximations) ;
	expandItem(joins) ;
	expandItem(settheoretic) ;
	expandItem(differential_calculus) ;
	expandItem(constraints) ;
                
	tree->setRootIsDecorated(false) ;
	tree->setItemDelegate(new SheetDelegate(tree, this)) ;
	tree->header()->hide() ;
	tree->header()->setStretchLastSection(true) ;
	
	connect(tree, SIGNAL(itemPressed(QTreeWidgetItem*,int)), this, SLOT(expandItem(QTreeWidgetItem*)));
	connect(tree, SIGNAL(itemClicked(QTreeWidgetItem *, int)), this, SLOT(emmitSignals(QTreeWidgetItem*)));
        
	setWidget(tree);  
}

void ObjectToolDock::expandItem(QTreeWidgetItem* item) {
	if (item == 0) return;
	if (item->parent() == 0) tree->setItemExpanded(item, !tree->isItemExpanded(item));  
}

void ObjectToolDock::emmitSignals(QTreeWidgetItem * item) {
	if(item == point) emit pointClicked() ;
	if(item == plane) emit planeClicked() ;
	if(item == curve) emit curveClicked() ;
	if(item == surface) emit surfaceClicked() ;
	
	// Intersections
	if(item == icintersection) emit icIntersectionClicked() ;
	if(item == pcintersection) emit pcIntersectionClicked() ;
	if(item == isintersection) emit isIntersectionClicked() ;
	if(item == psintersection) emit psIntersectionClicked() ;
	
	// Interpolations
	if(item == bscinterpolation) emit bscInterpolationClicked() ;
	
	// Approximations
	if(item == bscapproximation) emit bscApproximationClicked() ;
	if(item == bssapproximation) emit bssApproximationClicked() ;

	// Joins
	if(item == bscjoin) emit bscJoinClicked() ;

	// Set-theoretic Modeling
	if(item == bunion) emit bunionClicked() ;
	if(item == bintersection) emit bintersectionClicked() ;
	if(item == bdifference) emit bdifferenceClicked() ;
	if(item == curves_arrangement) emit curvesArrangementClicked() ;
	if(item == surfaces_arrangement) emit surfacesArrangementClicked() ;
	
	// Differential Calculus
	if(item == critical_points) emit criticalPointsClicked() ;
	if(item == first_derivative_curve) emit firstDerivativeClicked() ;
	if(item == second_derivative_curve) emit secondDerivativeClicked() ;
	if(item == jacobi_curve) emit jacobiCurveClicked() ;
	if(item == hessian_curve) emit hessianCurveClicked() ;
	if(item == graphed_surface) emit graphedSurfaceClicked() ;
	
	// Constraints
	if(item == incidenceConstraint) emit incidenceConstraintClicked() ;
	if(item == distanceConstraint) emit distanceConstraintClicked() ;
	if(item == angleConstraint) emit angleConstraintClicked() ;
}

//************** AF SandBox
int ObjectToolDock::getSelectedObjectType()
{
	if (tree->currentItem() == point)
		return 1;
	if (tree->currentItem() == plane)
		return 2;

	return 0 ;
}
//*************************
