/*****************************************************************************
 * A x e l
 *****************************************************************************
 * ParametricCurvesArrangementDock
 * 2007-06-20
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# ifndef PARAMETRICCURVESARRANGEMENTDOCK_H
# define PARAMETRICCURVESARRANGEMENTDOCK_H

# include "ui_ParametricCurvesArrangementWidget.h"

# include <QScrollArea>
# include <QDockWidget>
# include <QData/QGraph.h>
# include <QArrangement/Curves/QParametricCurvesArrangement.h>

using namespace std ;
using namespace QAxel ;
using namespace QData ;
using namespace QArrangement ;

class MainWindow ;
class QHBoxLayout ;
class QSplitter ;
class QToolBox ;
class QWidget ;
class QGraphWidget ;
class QLabel ;

class ParametricCurvesArrangementWidget : public QWidget
{
    Q_OBJECT
    
public:
    ParametricCurvesArrangementWidget(QWidget * parent = 0) : QWidget(parent) {
	ui.setupUi(this);
	connect(ui.playButton,            SIGNAL(clicked()),         this, SLOT(emitPlaySignal())) ;
	connect(ui.pauseButton,           SIGNAL(clicked()),         this, SLOT(emitPauseSignal())) ;
	connect(ui.stopButton,            SIGNAL(clicked()),         this, SLOT(emitStopSignal())) ;
	connect(ui.cellVerbosityCheckBox, SIGNAL(stateChanged(int)), this, SLOT(emitCellVerbosityModeChanged(int))) ;
	connect(ui.clickLocationCheckBox, SIGNAL(stateChanged(int)), this, SLOT(emitClickLocationMode(int))) ;
	connect(ui.clearButton,           SIGNAL(clicked()),         this, SLOT(emitClearLocation())) ;
    }
    
    double multiplicityFilteringRadius(void) { return ui.multipleRootsFilteringRadiusBox->value() ; }
    double isoleSolverPrecision(void)        { return ui.isoleSolverPrecisionBox->value() ; }
    double sbdSolverPrecision(void)          { return ui.sbdSolverPrecisionBox->value() ; }
    double cellPrecision(void)               { return ui.minCellBox->value() ; }
    double cellPrecisionMax(void)            { return ui.maxCellBox->value() ; }
    bool cellVerbosity(void)                 { return ui.cellVerbosityCheckBox->checkState() == Qt::Checked ; }
    bool fusionVerbosity(void)               { return ui.fusionVerbosityCheckBox->checkState() == Qt::Checked ; }

signals:
    void play(void) ;
    void pause(void) ;
    void stop(void) ;
    void showSubdivisionCells(void) ;
    void hideSubdivisionCells(void) ;
    void enterClickLocationMode(void) ;
    void leaveClickLocationMode(void) ;
    void clearLocation(void) ;
    
public slots:
    void emitPlaySignal(void)  { emit play() ; }
    void emitPauseSignal(void) { emit pause() ; }
    void emitStopSignal(void)  { emit stop() ; }
    
    void emitCellVerbosityModeChanged(int state) {
	switch(state) {
	case Qt::Unchecked:
	    emit hideSubdivisionCells() ;
	    break ;
	case Qt::Checked:
       	    emit showSubdivisionCells() ;
	    break ;
	default:
	    break ;
	}
    }

    void emitClickLocationMode(int state) {
	switch(state) {
	case Qt::Unchecked:
	    emit leaveClickLocationMode() ;
	    break ;
	case Qt::Checked:
       	    emit enterClickLocationMode() ;
	    break ;
	default:
	    break ;
	}
    }

    void emitClearLocation(void) {
	emit clearLocation() ;
    }

    
private:
    Ui::ParametricCurvesArrangementWidget ui ;
} ;

class ParametricCurvesArrangementDock : public QDockWidget 
{
    Q_OBJECT       
    
public:
    ParametricCurvesArrangementDock(const QString & title, QWidget * parent) ;
							     
public slots:
    void enterClickLocationMode(void) ;
    void leaveClickLocationMode(void) ;
    void clickLocation(const QPoint & p) ;
    void clearLocation(void) ;
    void showSubdivisionCells(void) ;
    void hideSubdivisionCells(void) ;

private:
    QParametricCurvesArrangement * algo ;
				      
private slots:
    void play(void) ;
    void pause(void) ;
    void stop(void) ;
    
private:
    void setupUi(void) ;
    
private:
    MainWindow * mainwindow ;
    ParametricCurvesArrangementWidget * acw ;
    QScrollArea  * scrollArea ;
    QHBoxLayout  * hboxLayout ;
    QSplitter    * splitter ;
    QGraphWidget * graphWidget ;
    QGraphWidget * quad3Widget ;
    QLabel       * deamonPixmap ;

private:
    QList<QPoint3f *> locatedPoints ;
} ;

# endif // PARAMETRICCURVESARRANGEMENTDOCK_H
