/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QParameterSpaceView.h
 * 2007-06-13
 * Julien Wintz & Stephane Chau
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# ifndef QPARAMETERSPACEVIEW_H
# define QPARAMETERSPACEVIEW_H

# include <QBrush>
# include <QFont>
# include <QImage>
# include <QPen>
# include <QImage>

# include <QAxel/QPoint3f.h>
# include <QAxel/QAxelObject.h>

# include <QGLViewer/qglviewer.h>

using namespace QAxel;
using namespace qglviewer;

class QParameterSpaceView : public QGLViewer
{
    Q_OBJECT
    Q_OVERRIDE(QString objectName SCRIPTABLE true)
	
public:
    QParameterSpaceView(QAxelObject * object, QWidget *parent);

public:
    virtual void init();
    virtual void draw();
    virtual void drawWithNames();

    virtual void endSelection(const QPoint&);
    virtual void keyPressEvent(QKeyEvent *e) ;
    virtual void mousePressEvent(QMouseEvent *e) ;
    virtual void mouseMoveEvent(QMouseEvent *e) ;
    virtual void mouseReleaseEvent(QMouseEvent *e) ;
    virtual void wheelEvent(QWheelEvent *e) ;
	
    void drawGrid(float width, float height, int nbSubdivisions=10) ;

    QPoint3f * convertCoordinatesToPoint(int x, int y) ; // Point in parameter space R(u,v)
    QPoint3f * convertClickToPoint(QMouseEvent * e) ;    // Point in parameter space R(u,v)
    QPoint3f * evaluateClick(float u, float v) ;         // Point is euclidean space R[x,y,z]

signals:
    void parameterClicked(float u, float v) ;
    void evaluation(QAxelObject *) ;
    void updated(void) ;

private:
    QAxelObject * m_object ; 
    QPoint3f    * m_point ;
    float m_width ;
    float m_height ;
    int m_nbSubdivisions ;
    
private:
    CameraConstraint * cameraConstraint ;
} ;

# endif
