/*****************************************************************************
 * A x e l
 *****************************************************************************
 * ScriptingDock
 * 2006-05-23
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# include <QPushButton>
# include <QTextEdit>
# include <QFile>
# include <QFileDialog>
# include <QMessageBox>
# include <QTextStream>

# include <qsproject.h>
# include <qsinterpreter.h>
# include <qsscript.h>
# include <qseditor.h>

# include <QScript/QAxelScriptFactory.h>
# include <QScript/QGuiScriptFactory.h>

# include <QGui/Viewer.h>
# include <QGui/ScriptingDock.h>

using namespace std ;

ScriptingDock::ScriptingDock(Viewer * viewer, const QString & title, QWidget * parent) : QDockWidget(title, parent) 
{
    this->viewer = viewer ;
    
    project = new QSProject(this, "AXELScript" );
    project->addObject(viewer) ;
    project->addObject(parent) ;
    viewer->camera()->setObjectName("Camera") ;
    project->addObject(viewer->camera()) ;
    
    interpreter = project->interpreter();
    interpreter->addObjectFactory(new QAxelScriptFactory) ;
    interpreter->addObjectFactory(new QGuiScriptFactory) ;
    
    script = project->createScript("default") ;
    editor = project->createEditor(script) ;
    
    setWidget(editor);
}

void ScriptingDock::load(QString s)
{
    QString filename ;
    if(!s.isEmpty()) filename = s ;
    else  filename = QFileDialog::getOpenFileName(this, "Choose an Axel script", ".", "AXS scripts (*.axs)") ;
    
    QFile * file = new QFile(filename) ;
    
    if( !file->open(QIODevice::ReadOnly | QIODevice::Text) )  {
	QMessageBox::information(0, "Axel","Error of reading","&Ok",0) ;
	return ;
    }
	
    QTextStream in(file);
    editor->setText(in.readAll()) ;
    
    file->close() ;
}

void ScriptingDock::setText(QString text)
{
    editor->setText(text) ;
}

QString ScriptingDock::text(void)
{
    return editor->text() ;
}

void ScriptingDock::save(void) 
{
    cout << "Script : save" << endl ;
}

void ScriptingDock::saveAs(void)
{
    cout << "Script : save as" << endl ;
}

void ScriptingDock::run(void)
{
    interpreter->evaluate(editor->text()) ;
    viewer->updateGL() ;
}

void ScriptingDock::addObject(QAxelObject * o)
{
    project->addObject((QObject *)o) ;
}


void ScriptingDock::addObject(QObject * o)
{
    project->addObject(o) ;
}
