/*****************************************************************************
 * A x e l
 *****************************************************************************
 * ToolPropertiesWidgets
 * 2006-09-28
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# ifndef TOOLPROPERTIESWIDGETS_H
# define TOOLPROPERTIESWIDGETS_H

# include <QAxel/QBoundingBox.h>
# include <QAxel/QCurve.h>
# include <QAxel/QSurface.h>

# include <QIntersection/Curves/QBezierClipper.h>

# include "ui_PointToolWidget.h"
# include "ui_PlaneToolWidget.h"
# include "ui_CurveToolWidget.h"
# include "ui_SurfaceToolWidget.h"
# include "ui_ConvertToMeshToolWidget.h"
# include "ui_InterpolationBSplineCurveToolWidget.h"
# include "ui_ApproximationBSplineCurveToolWidget.h"
# include "ui_ApproximationBSplineSurfaceToolWidget.h"
# include "ui_JoinBSplineCurveToolWidget.h"
# include "ui_DistanceConstraintWidget.h"
# include "ui_IncidenceConstraintWidget.h"
# include "ui_AngleConstraintWidget.h"
# include "ui_CurveArrangementWidget.h"
# include "ui_FirstDerivativeWidget.h"
# include "ui_SecondDerivativeWidget.h"
# include "ui_JacobiCurveWidget.h"
# include "ui_HessianCurveWidget.h"
# include "ui_GraphedSurfaceWidget.h"
# include "ui_IcIntersectionWidget.h"
# include "ui_PcIntersectionWidget.h"
# include "ui_IsIntersectionWidget.h"
# include "ui_PsIntersectionWidget.h"
# include "ui_YafrayRenderWidget.h"
# include "ui_SurfRenderWidget.h"
# include "ui_NoToolWidget.h"
# include "ui_BezierClipperToolWidget.h"

using namespace QAxel ;

class MainWindow ;

class NoToolWidget : public QWidget
{
  Q_OBJECT

public:
  NoToolWidget(MainWindow * w, QWidget * parent) ;

private:
 MainWindow * mw ;
 Ui::NoToolWidget ui ;
} ;

class PointToolWidget : public QWidget 
{
	Q_OBJECT
			
public :
	PointToolWidget(MainWindow * w, QWidget * parent) ;

public slots :
	void create(void) ;
	
private :
	MainWindow * mw ;
	Ui::PointToolWidget ui ;
} ;

class PlaneToolWidget : public QWidget 
{
	Q_OBJECT
		
public :
	PlaneToolWidget(MainWindow * w, QWidget * parent) ;

public slots :
	void create(void) ;

private :
	MainWindow * mw ;          
	Ui::PlaneToolWidget ui ;	
} ;

class CurveToolWidget : public QWidget
{
	Q_OBJECT

public :
	CurveToolWidget(MainWindow * mw, QWidget * parent) ;
       ~CurveToolWidget(void);
		
public slots :
	void create(void) ;

private :
	QBoundingBox * bb ;
	QImplicitCurve::Method method ;
	MainWindow * mw ;
	Ui::CurveToolWidget ui ;

} ;

class SurfaceToolWidget : public QWidget 
{
	Q_OBJECT

public :
	SurfaceToolWidget(MainWindow * mw, QWidget * parent) ;
	
public slots :
	void create(void) ;
	
private :
	QBoundingBox * bb ;
	MainWindow * mainwindow ;
	Ui::SurfaceToolWidget ui ;		
} ;

class ConvertToMeshToolWidget : public QWidget 
{
	Q_OBJECT

public :
	ConvertToMeshToolWidget(MainWindow * mw, QWidget * parent) ;
	
public slots :
	void convert(void) ;

private :
	MainWindow * mainwindow ;
	Ui::ConvertToMeshToolWidget ui ;		
} ;


class InterpolationBSplineCurveToolWidget : public QWidget
{
	Q_OBJECT

public :
	InterpolationBSplineCurveToolWidget(MainWindow * mw, QWidget * parent) ;
		
public slots :
        void compute(void) ;

private :
	QBoundingBox * bb ;
	QImplicitCurve::Method method ;
	MainWindow * mw ;
	Ui::InterpolationBSplineCurveToolWidget ui ;

public :
        QList<QPoint3f *> points ;
        QBSplineCurve * curve ;

} ;


class ApproximationBSplineCurveToolWidget : public QWidget
{
	Q_OBJECT

public :
	ApproximationBSplineCurveToolWidget(MainWindow * mw, QWidget * parent) ;
		
public slots :
        void compute(void) ;

private :
	QBoundingBox * bb ;
	QImplicitCurve::Method method ;
	MainWindow * mw ;
	Ui::ApproximationBSplineCurveToolWidget ui ;

} ;
class ApproximationBSplineSurfaceToolWidget : public QWidget
{
	Q_OBJECT

public :
	ApproximationBSplineSurfaceToolWidget(MainWindow * mw, QWidget * parent) ;
		
public slots :
        void compute(void) ;

private :
	QBoundingBox * bb ;
	MainWindow * mw ;
	Ui::ApproximationBSplineSurfaceToolWidget ui ;

} ;


class JoinBSplineCurveToolWidget : public QWidget
{
	Q_OBJECT

public :
	JoinBSplineCurveToolWidget(MainWindow * mw, QWidget * parent) ;
		
public slots :
        void compute(void) ;

private :
	QBoundingBox * bb ;
	QImplicitCurve::Method method ;
	MainWindow * mw ;
	Ui::JoinBSplineCurveToolWidget ui ;

} ;


class DistanceConstraintWidget : public QWidget
{
	Q_OBJECT
	
public :
	DistanceConstraintWidget(MainWindow * mv, QWidget * parent);

public slots :
	void create(void);
	void select(void);
	void changed(void);
	
private :
	MainWindow * mainwindow ;
	Ui::DistanceConstraintWidget ui;
};

class IncidenceConstraintWidget : public QWidget
{
	Q_OBJECT
	
public :
	IncidenceConstraintWidget(MainWindow * mw, QWidget * parent);

public slots :
	void create(void) ;
	void select(void) ;
	void changed(void) ; 

private :
	MainWindow * mainwindow ; 
	Ui::IncidenceConstraintWidget ui ;
};

class AngleConstraintWidget : public QWidget
{
	Q_OBJECT
	
public :
	AngleConstraintWidget(MainWindow * mw, QWidget * parent);

public slots :
	void create(void) ;
	void select(void) ;
	void changed(void) ; 

private :
	MainWindow * mainwindow ; 
	Ui::AngleConstraintWidget ui ;
};

class CurveArrangementWidget : public QWidget
{
	Q_OBJECT
	
public:
	CurveArrangementWidget(MainWindow * mw, QWidget * parent) ;

public slots: 
	void launch(void) ;

private:
	MainWindow * mainwindow ;
	Ui::CurveArrangementWidget ui ;	
} ;

class FirstDerivativeWidget : public QWidget
{
	Q_OBJECT
	
public:
	FirstDerivativeWidget(MainWindow * mw, QWidget * parent) ;
	
public slots:
	void compute(void) ;
	
private:
	MainWindow * mainwindow ;
	Ui::FirstDerivativeWidget ui ;
} ;

class SecondDerivativeWidget : public QWidget
{
	Q_OBJECT
	
public:
	SecondDerivativeWidget(MainWindow * mw, QWidget * parent) ;
	
public slots:
	void compute(void) ;
	
private:
	MainWindow * mainwindow ;
	Ui::SecondDerivativeWidget ui ;
} ;

class JacobiCurveWidget : public QWidget
{
	Q_OBJECT
	
public:
	JacobiCurveWidget(MainWindow * mw, QWidget * parent) ;
	
public slots:
	void compute(void) ;
	
private:
	MainWindow * mainwindow ;
	Ui::JacobiCurveWidget ui ;
} ;

class HessianCurveWidget : public QWidget
{
	Q_OBJECT
	
public:
	HessianCurveWidget(MainWindow * mw, QWidget * parent) ;
	
public slots:
	void compute(void) ;
	
private:
	MainWindow * mainwindow ;
	Ui::HessianCurveWidget ui ;
} ;

class GraphedSurfaceWidget : public QWidget
{
	Q_OBJECT

public:
	GraphedSurfaceWidget(MainWindow * mw, QWidget * parent) ;
	
public slots:
	void create(void) ;
	
private:
	MainWindow * mainwindow ;
	Ui::GraphedSurfaceWidget ui ;
} ;

class IcIntersectionWidget : public QWidget
{
	Q_OBJECT

public:
	IcIntersectionWidget(MainWindow * mw, QWidget * parent) ;

public slots:
	void compute(void) ;
	
private:
	MainWindow * mainwindow ;
	Ui::IcIntersectionWidget ui ;
} ;

class PcIntersectionWidget : public QWidget
{
	Q_OBJECT

public:
	PcIntersectionWidget(MainWindow * mw, QWidget * parent) ;

public slots:
	void compute(void) ;
	
private:
	MainWindow * mainwindow ;
	Ui::PcIntersectionWidget ui ;
} ;


class IsIntersectionWidget : public QWidget
{
	Q_OBJECT

public:
	IsIntersectionWidget(MainWindow * mw, QWidget * parent) ;

public slots:
	void compute(void) ;
	
private:
	MainWindow * mainwindow ;
	Ui::IsIntersectionWidget ui ;
} ;

class PsIntersectionWidget : public QWidget
{
	Q_OBJECT

public:
	PsIntersectionWidget(MainWindow * mw, QWidget * parent) ;

public slots:
	void compute(void) ;
	
private:
	MainWindow * mainwindow ;
	Ui::PsIntersectionWidget ui ;
} ;

class YafrayRenderWidget : public QWidget
{
	Q_OBJECT

public:
	YafrayRenderWidget(MainWindow * mw, QWidget * parent) ;

public slots:
	void render(void) ;

private:
	MainWindow * mainwindow ;
	Ui::YafrayRenderWidget ui ;
} ;

class SurfRenderWidget : public QWidget
{
	Q_OBJECT

public:
	SurfRenderWidget(MainWindow * mw, QWidget * parent) ;

public slots:
	void render(void) ;

private:
	MainWindow * mainwindow ;
	Ui::SurfRenderWidget ui ;
} ;

class BezierClipperToolWidget : public QWidget
{
    Q_OBJECT

public:
    BezierClipperToolWidget(MainWindow * mw, QWidget * parent) ;
    
private:
    MainWindow * mainwindow ;
    Ui::BezierClipperToolWidget ui ;

    QBezierClipper * algo ;
} ;

# endif // TOOLPROPERTIESWIDGETS_H
