/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QBezierClipper
 * 2007-04-02
 * Julien Wintz
 * Brian Moore
 *****************************************************************************
 *               Copyright (C) 2007 INRIA Sophia-Antipolis
 *               Copyright (C) 2007 RICAM
 *****************************************************************************/

# ifndef QBEZIERCLIPPER_H
# define QBEZIERCLIPPER_H

# include <QObject>
# include <QAxel/QRationalCurve.h>
# include <QAxel/QBSplineCurve.h>
# include <QAxel/QNurbsCurve.h>
# include <QAxel/QImplicitCurve2D.h>
# include <QAxel/QImplicitCurve3D.h>
# include <QAxel/QImplicitCurve4D.h>
# include <QAxel/QRationalSurface.h>
# include <QAxel/QBSplineSurface.h>
# include <QAxel/QNurbsSurface.h>
# include <QAxel/QImplicitSurface.h>
# include <QData/QCell.h>
# include <QGui/MainWindow.h>

using namespace QAxel ;
using namespace QData ;

class QBezierClipper : public QObject
{
    Q_OBJECT

public:
    QBezierClipper(QImplicitCurve2D * c1, QImplicitCurve2D * c2, MainWindow * mainwindow) ;

public:
    float epsilon(void) { return c_epsilon ; }

public slots:
    void run(void) ;
    void linearApproximation(void) ;
    void quadraticApproximation(void) ;
    float * degreeElevationSurface(float *, int, int, int, int);
    float computeCoeff(float *, int, int, int);
    void clip(void) ;
    void setEpsilon(float epsilon) { c_epsilon = epsilon ; }
    void setEpsilon(double epsilon) { c_epsilon = epsilon ; }
    void setLinearGraphedSurfacesAreShown(bool b) { c_showLinearGraphedSurfaces = b ; }
    void setLinearOffsetSurfacesAreShown(bool b) { c_showLinearOffsetSurfaces = b ; }
    void setQuadraticGraphedSurfacesAreShown(bool b) { c_showQuadraticGraphedSurfaces = b ; }
    void setQuadraticOffsetSurfacesAreShown(bool b) { c_showQuadraticOffsetSurfaces = b ; }
    void setCriticalPointsAreShown(bool b) { c_showAllCriticalPoints = b ; }
    void setOnlyRelevantCriticalPointsAreShown(bool b) { c_showOnlyRelevantCriticalPoints = b ; }

private:
    QImplicitSurface * graph(QImplicitCurve2D * c) ;
    void computeLinearErrorBound(float * array, float * approximation, int deg_x, int deg_y) ;
    void computeQuadraticErrorBound(float * array, float * approximation, int deg_x, int deg_y) ;
    float *  mult(float * B, int deg_x, int deg_y, int nbRows, int nbColumns) ;

private:
    QImplicitCurve2D * c_c1 ; // -> p
    QImplicitCurve2D * c_c2 ; // -> q
    MainWindow * c_mw ;

private:
    float c_epsilon ;

    float c_linearErrorBound ;
    float c_quadraticErrorBound ;

    float * c_linearApproximation ;
    float * c_quadraticApproximation ;

    QImplicitCurve2D * c_p  ; // -> l^
    QImplicitCurve2D * c_p1 ;
    QImplicitCurve2D * c_p2 ;

    QImplicitCurve2D * c_q  ; // -> k^
    QImplicitCurve2D * c_q1 ;
    QImplicitCurve2D * c_q2 ;

    QImplicitSurface * c_s1 ;
    QImplicitSurface * s_p1 ;
    QImplicitSurface * s_p2 ;

    QImplicitSurface * c_s2 ;
    QImplicitSurface * s_q1 ;
    QImplicitSurface * s_q2 ;

    QCell * cell ;

    bool c_showLinearGraphedSurfaces ;
    bool c_linearGraphedSurfacesIsDisplayed;
    bool c_showLinearOffsetSurfaces ;
    bool c_linearOffsetSurfacesIsDisplayed;

    bool c_showQuadraticGraphedSurfaces ;
    bool c_quadraticGraphedSurfacesIsDisplayed;
    bool c_showQuadraticOffsetSurfaces ;
    bool c_quadraticOffsetSurfacesIsDisplayed;

    bool c_showAllCriticalPoints ;
    bool c_showOnlyRelevantCriticalPoints ;

    int c_step ;
} ;

# endif 
