/***************************************************************************
                          p5client.cpp  -  description
                             -------------------
    begin                : Thu Feb 27 2003
    copyright            : (C) 2003 by Silviu Simen
    email                : ssimen@vryad.com
 ***************************************************************************/

#include <p5sdk.h>
#include <p5client.h>
#include <stdio.h>
#include <stdlib.h>

#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/sem.h>
#include <sys/types.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>

P5SharedMem *shm_data;
int cli_num_p5 = 0;
int cli_p5_struct_size = 0;
int semaphoreid;

void sem_P (int semid)
{
  struct sembuf psembuf;
  psembuf.sem_num = 0;
  psembuf.sem_op = -1;
  psembuf.sem_flg = 0;
  semop (semid, &psembuf, 1);
}

void sem_V (int semid)
{
  struct sembuf vsembuf;
  vsembuf.sem_num = 0;
  vsembuf.sem_op = 1;
  vsembuf.sem_flg = 0;
  semop (semid, &vsembuf, 1);
}

void cli_Close ()
{
  shmdt (shm_data);
}

int cli_NumP5 ()
{
  return cli_num_p5;
}

void cli_Init ()
{
  key_t p5key = ftok (SHM_FILE, SHM_PROJ);
  if (p5key == -1)
  {
    printf ("Cannot get shared memory key !\n");
    exit (0);
  }

  int sharedmemid = shmget (p5key, 0, 0);
  if (sharedmemid == -1)
  {
    printf ("Cannot get shared memory id.\n");
    printf ("Is the p5server running ?\n");
    exit (0);
  }

  shm_data = (P5SharedMem *) shmat (sharedmemid, NULL, 0);
  if ((int) shm_data == -1)
  {
    printf ("Error attaching shared memory block !\n");
    exit (0);
  }

  semaphoreid = semget (p5key, 1, 0);
  if (semaphoreid == -1)
  {
    printf ("Cannot get semaphore !\n");
    exit (0);
  }

  sem_P (semaphoreid);
  cli_num_p5 = shm_data->nump5s;
  cli_p5_struct_size = shm_data->p5datasize;
  sem_V (semaphoreid);
}

void cli_GetP5Data (int j, P5Data * p5data)
{
  sem_P (semaphoreid);
  //Cannot simply use     *p5data = shm_data->p[j];
  // because size of P5Data might be larger on server in future versions
  memcpy (p5data, (unsigned char*)(shm_data->p)+ j*cli_p5_struct_size, sizeof(P5Data));
  sem_V (semaphoreid);
}
