/*****************************************************************************
 * A x e l
 *****************************************************************************
 * OStreamGcml
 * 2006-07-04
 * Arnaud Fabre
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *               Copyright (C) 2006 LSIIT - UMR CNRS-ULP - Strasbourg
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# include <iostream>

# include <QFile>
# include <QLabel>
# include <QStringList>
# include <QProgressDialog>
# include <QApplication>
# include <QMessageBox>
# include <QTextStream>

# include <QDomDocument>
# include <QDomNodeList>
# include <QDomElement>

# include <QAxel/QPoint3f.h>
# include <QAxel/QPlane.h>
# include <QAxel/QConstraintDistance.h>
# include <QAxel/QConstraintIncidence.h>

# include <QGui/Viewer.h>
# include <QStream/OStream/OStreamGcml.h>

using namespace std ;
using namespace qglviewer ;

namespace QStream {

OStreamGcml::OStreamGcml(QString s, Viewer * v) 
{
	filename = s ;
	viewer = v ;
	
	domdocument = new QDomDocument() ;
}

void OStreamGcml::run(void)
{
// 	std::cerr << "Gcml export" << std::endl;
	
	QList<QAxelObject *> points = viewer->listObjectsFromType("Point3f");
	QString valPoints;
	QString sPoints;
	if(points.size())
		sPoints = "point";
	for(QList<QAxelObject *>::iterator itp = points.begin(); itp != points.end(); itp++)
	{
		Vec v = (*itp)->frame()->position();
		
		QString x,y,z;
		valPoints += (*itp)->name() + "=mkPoint(" + x.setNum(v.x) + "," + y.setNum(v.y) + "," + z.setNum(v.z) + ")\n";
// 		std::cerr << (*itp)->name().toStdString() << "=mkPoint(" << v.x << "," << v.y << "," << v.z << ")" << std::endl;
		
		sPoints += " " + (*itp)->name();
	}
	
// 	std::cerr << sPoints.toStdString() << std::endl;
	
	QList<QAxelObject *> planes = viewer->listObjectsFromType("Plane");
	QString valPlanes;
	QString sPlanes;
	if (planes.size())
		sPlanes = "plane";
	QString sIncidences;
	for(QList<QAxelObject *>::iterator itp = planes.begin(); itp != planes.end(); itp++)
	{
		QPlane *p = dynamic_cast<QPlane *> (*itp);
		
		QString a,b,c,d;
		valPlanes += p->name() + "=mkPlane(" + a.setNum(p->a()) + "," + b.setNum(p->b()) + "," + c.setNum(p->c()) + "," + d.setNum(p->d()) + ")\n";
// 		std::cerr << p->name().toStdString() << "=mkPlane(" << p->a() << "," << p->b() << "," << p->c() << "," << p->d() << ")" << std::endl;
		
		sIncidences += "norm(" + p->name() + ")\n";
		sPlanes += " " + p->name();
	}
	
// 	std::cerr << sPlanes.toStdString() << std::endl;
	
	QList<QAxelObject *> distances = viewer->listObjectsFromType("ConstraintDistance");
	QString valMeasures;
	QString skeMeasures;
	QString sMeasures;
	if(distances.size())
		sMeasures = "measure";
	QString sDistances;
	for(QList<QAxelObject *>::iterator itp = distances.begin(); itp != distances.end(); itp++)
	{
		QConstraintDistance *c = dynamic_cast<QConstraintDistance *> (*itp);
		
		sDistances += "distpp(" + c->getObject(0)->name() + "," + c->getObject(1)->name() + "," + c->name() + ")\n";
// 		std::cerr << "distpp("<< c->getObject(0)->name().toStdString() << "," << c->getObject(1)->name().toStdString() << "," << c->name().toStdString() << ")" << std::endl;
		
		QString v1,v2;
		valMeasures += c->name() + "=mkMeasure(" + v1.setNum(c->getValue()) + ")\n";
		skeMeasures += c->name() + "=mkMeasure(" + v2.setNum(c->computeDistance()) + ")\n";
				
		sMeasures += " " + c->name();
	}
	
// 	std::cerr << sMeasures.toStdString() << std::endl;
	
	QList<QAxelObject *> incidences = viewer->listObjectsFromType("ConstraintIncidence");
	
	for(QList<QAxelObject *>::iterator itp = incidences.begin(); itp != incidences.end(); itp++)
	{
		QConstraint *c = dynamic_cast<QConstraint *> (*itp);
		
		sIncidences += "onP(" + c->getObject(0)->name() + "," + c->getObject(1)->name() + ")\n";
// 		std::cerr << "onP(" << c->getObject(0)->name().toStdString() << "," << c->getObject(1)->name().toStdString() << ")" << std::endl;
	}
	
	_unknowns = "\n" + sPoints + "\n" + sPlanes + "\n";
	_parameters = "\n" +sMeasures;
	_constraints = "\n" +sDistances + sIncidences;
	_sketch = "\n" + skeMeasures + valPoints + valPlanes;
	_valuation = "\n" + valMeasures;
	
	buildDomTree();
	save();
}

void OStreamGcml::buildDomTree(void)
{
	QDomElement gcml = domdocument->createElement("gcml");
	domdocument->appendChild(gcml);

	QDomElement universe = domdocument->createElement("universe");
	gcml.appendChild(universe);

	QDomElement syntax = domdocument->createElement("syntax");
	universe.appendChild(syntax);
	syntax.setAttribute("ref","atomic3d_syntax.gcml");
	
	QDomElement semantic = domdocument->createElement("semantic");
	universe.appendChild(semantic);
	semantic.setAttribute("ref","atomic3d_semantic.gcml");
	
	QDomElement gcs = domdocument->createElement("gcs");
	gcml.appendChild(gcs);
	
	QDomElement gcssyntax = domdocument->createElement("syntax");
	gcs.appendChild(gcssyntax);
	
	QDomElement unknowns = domdocument->createElement("unknowns");
	gcssyntax.appendChild(unknowns);
	
	QDomText unk = domdocument->createTextNode(_unknowns);
	unknowns.appendChild(unk);
	
	QDomElement parameters = domdocument->createElement("parameters");
	gcssyntax.appendChild(parameters);
	
	QDomText par = domdocument->createTextNode(_parameters);
	parameters.appendChild(par);
	
	QDomElement constraints = domdocument->createElement("constraints");
	gcssyntax.appendChild(constraints);
	
	QDomText con = domdocument->createTextNode(_constraints);
	constraints.appendChild(con);
	
	QDomElement gcssemantic = domdocument->createElement("semantic");
	gcs.appendChild(gcssemantic);
	
	QDomElement valuation = domdocument->createElement("valuation");
	gcssemantic.appendChild(valuation);
	
	QDomText val = domdocument->createTextNode(_valuation);
	valuation.appendChild(val);
	
	QDomElement sketch = domdocument->createElement("sketch");
	gcssemantic.appendChild(sketch);
	
	QDomText ske = domdocument->createTextNode(_sketch);
	sketch.appendChild(ske);
}

void OStreamGcml::save()
{
	QFile file(filename);
	if( !file.open(QIODevice::WriteOnly | QIODevice::Text) )  
	{
		QMessageBox::information(0, "Axel","Error of reading","&Ok",0) ;
		return ;
	}
	
	QTextStream ts( &file );
	ts << domdocument->toString();
	file.close();
}

} ;
