/*****************************************************************************
 * A x e l
 *****************************************************************************
 * 0StreamSurf
 * 2007-02-07
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# include <cmath>
# include <iostream>

# include <QFile>
# include <QLabel>
# include <QColor>
# include <QStringList>
# include <QProgressDialog>
# include <QApplication>
# include <QMessageBox>
# include <QTextStream>
# include <QDomDocument>
# include <QDomNodeList>
# include <QDomElement>

# include <QGui/Viewer.h>
# include <QAxel/QFace.h>
# include <QAxel/QImplicitCurve2D.h>

# include <QAxel/QRationalSurface.h>
# include <QAxel/QBSplineSurface.h>
# include <QAxel/QNurbsSurface.h>
# include <QAxel/QImplicitSurface.h>
# include <QStream/OStream/OStreamSurf.h>

using namespace std ;
using namespace qglviewer ;

namespace QStream {

OStreamSurf::OStreamSurf(QString s, Viewer * v) 
{
	filename = s ;
	viewer = v ;
	
	// Default parameters
	outputWidth         = 800 ;
	outputHeight        = 600 ;
	backgroundColor     = Qt::white ;
	surfaceOutsideColor = Qt::yellow ;
	surfaceInsideColor  = Qt::red ;
}

void OStreamSurf::run(void)
{	
	content  = "// Generated by Axel::OStreamSurf\n\n" ;
	
	setUpImage() ;
	setUpLights() ;
	setUpScene() ;
	
	int current = 1 ;
	
	QList<QAxelObject *> isurfaces = viewer->listObjectsFromType("ImplicitSurface");
	foreach(QAxelObject * o, isurfaces) {
		QImplicitSurface * s = dynamic_cast<QImplicitSurface *>(o) ;
		QString number = "" ;
		if(current > 1) number.setNum(current) ;
		float rot_x = 0.0, rot_y = 0.0, rot_z = 0.0 ; float m[3][3] ;
		s->frame()->orientation().normalized().getRotationMatrix(m) ;
		getAngles(m, &rot_x, &rot_y, &rot_z) ;
		content += "surface" + number + " = " + s->equation() + ";\n";
		content += "surface" + number + " = rotate(surface" + number + ", 0.0, yAxis);\n" ;
		content += "surface" + number + " = rotate(surface" + number + ", 0.0, xAxis);\n" ;
		content += "surface" + number + " = rotate(surface" + number + ", 0.0, zAxis);\n" ;
		content += "surface" + number + " = rotate(surface" + number + ", " + num.setNum(rot_y, 'f', 3) + ", yAxis);\n" ;
		content += "surface" + number + " = rotate(surface" + number + ", " + num.setNum(rot_x, 'f', 3) + ", xAxis);\n" ;
		content += "surface" + number + " = rotate(surface" + number + ", " + num.setNum(rot_z, 'f', 3) + ", zAxis);\n" ;
		content += "surface" + number + "_red   = "  + num.setNum(surfaceOutsideColor.red())   + ";\n" ;
		content += "surface" + number + "_green = "  + num.setNum(surfaceOutsideColor.green()) + ";\n" ;
		content += "surface" + number + "_blue  = "  + num.setNum(surfaceOutsideColor.blue())  + ";\n" ;
		content += "inside"  + number + "_red    = " + num.setNum(surfaceInsideColor.red())    + ";\n" ;
		content += "inside"  + number + "_green  = " + num.setNum(surfaceInsideColor.green())  + ";\n" ;
		content += "inside"  + number + "_blue   = " + num.setNum(surfaceInsideColor.blue())   + ";\n" ;
//		content += "clip = ball;\n" ;
//		content += "radius   = " + num.setNum(s->boundingbox()->size()/2) + ";\n" ;
//		content += "center_x = " + num.setNum(s->frame()->position().x)   + ";\n" ;
//		content += "center_y = " + num.setNum(s->frame()->position().y)   + ";\n" ;
//		content += "center_z = " + num.setNum(s->frame()->position().z)   + ";\n" ;
		content += "ambient" + number + "      = 40;\n" ;
		content += "diffuse" + number + "      = 80;\n" ;
		content += "reflected" + number + "    = 80;\n" ;
		content += "transmitted" + number + "  = 20;\n" ;
		content += "smoothness" + number + "   = 50;\n" ;
		content += "transparence" + number + " = 0;\n"  ;
		content += "thickness" + number + "    = 10;\n" ;
		current++ ;
	}
	
	int nbcurves2D = 1 ;
	QList<QAxelObject *> icurves2D = viewer->listObjectsFromType("ImplicitCurve2D");
	foreach(QAxelObject * o, icurves2D) {
		QImplicitCurve2D * c = dynamic_cast<QImplicitCurve2D *>(o) ;
		content += "curve" ;
		if(nbcurves2D > 1) content += num.setNum(nbcurves2D++) ;
		content += "=" + c->equation() + ";\n";
	}
		
	// int nbcurves3D = 1 ;
	// QList<QAxelObject *> icurves3D = viewer->listObjectsFromType("ImplicitCurve3D");
	// foreach(QAxelObject * o, icurves3D) {
	// 	QImplicitCurve3D * c = dynamic_cast<QImplicitCurve3D *>(o) ;
	// 	content += "curve" ;
	// 	if(nbsurfaces > 1) content += num.setNum(nbcurves3D++) ;
	// 	content += "=" + c->equation() + ";\n";
	// 	content += "draw_curve;\n" ;
	// }
	
	content += "\n" ;
	content += "clear_screen;\n" ;
	content += "draw_surface;\n" ;
	content += "\n" ;

	setUpOutput() ;

	save();
}

void OStreamSurf::getAngles(float m[3][3], float * rot_x, float * rot_y, float * rot_z)
{   
    (*rot_x) = asin(m[2][1]) ;
    (*rot_y) = atan2(m[2][0],m[2][2]) ;
    (*rot_z) = -atan2(m[0][1],m[1][1]) ;
}

void OStreamSurf::setUpOutput(void)
{
	content += "// Output setup\n" ;
	content += "color_file_format = ppm;\n" ;
	content += "filename = \"/tmp/axel_render.ppm\";\n" ;
	content += "save_color_image;\n" ;
}

void OStreamSurf::setUpImage(void)
{
	content += "// Image setup\n" ;
	content += "width  = " + num.setNum(outputWidth) + ";\n" ;
	content += "height = " + num.setNum(outputHeight) + ";\n" ;
	content += "\n" ;
}

void OStreamSurf::setUpLights(void)
{
	content += "// Lighting setup\n" ;
	content += "light1_x       = -100.0;\n" ;
	content += "light1_y       = 100.0;\n" ;
	content += "light1_z       = 100.0;\n" ;
	content += "light1_vol     = 50;\n" ;
	content += "light1_red     = 255;\n" ;
	content += "light1_green   = 255;\n" ;
	content += "light1_blue    = 255;\n" ;
	content += "light2_x       = 100.0;\n" ;
	content += "light2_y       = 100.0;\n" ;
	content += "light2_z       = 100.0;\n" ;
	content += "light2_vol     = 70;\n" ;
	content += "light2_red     = 255;\n" ;
	content += "light2_green   = 255;\n" ;
	content += "light2_blue    = 255;\n" ;
	content += "light3_x       = 0.0;\n" ;
	content += "light3_y       = -100.0;\n" ;
	content += "light3_z       = 100.0;\n" ;
	content += "light3_vol     = 30;\n" ;
	content += "light3_red     = 255;\n" ;
	content += "light3_green   = 255;\n" ;
	content += "light3_blue    = 255;\n" ;
	content += "light4_x       = 0.0;\n" ;
	content += "light4_y       = 0.0;\n" ;
	content += "light4_z       = 0.0;\n" ;
	content += "light4_vol     = 0;\n" ;
	content += "light4_red     = 0;\n" ;
	content += "light4_green   = 0;\n" ;
	content += "light4_blue    = 0;\n" ;
	content += "light5_x       = 0.0;\n" ;
	content += "light5_y       = 0.0;\n" ;
	content += "light5_z       = 0.0;\n" ;
	content += "light5_vol     = 0;\n" ;
	content += "light5_red     = 0;\n" ;
	content += "light5_green   = 0;\n" ;
	content += "light5_blue    = 0;\n" ;
	content += "light6_x       = 0.0;\n" ;
	content += "light6_y       = 0.0;\n" ;
	content += "light6_z       = 0.0;\n" ;
	content += "light6_vol     = 0;\n" ;
	content += "light6_red     = 0;\n" ;
	content += "light6_green   = 0;\n" ;
	content += "light6_blue    = 0;\n" ;
	content += "light7_x       = 0.0;\n" ;
	content += "light7_y       = 0.0;\n" ;
	content += "light7_z       = 0.0;\n" ;
	content += "light7_vol     = 0;\n" ;
	content += "light7_red     = 0;\n" ;
	content += "light7_green   = 0;\n" ;
	content += "light7_blue    = 0;\n" ;
	content += "light8_x       = 0.0;\n" ;
	content += "light8_y       = 0.0;\n" ;
	content += "light8_z       = 0.0;\n" ;
	content += "light8_vol     = 0;\n" ;
	content += "light8_red     = 0;\n" ;
	content += "light8_green   = 0;\n" ;
	content += "light8_blue    = 0;\n" ;
	content += "light9_x       = 0.0;\n" ;
	content += "light9_y       = 0.0;\n" ;
	content += "light9_z       = 0.0;\n" ;
	content += "light9_vol     = 0;\n" ;
	content += "light9_red     = 0;\n" ;
	content += "light9_green   = 0;\n" ;
	content += "light9_blue    = 0;\n" ;
	content += "root_finder    = d_chain_bisection; epsilon = 0.00000001;\n" ;
	content += "illumination   = ambient_light + diffuse_light + reflected_light + transmitted_light;\n" ;
	content += "\n" ;
}

void OStreamSurf::setUpScene(void)
{
	content += "// Scene setup\n" ;
	content += "antialiasing = 8;\n"  ;
	content += "\n" ;
	content += "background_red   = " + num.setNum(backgroundColor.red()) + ";\n" ;
	content += "background_green = " + num.setNum(backgroundColor.red()) + ";\n" ;
	content += "background_blue  = " + num.setNum(backgroundColor.red()) + ";\n" ;
	content += "\n" ;
	content += "first  = translate;\n" ;
	content += "second = rotate;\n" ;
	content += "third  = scale;\n" ;
	content += "\n" ;
	content += "origin_x = 0.0; \n" ;
	content += "origin_y = 0.0; \n" ;
	content += "origin_z = 0.0; \n" ;
	content += "\n" ;
	float rot_x = 0.0, rot_y = 0.0, rot_z = 0.0 ; float m[3][3] ; 
	viewer->camera()->orientation().normalized().getInverseRotationMatrix(m) ;
	getAngles(m, &rot_x, &rot_y, &rot_z) ;
	content += "rot_x = " + num.setNum(rot_x) + ";\n" ;
	content += "rot_y = " + num.setNum(rot_y) + ";\n" ;
	content += "rot_z = " + num.setNum(rot_z) + ";\n" ;
	content += "\n" ;
	float scale = viewer->camera()->distanceToSceneCenter()*0.04 ;
	content += "scale_x = " + num.setNum(scale) + ";\n" ;
	content += "scale_y = " + num.setNum(scale) + ";\n" ;
	content += "scale_z = " + num.setNum(scale) + ";\n" ;
	content += "\n" ;
	// content += "spec_z = " + num.setNum(viewer->camera()->distanceToSceneCenter()) + ";\n" ;
	// content += "\n" ;
	if(viewer->camera()->type() == Camera::ORTHOGRAPHIC) content += "perspective = parallel;\n" ;
	else content += "perspective = central;\n" ;
	content += "\n" ;
}

QString OStreamSurf::filterEquation(QString equation)
{
    QString s = equation ;
    cout << "filtering " << s.toAscii().constData() << endl ;
    QRegExp rx("([+-]*[0-9]+\\.?[0-9]*[eE]?[+-]?[0-9]*)(x|y|z)") ;
    if(s.contains(rx)) {
	cout << "found : " << rx.cap(1).toAscii().constData() << " and " << rx.cap(2).toAscii().constData() << endl ;
//	s.replace(rx, "\\1*\\2") ;
    }
    return s ;
}

void OStreamSurf::save()
{
	QFile file(filename);
	if(!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
		QMessageBox::information(0, "Axel","Error of reading","&Ok",0) ;
		return ;
	}
	
	QTextStream ts(&file);
	ts << content ;
	file.close();
}

} ;
