/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QSynapsPlugin
 * 2006-05-13
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :         
 ****************************************************************************/

# include <QtGui>

# include <QSynaps/QSynapsPlugin.h>

# include <QAxel/QSingularity.h>
# include <QAxel/QVertex.h>
# include <QAxel/QEdge.h>
# include <QAxel/QFace.h>
# include <QAxel/QMesh.h>
# include <QAxel/QCurve.h>

# include <QData/QBoundingBox4D.h>
# include <QData/QCell.h>
# include <QData/QCell4D.h>
# include <QData/QRegion3f.h>

# include <synaps/arithm/gmp.h>
# include <synaps/arithm/binomial.h>
# include <synaps/usolve/bezier/SlvBzStd.h>
# include <synaps/usolve/bezier/SlvBzBdg.h>
# include <synaps/msolve/sbdslv.h>
# include <synaps/topology/regularity.h>
# include <synaps/topology/mesh.h>
# include <synaps/topology/TopSbdBdg2d.h>
//# include <synaps/topology/TopSweep2d.h>
# include <synaps/topology/TopSbdCurve3d.h>
# include <synaps/topology/topdeg2d.h>

# include <synaps/upol/bezier/rep1d.h>
# include <synaps/usolve/bezier/sbd1d.h>
# include <QGui/color.h>

# define VARIDX(poly, idx) (idx)
# define ad(i,j) ((i)*(uintervals+1)+(j))

typedef UPolDse<double> upol_t ;
typedef MPol<double> mpol_t ;
typedef MPolDse<double, mpoldse::bernstein<double> > bpol_t ;

QString toString(mpol_t p, Variables V = "x y z")
{
    stringstream os ; MPOLDST::print(os,p, Variables(V)) ;
    string str = os.str() ;
    QString equation = QString::fromStdString(str) ;
    return equation ;
}

QString QSynapsPlugin::name(void) const
{
    return "Synaps" ;
}

QStringList QSynapsPlugin::features(void) const
{
    return QStringList() ;
}

void QSynapsPlugin::init(MainWindow * window)
{

}

QMesh * QSynapsPlugin::Mesh(QImplicitCurve2D * c) 
{
    MPol<QQ> p(c->equation().toAscii().constData(), Variables("x y")) ;
    topology::point_graph<double> g ;
    switch(c->method()) {
    case QImplicitCurve::Sweep :
//	 	topology::assign(g, p, ::SYNAPS::TopSweep2d<double>()) ;
	break ;
    case QImplicitCurve::Subdivision :
    default :
      topology::assign(g, p, ::SYNAPS::TopSbdBdg2d<double>(0.5e-3, 1e-2),
		       (double)c->boundingbox()->xmin()-0.001,
		       (double)c->boundingbox()->xmax(),
		       (double)c->boundingbox()->ymin(),
		       (double)c->boundingbox()->ymax()+0.001) ;
	break ;
    }
	
    int nVertices = g.Vertices.size() ;
    int nEdges = g.nb_edges ;
	
    QMesh * m = new QMesh ;

    m->setMeshType(QMesh::Curve) ;
    m->setNumberOfVertices(nVertices) ;
    m->setNumberOfEdges(nEdges) ;
	
    int current_index = 0 ;
    for(int i=0 ; i<g.Vertices.size() ; i++) {
	double x = g.Vertices[i][0] ;
	double y = g.Vertices[i][1] ;
	double z = 0.0 ;
		
	int index = current_index ; current_index++ ;
	QVertex * v = new QVertex(index,x,y,z,m) ;
	m->push_vertex(v) ;
    }
	
    for(int i=0; i<(int)rep(rep(g).Edges).size(); ++i) {
	for(int j=0; j<(int)rep(rep(g).Edges)[i].size(); ++j) {
	    int index1 = i ;
	    int index2 = rep(rep(g).Edges)[i][j] ;
	    QEdge * e = new QEdge(index1, index2, m) ;
	    m->push_edge(e) ;
	}
    }

    return m ;
}

QMesh * QSynapsPlugin::Mesh(QImplicitCurve3D * c)
{
    MPol<double> p1(c->equation1().toAscii().constData(), Variables("x y z")) ;
    MPol<double> p2(c->equation2().toAscii().constData(), Variables("x y z")) ;

    topology::point_graph<double> g ;
    topology::assign(g, p1, p2, ::SYNAPS::TopSbdCurve3d<double>(1e-6), 
		     (double)c->boundingbox()->xmin()-0.001, 
		     (double)c->boundingbox()->xmax()+0.001,
		     (double)c->boundingbox()->ymin()-0.001,
		     (double)c->boundingbox()->ymax()+0.001,
		     (double)c->boundingbox()->zmin()-0.001,
		     (double)c->boundingbox()->zmax()+0.001) ;
	
    int nVertices = g.Vertices.size() ;
    int nEdges = g.nb_edges ;
	
	
    QMesh * m = new QMesh ;

    m->setMeshType(QMesh::Curve) ;
    m->setNumberOfVertices(nVertices) ;
    m->setNumberOfEdges(nEdges) ;
	
    int current_index = 0 ;
    for(int i=0 ; i<g.Vertices.size() ; i++) {
	double x = g.Vertices[i][0] ;
	double y = g.Vertices[i][1] ;
	double z = g.Vertices[i][2] ;
		
	int index = current_index ; current_index++ ;
	QVertex * v = new QVertex(index,x,y,z,m) ;
	m->push_vertex(v) ;
    }
	
    for(int i=0; i<(int)rep(rep(g).Edges).size(); ++i) {
	for(int j=0; j<(int)rep(rep(g).Edges)[i].size(); ++j) {
	    int index1 = i ;
	    int index2 = rep(rep(g).Edges)[i][j] ;
	    QEdge * e = new QEdge(index1, index2, m) ;
	    m->push_edge(e) ;
	}
    }

    return m ;
}

extern MPol<double> poly ;

QMesh * QSynapsPlugin::Mesh(QImplicitSurface * s) 
{
    scene<MPol<double>, linalg::rep3d<double> > scn;
    MPol<double> p(s->equation().toAscii().constData(), Variables("x y z")) ;
	
    s->setEquation(toString(p)); // Then we get the right notation for the polynomials

    poly = p ;
	
    mesh(scn, p, 
	 (double)s->boundingbox()->xmin()-0.001,
	 (double)s->boundingbox()->xmax()+0.001,
	 (double)s->boundingbox()->ymin()-0.001,
	 (double)s->boundingbox()->ymax()+0.001,
	 (double)s->boundingbox()->zmin()-0.001,
	 (double)s->boundingbox()->zmax()+0.001,
	 pow(0.5, 5), pow(0.5, 7));
	
    int nVertices = scn.listVertices.size() ;
    int nFaces = scn.listTriangles.size() ;
	
    QMesh * m = new QMesh ;

    m->setMeshType(QMesh::Surface) ;
    m->setNumberOfVertices(nVertices) ;
    m->setNumberOfFaces(nFaces) ;
	
    for ( int i = 0 ; i < nVertices ; i++ ) {
	QVertex * v = new QVertex(
	    i, 
	    scn.x+scn.u*scn.listVertices[i]->point[0], 
	    scn.y+scn.v*scn.listVertices[i]->point[1], 
	    scn.z+scn.w*scn.listVertices[i]->point[2], m) ;
	m->push_vertex(v) ;
    }
	
    for ( int i = 0 ; i < nFaces ; i++ ) {
	QFace * f = new QFace(i,m) ;
	f->push_index(scn.listTriangles[i].a->index) ;
	f->push_index(scn.listTriangles[i].b->index) ;
	f->push_index(scn.listTriangles[i].c->index) ;
	m->push_face(f) ;
		
// 		// Computing edges
// 		QEdge * e1 = new QEdge(f->indexes_[0], f->indexes_[1], m) ;
// 		QEdge * e2 = new QEdge(f->indexes_[1], f->indexes_[2], m) ;
// 		QEdge * e3 = new QEdge(f->indexes_[2], f->indexes_[0], m) ;
// 		m->push_edge(e1) ;
// 		m->push_edge(e2) ;
// 		m->push_edge(e3) ;   
    }
	
    // Computing normals
    MPol<double> dx = diff(p, "x") ;
    MPol<double> dy = diff(p, "y") ;
    MPol<double> dz = diff(p, "z") ;
	
    for ( int i = 0 ; i < nVertices ; i++ ) {
	double * v = m->getVertex(i) ;
	double dxv = dx(v) ;
	double dyv = dy(v) ;
	double dzv = dz(v) ; 
	double div = sqrt(dxv*dxv + dyv*dyv + dzv*dzv) ;
	m->setVertexNormal(i, dxv/div, dyv/div, dzv/div) ;
    }
	
    if  (!scn.SingCells.empty() ) {
	int size = (int) scn.SingCells.size();

	for ( int i= 0 ; i < size-1 ; i = i + 2  ) {
	    double x0 = scn.x+scn.u*scn.SingCells[i][0] ;
	    double y0 = scn.y+scn.v*scn.SingCells[i][1] ;
	    double z0 = scn.z+scn.w*scn.SingCells[i][2] ;

	    double x1 = scn.x+scn.u*scn.SingCells[i+1][0] ;
	    double y1 = scn.y+scn.v*scn.SingCells[i+1][1] ;
	    double z1 = scn.z+scn.w*scn.SingCells[i+1][2] ;
			
	    double x = (x0+x1)/2 ;
	    double y = (y0+y1)/2 ;
	    double z = (z0+z1)/2 ;
	    double radius = z1-z ;
			
	    QSingularity * sing = new QSingularity(x, y, z, radius, s) ;
	    s->push_singularity(sing) ;
	}
    }

    return m ;
}

QMesh * QSynapsPlugin::Mesh(QRationalCurve * c)
{
    UPolDse<double> px(c->px().toAscii().constData()) ;
    UPolDse<double> py(c->py().toAscii().constData()) ;
    UPolDse<double> pz(c->pz().toAscii().constData()) ;
	
    double tmin = c->tmin(), tmax = c->tmax() ;
    int tintervals = 20 ;
    double step = (tmax - tmin)/tintervals ;
    double value = tmin ;
	
    QMesh * m = new QMesh ;
    m->clear() ;
    m->setMeshType(QMesh::Curve) ;
    int nVertices = tintervals+1 ;
    int nEdges = nVertices-1 ;
    m->setNumberOfVertices(nVertices) ;
    m->setNumberOfEdges(nEdges) ;
	
    int current_index = 0 ;
    for(unsigned i = 0 ; i <= tintervals ; ++i) {
	double x = px(value) ;
	double y = py(value) ;
	double z = pz(value) ;
	QVertex * v = new QVertex(current_index, x, y, z, m) ; m->push_vertex(v) ;
	if(current_index > 0) {
	    QEdge * e = new QEdge(current_index-1, current_index, m) ;
	    m->push_edge(e) ;
	}
	value += step ;
	current_index++ ;
    }
}

QMesh * QSynapsPlugin::Mesh(QRationalSurface * s) 
{	
    MPol<double> px(s->px().toAscii().constData(), Variables("u v")) ;
    MPol<double> py(s->py().toAscii().constData(), Variables("u v")) ;
    MPol<double> pz(s->pz().toAscii().constData(), Variables("u v")) ;
	
    double umin = s->umin(), umax = s->umax() ;
    double vmin = s->vmin(), vmax = s->vmax() ;
    int uintervals = 100 ;
    int vintervals = 100 ;
		
    double stepU = (umax - umin)/uintervals ;
    double stepV = (vmax - vmin)/vintervals ;
		
    double value[] = {umin, vmin};
	
    int nVertices = (uintervals+1)*(vintervals+1) ;
    int nFaces = 2*(uintervals)*(vintervals) ;

    QMesh * m = new QMesh ;
			
    m->setNumberOfVertices(nVertices) ;
    m->setNumberOfFaces(nFaces) ;
	
    int current_index = 0 ;
    for(unsigned i = 0 ; i <= uintervals ; ++i) {
	value[1] = vmin ;
	for(unsigned j = 0 ; j <= vintervals ; ++j) {
	    double p[] = { value[0], value[1] } ;
	    double x = px(p) ;
	    double y = py(p) ;
	    double z = pz(p) ;
	    QVertex * v = new QVertex(current_index++, x, y, z, m) ; m->push_vertex(v) ;
	    value[1]+=stepV;
	}
	value[0]+=stepU;
    }
	
    int current_face = 0 ;

    for(int i = 0 ; i < uintervals ; i++ )
	for(int j = 0 ; j < vintervals ; j++) {
	    QFace * f = new QFace(current_face++, m) ;
	    f->push_index(ad(i,j)) ;
	    f->push_index(ad(i,j+1)) ;
	    f->push_index(ad(i+1,j)) ;
	    m->push_face(f) ;
			
	    QFace * f1 = new QFace(current_face++, m) ;
	    f1->push_index(ad(i+1,j+1)) ;
	    f1->push_index(ad(i+1,j)) ;
	    f1->push_index(ad(i,j+1)) ;
	    m->push_face(f1) ;
	}
	
    m->computeNormals() ;
}

// QString QSynapsPlugin::globalEquation(QString localEquation, const GLdouble * matrix) 
// {
// 	using namespace std ;	
// // 	frame's matrix D :
// // 	R00 R01 R02 0	|	0  1  2  3
// // 	R10 R11 R12 0	|	4  5  6  7
// // 	R20 R21 R22 0	|	8  9  10  11
// // 	T00 T01 T02 1  	|	12 13 14 15  
// 	// 	
// // 	For p of coordinates X=(x,y,z)
// // 	Pl -> Pw
// // 	X  -> RX-T
// 	// 	 
// // 	So, for f(x,y,z) = f(X)
// // 	f(X) -> f(RX-T)
// 	
// 	MPol<double> p(localEquation.toAscii().constData(), Variables("x y z")) ;
// 	MPol<double> g  ;
// 
// 	double R00 = matrix[0] ; double R01 = matrix[1] ; double R02 = matrix[2] ;
// 	double R10 = matrix[4] ; double R11 = matrix[5] ; double R12 = matrix[6] ;
// 	double R20 = matrix[8] ; double R21 = matrix[9] ; double R22 = matrix[10] ;
// 	double T00 = matrix[12] ; double T01 = matrix[13] ; double T02 = matrix[14] ;
// 	
// 	typedef MPol<double> MPol_t ;
// 	
// 	MPol_t x("x") ; MPol_t y("y") ; MPol_t z("z") ;
// 	MPol_t lx(R00*x + R01*y + R02*z - T00) ;
// 	MPol_t ly(R10*x + R11*y + R12*z - T01) ;
// 	MPol_t lz(R20*x + R21*y + R22*z - T02) ;
// 	
// 	for(MPol_t::const_iterator it = p.begin() ; it != p.end() ; it++) {
// 		g += ( it->coeff() * pow(lx,((*it)[0])) * pow(ly,((*it)[1])) * pow(lz,((*it)[2])) ) ;
// 	}
// 	
// 	stringstream os ; MPOLDST::print(os,g) ; 
// 	string str = os.str() ;
// 	QString equation = QString::fromStdString(str) ;
// 
// 	return equation ;
// }

bpol_t toBernstein1D(mpol_t p, double xmin, double xmax)
{
	bpol_t tmp(p) ;
    mpoldse::restrict(tmp.rep(),VARIDX(tmp,0),xmin,xmax);
    return tmp ;
}


bpol_t toBernstein2D(mpol_t p, double xmin, double xmax, double ymin, double ymax)
{
	bpol_t tmp(p) ;
    mpoldse::restrict(tmp.rep(),VARIDX(tmp,0),xmin,xmax);
    mpoldse::restrict(tmp.rep(),VARIDX(tmp,1),ymin,ymax);
    return tmp ;
}

bpol_t toBernstein3D(mpol_t p, double xmin, double xmax, double ymin, double ymax, double zmin, double zmax) 
{
	bpol_t tmp(p) ;
    mpoldse::restrict(tmp.rep(),VARIDX(tmp,0),xmin,xmax);
    mpoldse::restrict(tmp.rep(),VARIDX(tmp,1),ymin,ymax);
    mpoldse::restrict(tmp.rep(),VARIDX(tmp,2),zmin,zmax);
    return tmp ;
}

bpol_t toBernstein4D(mpol_t p, double smin=0, double smax=1, double tmin=0, double tmax=1, double umin=0, double umax=1, double vmin=0, double vmax=1) 
{
	bpol_t tmp(p) ;
    mpoldse::restrict(tmp.rep(),VARIDX(tmp,0),smin,smax);
    mpoldse::restrict(tmp.rep(),VARIDX(tmp,1),tmin,tmax);
    mpoldse::restrict(tmp.rep(),VARIDX(tmp,2),umin,umax);
    mpoldse::restrict(tmp.rep(),VARIDX(tmp,3),vmin,vmax);
    return tmp ;
}

bpol_t toBernstein4D(mpol_t p, QBoundingBox4D * bb) 
{
	bpol_t tmp(p) ;
    mpoldse::restrict(tmp.rep(),VARIDX(tmp,0), (double) bb->smin(), (double) bb->smax()); 
    mpoldse::restrict(tmp.rep(),VARIDX(tmp,1), (double) bb->tmin(), (double) bb->tmax());
    mpoldse::restrict(tmp.rep(),VARIDX(tmp,2), (double) bb->umin(), (double) bb->umax());
    mpoldse::restrict(tmp.rep(),VARIDX(tmp,3), (double) bb->vmin(), (double) bb->vmax());
    return tmp ;
}

// Check if a Bernstein polynomial has values > epsilon
bool isPositiv(bpol_t p, double epsilon = 1e-6)
{
	const double * coeff = p.begin();
	while (coeff <= p.end())
	{
		if ( (*coeff) <= -abs(epsilon)) return false;
		else ++coeff;
	}
	return true;
}

// Check if a Bernstein polynomial has values < epsilon
bool isNegativ(bpol_t p, double epsilon = 1e-6)
{
	const double * coeff = p.begin();
	while (coeff <= p.end())
	{
		if ( (*coeff) >= abs(epsilon)) return false;
		else ++coeff;
	}
	return true;
}

// Check if a list of Bernstein polynomials are of the same sign (within a tolerance epsilon).
bool areOfSameSign(QList<bpol_t> L, double epsilon = 1e-6)
{
	QList<bpol_t>::iterator ipol = L.begin();
	bool res = true;
	if (isPositiv(*ipol,epsilon))
	{
		++ipol;
		while ( res && (ipol < L.end()) )
		{
			if (!isPositiv(*ipol, epsilon))
				res = false;
			++ipol;
		}
	}
	else if (isPositiv(*ipol,epsilon))
	{
		++ipol;
		while ( res && (ipol < L.end()) )
		{
			if (!isNegativ(*ipol,epsilon))
				res = false;
			++ipol;
		}
	}
	else
		res = false;
	
	return res;
}

/// Check if a bernstein polynomial has no root (within a tolerance epsilon).
bool hasNoRoot(bpol_t pol, double epsilon = 1e-6)
{
	const double * coeff = pol.begin();
	while( (abs(*coeff) < abs(epsilon)) && (abs(*coeff) > -abs(epsilon)) && (coeff < pol.end()) ) ++coeff;
	double lastCoeff = *coeff;
	while( ++coeff < pol.end() )
		if ( (abs(*coeff) >= abs(epsilon)) || (abs(*coeff) <= -abs(epsilon)) )
		{
			if ( lastCoeff * (*coeff) < 0 ) return false;
			lastCoeff = *coeff;
		}
	return true;
}

bool QSynapsPlugin::xmonotonic(QImplicitCurve2D * c, QCell * bb) 
{
    mpol_t p(c->equation().toAscii().constData(), Variables("x y")) ;
    bpol_t b = toBernstein2D(p, bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax()) ;
    return is_regular(b, 0) ;
}

bool QSynapsPlugin::ymonotonic(QImplicitCurve2D * c, QCell * bb)
{
    mpol_t p(c->equation().toAscii().constData(), Variables("x y")) ;
    bpol_t b = toBernstein2D(p, bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax()) ;
    return is_regular(b, 1) ;
}

QList<QPoint3f *> QSynapsPlugin::xtremPoints(QImplicitCurve2D * c, QCell * bb, double epsilon)
{
    QList<QPoint3f *> l ;
    
    mpol_t p(c->equation().toAscii().constData(), Variables("x y")) ; bpol_t bpols[2] ;
    bpols[0] = toBernstein2D(p, bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax()) ;
    // mpol_t dx = diff(p, "x") ; bpols[1] = toBernstein2D(dx, bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax()) ;
    // mpol_t dy = diff(p, "y") ; bpols[1] = toBernstein2D(dy, bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax()) ;
    diff(bpols[1], bpols[0], 1) ; //bpols[1] = toBernstein2D(dy, bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax()) ;
    
    std::list<bpol_t> eqnList ; eqnList.push_back(bpols[0]) ; eqnList.push_back(bpols[1]) ;

    SbdSolver< double, SBDSLV_RDRDL> solver(bpols, bpols+2, epsilon) ;
    std::vector<double> results ; int nSol, nVar ;
    solver.solve(results, nSol, nVar, eqnList.begin(), eqnList.end());
	
    for(int s = 0 ; s < nSol ; s++) {
	double x = results[nVar*s+0], y = results[nVar*s+1], z = 0.0 ;
	x = (bb->xmax()-bb->xmin())*(double)(results[nVar*s+0]) + bb->xmin() ;
	y = (bb->ymax()-bb->ymin())*(double)(results[nVar*s+1]) + bb->ymin() ;
	l << new QPoint3f(x, y, z) ;
    }
    
    return l ;
}

QList<QPoint3f *> QSynapsPlugin::ytremPoints(QImplicitCurve2D * c, QCell * bb, double epsilon) 
{
    QList<QPoint3f *> l ;
	
    mpol_t p(c->equation().toAscii().constData(), Variables("x y")) ; bpol_t bpols[2] ;
    bpols[0] = toBernstein2D(p, bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax()) ;
    // mpol_t dx = diff(p, "x") ; bpols[1] = toBernstein2D(dx, bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax()) ;
    // mpol_t dy = diff(p, "y") ; bpols[1] = toBernstein2D(dy, bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax()) ;
    diff(bpols[1], bpols[0], 0) ; // bpols[1] = toBernstein2D(dx, bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax()) ;	

    std::list<bpol_t> eqnList ; eqnList.push_back(bpols[0]) ; eqnList.push_back(bpols[1]) ;

    SbdSolver< double, SBDSLV_RDRDL> solver(bpols, bpols+2, epsilon) ;
    std::vector<double> results ; int nSol, nVar ;
    solver.solve(results, nSol, nVar, eqnList.begin(), eqnList.end());
	
    for(int s = 0 ; s < nSol ; s++) {
	double x = results[nVar*s+0], y = results[nVar*s+1], z = 0.0 ;
	x = (bb->xmax()-bb->xmin())*(double)(results[nVar*s+0]) + bb->xmin() ;
	y = (bb->ymax()-bb->ymin())*(double)(results[nVar*s+1]) + bb->ymin() ;
	l << new QPoint3f(x, y, z) ;
    }
	
    return l ;
}

QList<QPoint3f *> QSynapsPlugin::xtremPoints(QImplicitSurface * s, QCell * bb, double epsilon)
{
    QList<QPoint3f *> l ;
	
    mpol_t p(s->equation().toAscii().constData(), Variables("x y z")) ; bpol_t bpols[3] ;
    bpols[0] = toBernstein3D(p, bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax(), bb->zmin(), bb->zmax()) ;
    //mpol_t dy = diff(p, "y") ; bpols[1] = toBernstein3D(dy, bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax(), bb->zmin(), bb->zmax()) ;
    //mpol_t dz = diff(p, "z") ; bpols[2] = toBernstein3D(dz, bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax(), bb->zmin(), bb->zmax()) ;
    diff(bpols[1], bpols[0], 1) ; // x
    diff(bpols[2], bpols[0], 2) ; // y

    std::list<bpol_t> eqnList ; eqnList.push_back(bpols[0]) ; eqnList.push_back(bpols[1]) ; eqnList.push_back(bpols[2]) ;

    SbdSolver< double, SBDSLV_RDRDL> solver(bpols, bpols+3, epsilon) ;
    std::vector<double> results ; int nSol, nVar ;
    solver.solve(results, nSol, nVar, eqnList.begin(), eqnList.end());
	
    for(int s = 0 ; s < nSol ; s++) {
	double x = results[nVar*s+0], y = results[nVar*s+1], z = results[nVar*s+2] ;
	x = (bb->xmax()-bb->xmin())*(double)(results[nVar*s+0]) + bb->xmin() ;
	y = (bb->ymax()-bb->ymin())*(double)(results[nVar*s+1]) + bb->ymin() ;
	y = (bb->zmax()-bb->zmin())*(double)(results[nVar*s+2]) + bb->zmin() ;
	l << new QPoint3f(x, y, z) ;
    }
	
    return l ;
}

QList<QPoint3f *> QSynapsPlugin::ytremPoints(QImplicitSurface * s, QCell * bb, double epsilon)
{
    QList<QPoint3f *> l ;
	
    mpol_t p(s->equation().toAscii().constData(), Variables("x y z")) ; bpol_t bpols[3] ;
    bpols[0] = toBernstein3D(p, bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax(), bb->zmin(), bb->zmax()) ;
    // mpol_t dx = diff(p, "x") ; bpols[1] = toBernstein3D(dx, bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax(), bb->zmin(), bb->zmax()) ;
    // mpol_t dz = diff(p, "z") ; bpols[2] = toBernstein3D(dz, bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax(), bb->zmin(), bb->zmax()) ;
    diff(bpols[1], bpols[0], 0) ;
    diff(bpols[2], bpols[0], 2) ;	

    std::list<bpol_t> eqnList ; eqnList.push_back(bpols[0]) ; eqnList.push_back(bpols[1]) ; eqnList.push_back(bpols[2]) ;

    SbdSolver< double, SBDSLV_RDRDL> solver(bpols, bpols+3, epsilon) ;
    std::vector<double> results ; int nSol, nVar ;
    solver.solve(results, nSol, nVar, eqnList.begin(), eqnList.end());
	
    for(int s = 0 ; s < nSol ; s++) {
	double x = results[nVar*s+0], y = results[nVar*s+1], z = results[nVar*s+2] ;
	x = (bb->xmax()-bb->xmin())*(double)(results[nVar*s+0]) + bb->xmin() ;
	y = (bb->ymax()-bb->ymin())*(double)(results[nVar*s+1]) + bb->ymin() ;
	y = (bb->zmax()-bb->zmin())*(double)(results[nVar*s+2]) + bb->zmin() ;
	l << new QPoint3f(x, y, z) ;
    }
	
    return l ;
}

QList<QPoint3f *> QSynapsPlugin::ztremPoints(QImplicitSurface * s, QCell * bb, double epsilon)
{
    QList<QPoint3f *> l ;
	
    mpol_t p(s->equation().toAscii().constData(), Variables("x y z")) ; bpol_t bpols[3] ;
    bpols[0] = toBernstein3D(p, bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax(), bb->zmin(), bb->zmax()) ;
    // mpol_t dx = diff(p, "x") ; bpols[1] = toBernstein3D(dx, bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax(), bb->zmin(), bb->zmax()) ;
    // mpol_t dy = diff(p, "y") ; bpols[2] = toBernstein3D(dy, bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax(), bb->zmin(), bb->zmax()) ;
    diff(bpols[1], bpols[0], 0) ;
    diff(bpols[2], bpols[0], 1) ;	

    std::list<bpol_t> eqnList ; eqnList.push_back(bpols[0]) ; eqnList.push_back(bpols[1]) ; eqnList.push_back(bpols[2]) ;

    SbdSolver< double, SBDSLV_RDRDL> solver(bpols, bpols+3, epsilon) ;
    std::vector<double> results ; int nSol, nVar ;
    solver.solve(results, nSol, nVar, eqnList.begin(), eqnList.end());
	
    for(int s = 0 ; s < nSol ; s++) {
	double x = results[nVar*s+0], y = results[nVar*s+1], z = results[nVar*s+2] ;
	x = (bb->xmax()-bb->xmin())*(double)(results[nVar*s+0]) + bb->xmin() ;
	y = (bb->ymax()-bb->ymin())*(double)(results[nVar*s+1]) + bb->ymin() ;
	y = (bb->zmax()-bb->zmin())*(double)(results[nVar*s+2]) + bb->zmin() ;
	l << new QPoint3f(x, y, z) ;
    }
	
    return l ;
}

QList<QPoint3f *> QSynapsPlugin::singularPoints(QImplicitCurve2D * c, QCell * bb, double epsilon) 
{
    QList<QPoint3f *> l ;
	
    mpol_t p(c->equation().toAscii().constData(), Variables("x y")) ; bpol_t bpols[3] ;
    bpols[0] = toBernstein2D(p, bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax()) ;
    diff(bpols[1], bpols[0], 0) ; //bpols[1] = toBernstein2D(dx, bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax()) ;
    diff(bpols[2], bpols[0], 1) ; //bpols[2] = toBernstein2D(dy, bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax()) ;
	
    std::list<bpol_t> eqnList ; eqnList.push_back(bpols[0]) ; eqnList.push_back(bpols[1]) ; eqnList.push_back(bpols[2]) ;

    SbdSolver< double, SBDSLV_RDRDL> solver(bpols, bpols+3, epsilon) ;
    std::vector<double> results ; int nSol, nVar ;
    solver.solve(results, nSol, nVar, eqnList.begin(), eqnList.end());
	
    for(int s = 0 ; s < nSol ; s++) {
	// Scale the solutions down to the cell's coordinates.
	double x = results[nVar*s+0], y = results[nVar*s+1], z = 0.0 ;
	x = (bb->xmax()-bb->xmin())*(double)(results[nVar*s+0]) + bb->xmin() ;
	y = (bb->ymax()-bb->ymin())*(double)(results[nVar*s+1]) + bb->ymin() ;
		
	// cout << "RDL Solver found: (" << x << " " << y << ")" << endl ;
		
	// Since the subdivision solver gives several points according to the multiplicity
	// and we don't wanna compute the topological degree (that could fix this problem)
	// systematically, we merge the "so called" singular points given by the RDL solver
	// considering epsilon=0.01.
	// Since this computation is only for ONE curve, even if two curves' singular points
	// are distant of less than espilon, they will be both considered.
	QPoint3f * t = new QPoint3f(x, y, z) ;
	if(!l.size()) l << t ;
	else
	    foreach(QPoint3f * p, l) 
		if((abs(p->x()-t->x()) < 1e-1) && (abs(p->y()-t->y()) < 1e-1)) ;
	// cout << "to vanish" << endl ;
		else l << t ;
    }
	
    return l ;
}

QList<QPoint3f *> QSynapsPlugin::intersectionPoints(QImplicitCurve2D * c, QCell * bb, double epsilon)
{
    QList<QPoint3f *> l ;
	
    mpol_t p(c->equation().toLatin1().data(), Variables("x y")) ;
    bpol_t b = toBernstein2D(p, bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax()) ;
	
    // The 'face' function sets its first argument (a bpol_t) to be the Bernstein
    // representation on the border of the bounding box with the following encoding:
    // nAxis : [0|1|2] respectively for [x|y|z].
    // nFace : [0|1]   respectively for the first and the second (hyper)face encountered
    //                 along the corresponding axis.
    bpol_t faces[4] ;
    face(faces[0].rep(), b.rep(), 0, 0) ;
    face(faces[1].rep(), b.rep(), 0, 1) ;
    face(faces[2].rep(), b.rep(), 1, 0) ; 
    face(faces[3].rep(), b.rep(), 1, 1) ;
	
    for(int i = 0 ; i < 4 ; i++) {
	bezier::rep1d<double> bz(faces[i].rep().size(), faces[i].rep().begin()) ;
	bezier::Isole<double> slv ;
	std::vector<double> results ;
	slv.Run(bz, bz, epsilon, false) ;
	slv.GetMidPoints(results) ;
		 
	int nVars = 2 ;
	int nSol = results.size() ;
	
	for(int s = 0 ; s < nSol ; s++) {
	    double x = results[s], y = results[s], z = 0.0 ;
	    QPoint3f * pt ;
	    if(i == 0) { 
		x = bb->xmin() ; y = (bb->ymax()-bb->ymin())*y + bb->ymin() ;
		pt = new QPoint3f(x, y, z) ; pt->setRGB(255,0,0) ;
	    } else if(i == 1) {
		x = bb->xmax() ; y = (bb->ymax()-bb->ymin())*y + bb->ymin() ;
		pt = new QPoint3f(x, y, z) ; pt->setRGB(0,255,0) ;
	    } else if(i == 2) {
		x = (bb->xmax()-bb->xmin())*x + bb->xmin() ; y = bb->ymin() ;
		pt = new QPoint3f(x, y, z) ; pt->setRGB(0,0,255) ;
	    } else if(i == 3) {
		x = (bb->xmax()-bb->xmin())*x + bb->xmin() ; y = bb->ymax() ;
		pt = new QPoint3f(x, y, z) ; pt->setRGB(255,255,255) ;
	    }
	    l << pt ;
	}
    }
	
    return l ;
}

QList<QPoint3f *> QSynapsPlugin::intersectionPoints(QImplicitCurve2D * c, QCell * bb, int axis, int order, double epsilon)
{
    QList<QPoint3f *> l ;
	
    mpol_t p(c->equation().toLatin1().data(), Variables("x y")) ;
    bpol_t b = toBernstein2D(p, bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax()) ;
	
    bpol_t faces ;
    face(faces.rep(), b.rep(), axis, order) ;
	
    bezier::rep1d<double> bz(faces.rep().size(), faces.rep().begin()) ;
    bezier::Isole<double> slv ;
    std::vector<double> results ;
    slv.Run(bz, bz, epsilon, false) ;
    slv.GetMidPoints(results) ;
	 
    int nVars = 2 ;
    int nSol = results.size() ;

    for(int s = 0 ; s < nSol ; s++) {
	double x = results[s], y = results[s], z = 0.0 ;
	QPoint3f * pt ;
	if(axis == 0 && order == 0) { 
	    x = bb->xmin() ; y = (bb->ymax()-bb->ymin())*y + bb->ymin() ;
	    pt = new QPoint3f(x, y, z) ; pt->setRGB(255,0,0) ;
	} else if(axis == 0 && order == 1) {
	    x = bb->xmax() ; y = (bb->ymax()-bb->ymin())*y + bb->ymin() ;
	    pt = new QPoint3f(x, y, z) ; pt->setRGB(0,255,0) ;
	} else if(axis == 1 && order == 0) {
	    x = (bb->xmax()-bb->xmin())*x + bb->xmin() ; y = bb->ymin() ;
	    pt = new QPoint3f(x, y, z) ; pt->setRGB(0,0,255) ;
	} else if(axis == 1 && order == 1) {
	    x = (bb->xmax()-bb->xmin())*x + bb->xmin() ; y = bb->ymax() ;
	    pt = new QPoint3f(x, y, z) ; pt->setRGB(255,255,255) ;
	}
	l << pt ;
    }
	
    return l ;
}

/* Intersection points beetween the QImplicitCurve4D c and a face of the QCell4D  
	the array faceBorder codes the bounding box face. The first integer of this array represents the fixed variable and the second one the value
	of this fixed variable. For example [0,0] represents the face s = smin ; [2,1] represents the face u = umax. */

QList<QPoint4f *> QSynapsPlugin::intersectionPoints4D(QImplicitCurve4D * c, QBoundingBox4D * bb, int faceBorder[2], double epsilon)
{
	using MPOLDST::subs;
	QList<QPoint4f *> l ;
	Variables V = "s t u v";
	int fixedVariable = faceBorder[0] ;
	double varValue ;
	double xmin, xmax, ymin, ymax, zmin, zmax ; // The bounding box where we will solve the system.
	mpol_t f(c->eqf(), V) ; cout << "fInit = " << f << endl;
	mpol_t g(c->eqg(), V) ; cout << "gInit = " << g << endl;
	mpol_t h(c->eqh(), V) ; cout << "hInit = " << h << endl;
	
	// bpol_t bf = toBernstein4D(f, c->bb()) ;
	// bpol_t bg = toBernstein4D(g, c->bb()) ;
	// bpol_t bh = toBernstein4D(h, c->bb()) ;
	cout << FG_RED << __func__ << " - fixedVariable = " << fixedVariable << NOCOLOR << endl;
	switch (fixedVariable)
	{
		case 0: // variable s
			V = "t u v";
			xmin = bb->tmin(); xmax = bb->tmax();
			ymin = bb->umin(); ymax = bb->umax();
			zmin = bb->vmin(); zmax = bb->vmax();
			if (faceBorder[1] == 0) varValue = bb->smin(); else varValue = bb->smax();
		break;
		case 1: // variable t
			V = "u v s";
			xmin = bb->umin(); xmax = bb->umax();
			ymin = bb->vmin(); ymax = bb->vmax();
			zmin = bb->smin(); zmax = bb->smax();
			if (faceBorder[1] == 0) varValue = bb->tmin(); else varValue = bb->tmax();
		break;
		case 2: // variable u
			V = "v s t";
			xmin = bb->vmin(); xmax = bb->vmax();
			ymin = bb->smin(); ymax = bb->smax();
			zmin = bb->tmin(); zmax = bb->tmax();
			if (faceBorder[1] == 0) varValue = bb->umin(); else varValue = bb->umax();
		break;
		case 3: // variable v
			V = "s t u";
			xmin = bb->smin(); xmax = bb->smax();
			ymin = bb->tmin(); ymax = bb->tmax();
			zmin = bb->umin(); zmax = bb->umax();
			if (faceBorder[1] == 0) varValue = bb->vmin(); else varValue = bb->vmax();
		break;
		default: cout << "ERROR" << endl;
		break;
	}
	
	f = subs(f, fixedVariable, varValue); g = subs(g, fixedVariable, varValue); h = subs(h, fixedVariable, varValue); 
	mpol_t subsf(toString(f, V).toLatin1().data(), V); cout << "subsf = " << subsf << endl;
	mpol_t subsg(toString(g, V).toLatin1().data(), V); cout << "subsg = " << subsg << endl;
	mpol_t subsh(toString(h, V).toLatin1().data(), V); cout << "subsh = " << subsh << endl;
	bpol_t bpols[3] ;
	bpols[0] = toBernstein3D(subsf, xmin, xmax, ymin, ymax, zmin, zmax);cout << "bpols[0] = " << bpols[0] << endl;
	bpols[1] = toBernstein3D(subsg, xmin, xmax, ymin, ymax, zmin, zmax);cout << "bpols[1] = " << bpols[1] << endl;
	bpols[2] = toBernstein3D(subsh, xmin, xmax, ymin, ymax, zmin, zmax);cout << "bpols[2] = " << bpols[2] << endl;
	
	std::list<bpol_t> eqnList ; eqnList.push_back(bpols[0]) ; eqnList.push_back(bpols[1]) ; eqnList.push_back(bpols[2]) ;
	SbdSolver< double, SBDSLV_RDRDL> solver(bpols, bpols+3, epsilon) ;
	std::vector<double> results ; int nSol, nVar ;
	solver.solve(results, nSol, nVar, eqnList.begin(), eqnList.end());
	
	for(int s = 0 ; s < nSol ; s++) 
	{
		double x = results[nVar*s+0], y = results[nVar*s+1], z = results[nVar*s+2] ;
		x = (xmax-xmin)*(double)(results[nVar*s+0]) + xmin ;
		y = (ymax-ymin)*(double)(results[nVar*s+1]) + ymin ;
		y = (zmax-zmin)*(double)(results[nVar*s+2]) + zmin ;

		l << new QPoint4f(fixedVariable, varValue, x, y, z) ;
	}
	cout << FG_RED << "l.size() = " << l.size() << NOCOLOR << endl;
    return l ;
}

QPoint3f * QSynapsPlugin::map2D3D(float s, float t, QRationalSurface * s1)
{
	using MPOLDST::eval;
	mpol_t eqx(s1->px().toLatin1().data(), Variables("s t")) ;
    mpol_t eqy(s1->py().toLatin1().data(), Variables("s t")) ;
    mpol_t eqz(s1->pz().toLatin1().data(), Variables("s t")) ;

	float x = eval(eqx, s, t);
	float y = eval(eqx, s, t);
	float z = eval(eqx, s, t);
	QPoint3f * p  = new QPoint3f(x,y,z);
	return p;
}

QList<QPoint3f *> QSynapsPlugin::intersection(QImplicitCurve2D * c1, QImplicitCurve2D * c2, QBoundingBox * bb, double espilon)
{
    QList<QPoint3f *> l ;
	
    mpol_t p1(c1->equation().toAscii().constData(), Variables("x y")) ;
    mpol_t p2(c2->equation().toAscii().constData(), Variables("x y")) ;
	
    bpol_t bpols[2] ;
    bpols[0] = toBernstein2D(p1, bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax()) ;
    bpols[1] = toBernstein2D(p2, bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax()) ;
	
    std::list<bpol_t> eqnList ; eqnList.push_back(bpols[0]) ; eqnList.push_back(bpols[1]) ;

    SbdSolver< double, SBDSLV_RDRDL> solver(bpols, bpols+2, epsilon) ;
    std::vector<double> results ; int nSol, nVar ;
    solver.solve(results, nSol, nVar, eqnList.begin(), eqnList.end());
	
    for(int s = 0 ; s < nSol ; s++) {
	double x = results[nVar*s+0], y = results[nVar*s+1], z = 0.0 ;
	x = (bb->xmax()-bb->xmin())*(double)(results[nVar*s+0]) + bb->xmin() ;
	y = (bb->ymax()-bb->ymin())*(double)(results[nVar*s+1]) + bb->ymin() ;
	l << new QPoint3f(x, y, z) ;
    }
	
    return l ;
}

int QSynapsPlugin::numberOfBranches(QPoint3f * p, QImplicitCurve2D * c, QCell * bb)
{
    mpol_t pol(c->equation().toLatin1().data(), Variables("x y")) ;
    mpol_t dx = diff(pol, "x");
    mpol_t dy = diff(pol, "y");
    VectDse<double> dmn(4);
    dmn[0] = bb->xmin() ; dmn[1] = bb->xmax() ; 
    dmn[2] = bb->ymin() ; dmn[3] = bb->ymax() ;
    int deg = 2 * (1 - topdeg2d(dx, dy, dmn)) ;
    // cout << "topological degree of " << c->equation().toLatin1().data() << " in " << bb->toString().toLatin1().data() << " = " << deg << endl ;
    return deg ;
}

QImplicitCurve2D * QSynapsPlugin::computeFirstDerivativeCurve(QImplicitCurve2D * c, QString variable)
{
    mpol_t p(c->equation().toLatin1().data(), Variables("x y")) ;
    mpol_t d = diff(p, variable.toLatin1().data()) ;
    QImplicitCurve2D * dc = new QImplicitCurve2D(toString(d), c->boundingbox()) ;
    Mesh(dc) ;
    return dc ;
}

QImplicitCurve2D * QSynapsPlugin::computeSecondDerivativeCurve(QImplicitCurve2D * c, QString variable)
{
    mpol_t p(c->equation().toLatin1().data(), Variables("x y")) ;
    mpol_t d1 = diff(p,  variable.toLatin1().data()) ;
    mpol_t d2 = diff(d1, variable.toLatin1().data()) ;
    QImplicitCurve2D * dc = new QImplicitCurve2D(toString(d2), c->boundingbox()) ;
    Mesh(dc) ;
    return dc ;
}

QImplicitCurve2D * QSynapsPlugin::computeJacobiCurve(QImplicitCurve2D * c1, QImplicitCurve2D * c2)
{
    mpol_t f(c1->equation().toLatin1().data(), Variables("x y")) ;
    mpol_t g(c2->equation().toLatin1().data(), Variables("x y")) ;
    mpol_t dxf = diff(f, "x") ;
    mpol_t dyf = diff(f, "y") ;
    mpol_t dxg = diff(g, "x") ;
    mpol_t dyg = diff(g, "y") ;
    mpol_t h = dxf*dyg - dxg*dyf ;
    QImplicitCurve2D * jc = new QImplicitCurve2D(toString(h), c1->boundingbox()->intersect(c2->boundingbox())) ;
    Mesh(jc) ;
    return jc ;
}

QImplicitCurve2D * QSynapsPlugin::computeHessianCurve(QImplicitCurve2D * c)
{
    mpol_t p(c->equation().toLatin1().data(), Variables("x y")) ;
    mpol_t f = diff(p, "x") ;
    mpol_t g = diff(p, "y") ;
    mpol_t dxf = diff(f, "x") ;
    mpol_t dyf = diff(f, "y") ;
    mpol_t dxg = diff(g, "x") ;
    mpol_t dyg = diff(g, "y") ;
    mpol_t h = dxf*dyg - dxg*dyf ;
    QImplicitCurve2D * jc = new QImplicitCurve2D(toString(h), c->boundingbox()) ;
    Mesh(jc) ;
    return jc ;
}

QImplicitCurve4D * QSynapsPlugin::computeIntersectionEquation4D(QRationalSurface * s1, QRationalSurface * s2)
{
    mpol_t s1x(s1->px().toLatin1().data(), Variables("s t u v")) ;
    mpol_t s1y(s1->py().toLatin1().data(), Variables("s t u v")) ;
    mpol_t s1z(s1->pz().toLatin1().data(), Variables("s t u v")) ;
    mpol_t s2x(s2->px().toLatin1().data(), Variables("s t u v")) ;
    mpol_t s2y(s2->py().toLatin1().data(), Variables("s t u v")) ;
    mpol_t s2z(s2->pz().toLatin1().data(), Variables("s t u v")) ;
    mpol_t eq1(s1x-s2x) ;
    mpol_t eq2(s1y-s2y) ;
    mpol_t eq3(s1z-s2z) ;
    QString seq1 = toString(eq1, "s t u v") ;
    QString seq2 = toString(eq2, "s t u v") ;
    QString seq3 = toString(eq3, "s t u v") ;
    
    QImplicitCurve4D * res = new QImplicitCurve4D(seq1, seq2, seq3,
						  s1->umin(), s1->umax(),
						  s1->vmin(), s1->vmax(),
						  s2->umin(), s2->umax(),
						  s2->vmin(), s2->vmax()) ;
    return res ;
}

/* Local injectivity of a QImplicitCurve4D w.r.t one variable (i=0..3 for the 4 variables). 
It just consist in verifying that the jacobian is non zero by using the Bernstein form */

bool QSynapsPlugin::isLocallyInjective4D(QImplicitCurve4D * c, QBoundingBox4D * bb, int i)
{ 
	Variables V = c->var() ;
	mpol_t f(c->eqf(), V) ;
	mpol_t g(c->eqg(), V) ;
	mpol_t h(c->eqh(), V) ;
	bpol_t bf = toBernstein4D(f, bb) ;
	bpol_t bg = toBernstein4D(g, bb) ;
	bpol_t bh = toBernstein4D(h, bb) ;
	bpol_t f1, f2, f3, g1, g2, g3, h1, h2, h3 ;
	switch(i)
	{ 
		case 0: // variable s
			diff(f1, bf, 1); 
			diff(f2, bf, 2);
			diff(f3, bf, 3);
			diff(g1, bg, 1);
			diff(g2, bg, 2);
			diff(g3, bg, 3);
			diff(h1, bh, 1);
			diff(h2, bh, 2);
			diff(h3, bh, 3);
		break;
		case 1: // variable t 
			diff(f1, bf, 0);
			diff(f2, bf, 2);
			diff(f3, bf, 3);
			diff(g1, bg, 0);
			diff(g2, bg, 2);
			diff(g3, bg, 3);
			diff(h1, bh, 0);
			diff(h2, bh, 2);
			diff(h3, bh, 3);
		break;
		case 2: // variable u
			diff(f1, bf, 0);
			diff(f2, bf, 1);
			diff(f3, bf, 3);
			diff(g1, bg, 0);
			diff(g2, bg, 1);
			diff(g3, bg, 3);
			diff(h1, bh, 0);
			diff(h2, bh, 1);
			diff(h3, bh, 3);
		break;
		case 3: // variable v
			diff(f1, bf, 0);
			diff(f2, bf, 1);
			diff(f3, bf, 2);
			diff(g1, bg, 0);
			diff(g2, bg, 1);
			diff(g3, bg, 2);
			diff(h1, bh, 0);
			diff(h2, bh, 1);
			diff(h3, bh, 2);
		break; 
		default: 
			cout << __func__ << "ERROR in the variable choice" << endl ;
			return false;
		break; 
	}
	bpol_t det = f1*g2*h3 + f2*h1*g3 + f3*g1*h2 - f3*g2*h1 - f2*g1*h3 - f1*g3*h2; 
	return hasNoRoot(det);
}


/* If f is a polynomial in 4 variables var="s t u v", i=0 (it corresponds to the variable s) and bb = [a0,b0]x[a1,b1]x[a2xb2]x[a3,b3],
we test if for all s in [a0,b0] the map (t,u,v)->f(s,t,u,v) has an appropriate monotony on the edges of [a1,b1]x[a2xb2]x[a3,b3].
We suppose that the 2 couples (s,t) and (u,v) are seperated. It is the case for an intersection curve of 2 rational surfaces. */

bool QSynapsPlugin::isEdgeMonotonic4D(QString f, char * var, int i, QBoundingBox4D * bb)
{
	using MPOLDST::diff;
	using MPOLDST::subs;
	using MPOLDST::swap;
	mpol_t p(f.toLatin1().data(), Variables(var));
	mpol_t dp1, dp2, dp3;
	mpol_t poltmp1, poltmp2;
	double var1min, var1max, var2min, var2max;
	QList<bpol_t> L1, L2, L3;
	switch(i)
	{ 
		case 0: // we consider the map (t,u,v)->f(s,t,u,v) on the edges of [a1,b1]x[a2xb2]x[a3,b3]
			diff(dp1,p,1); // df/dt
			diff(dp2,p,2); // df/du 
			diff(dp3,p,3); // df/dv

			// First list of inequalities (just one in 2 variables (s,t))
			poltmp1 = dp1 ;
			var1min = bb->smin(); var1max = bb->smax(); var2min = bb->tmin(); var2max = bb->tmax();
			L1 << toBernstein2D(poltmp1, var1min, var1max, var2min, var2max);
			// cout << "L1 : " << endl; foreach(bpol_t p, L1) {cout << p << " = " ; VECTOR::print(cout,p.rep())<<endl;}

			// Second list of inequalities (2 in one variable u)
			poltmp1 = subs(dp2, 3 , (double) bb->vmin());
			poltmp2 = subs(dp2, 3 , (double) bb->vmax());

			poltmp1 = swap(poltmp1,0,2); // in order to have a polynomial with the variable x0 for toBernstein1D function.
			poltmp2 = swap(poltmp2,0,2);
			var1min = bb->umin(); var1max = bb->umax();
			L2 << toBernstein1D(poltmp1, var1min, var1max) << toBernstein1D(poltmp2, var1min, var1max);
			// cout << "L2 : " << endl; foreach(bpol_t p, L2) {cout << p << " = " ; VECTOR::print(cout,p.rep())<<endl;}
			
			// Third list of inequalities (2 in one variable v)
			poltmp1 = subs(dp3, 2 , (double) bb->umin());
			poltmp2 = subs(dp3, 2 , (double) bb->umax());

			poltmp1 = swap(poltmp1,0,3); // in order to have a polynomial with the variable x0 for toBernstein1D function.
			poltmp2 = swap(poltmp2,0,3);
			var1min = bb->vmin(); var1max = bb->vmax();
			L3 << toBernstein1D(poltmp1, var1min, var1max) << toBernstein1D(poltmp2, var1min, var1max);
			// cout << "L3 : " << endl; foreach(bpol_t p, L3) {cout << p << " = " ; VECTOR::print(cout,p.rep())<<endl;}
			return areOfSameSign(L1) && areOfSameSign(L2) && areOfSameSign(L3);
		break;
		case 1: // we consider the map (s,u,v)->f(s,t,u,v) on the edges of [a0,b0]x[a2xb2]x[a3,b3]
			diff(dp1,p,0); // df/ds
			diff(dp2,p,2); // df/du 
			diff(dp3,p,3); // df/dv
        	
			// First list of inequalities (just one in 2 variables (s,t))
			poltmp1 = dp1 ;
			var1min = bb->smin(); var1max = bb->smax(); var2min = bb->tmin(); var2max = bb->tmax();
			L1 << toBernstein2D(poltmp1, var1min, var1max, var2min, var2max);
			// cout << "L1 : " << endl; foreach(bpol_t p, L1) {cout << p << " = " ; VECTOR::print(cout,p.rep())<<endl;}
        	
			// Second list of inequalities (2 in one variable u)
			poltmp1 = subs(dp2, 3 , (double) bb->vmin());
			poltmp2 = subs(dp2, 3 , (double) bb->vmax());
        	
			poltmp1 = swap(poltmp1,0,2); // in order to have a polynomial with the variable x0 for toBernstein1D function.
			poltmp2 = swap(poltmp2,0,2);
			var1min = bb->umin(); var1max = bb->umax();
			L2 << toBernstein1D(poltmp1, var1min, var1max) << toBernstein1D(poltmp2, var1min, var1max);
			// cout << "L2 : " << endl; foreach(bpol_t p, L2) {cout << p << " = " ; VECTOR::print(cout,p.rep())<<endl;}
			
			// Third list of inequalities (2 in one variable v)
			poltmp1 = subs(dp3, 2 , (double) bb->umin());
			poltmp2 = subs(dp3, 2 , (double) bb->umax());
        	
			poltmp1 = swap(poltmp1,0,3); // in order to have a polynomial with the variable x0 for toBernstein1D function.
			poltmp2 = swap(poltmp2,0,3);
			var1min = bb->vmin(); var1max = bb->vmax();
			L3 << toBernstein1D(poltmp1, var1min, var1max) << toBernstein1D(poltmp2, var1min, var1max);
			// cout << "L3 : " << endl; foreach(bpol_t p, L3) {cout << p << " = " ; VECTOR::print(cout,p.rep())<<endl;}
			return areOfSameSign(L1) && areOfSameSign(L2) && areOfSameSign(L3);
		break;
		
		case 2: // we consider the map (s,t,v)->f(s,t,u,v) on the edges of [a0,b0]x[a1xb1]x[a3,b3]
			diff(dp1,p,0); // df/ds
			diff(dp2,p,1); // df/dt 
			diff(dp3,p,3); // df/dv
    		
			// First list of inequalities (2 in one variable s)
			poltmp1 = subs(dp1, 1 , (double) bb->tmin());
			poltmp2 = subs(dp1, 1 , (double) bb->tmax());
    		
			var1min = bb->smin(); var1max = bb->smax();
			L1 << toBernstein1D(poltmp1, var1min, var1max) << toBernstein1D(poltmp2, var1min, var1max);
			// cout << "L1 : " << endl; foreach(bpol_t p, L1) {cout << p << " = " ; VECTOR::print(cout,p.rep())<<endl;}
			
			// Second list of inequalities (2 in one variable t)
			poltmp1 = subs(dp2, 0 , (double) bb->smin());
			poltmp2 = subs(dp2, 0 , (double) bb->smax());
    		
			poltmp1 = swap(poltmp1,0,1); // in order to have a polynomial with the variable x0 for toBernstein1D function.
			poltmp2 = swap(poltmp2,0,1);
			var1min = bb->tmin(); var1max = bb->tmax();
			L2 << toBernstein1D(poltmp1, var1min, var1max) << toBernstein1D(poltmp2, var1min, var1max);
			// cout << "L2 : " << endl; foreach(bpol_t p, L2) {cout << p << " = " ; VECTOR::print(cout,p.rep())<<endl;}
			
			// Third list of inequalities (just one in 2 variables (u,v))
			poltmp1 = dp3 ;
			poltmp1 = swap(swap(poltmp1,0,2),1,3); // in order to have a polynomial with the variables x0 and x1 for toBernstein2D function.
			var1min = bb->umin(); var1max = bb->umax(); var2min = bb->vmin(); var2max = bb->vmax();
			L3 << toBernstein2D(poltmp1, var1min, var1max, var2min, var2max);
			// cout << "L3 : " << endl; foreach(bpol_t p, L3) {cout << p << " = " ; VECTOR::print(cout,p.rep())<<endl;}
    		return areOfSameSign(L1) && areOfSameSign(L2) && areOfSameSign(L3);
		break;
		case 3: // we consider the map (s,t,u)->f(s,t,u,v) on the edges of [a0,b0]x[a1xb1]x[a2,b2]
			diff(dp1,p,0); // df/ds
			diff(dp2,p,1); // df/dt 
			diff(dp3,p,2); // df/du
			
			// First list of inequalities (2 in one variable s)
			poltmp1 = subs(dp1, 1 , (double) bb->tmin());
			poltmp2 = subs(dp1, 1 , (double) bb->tmax());
			
			var1min = bb->smin(); var1max = bb->smax();
			L1 << toBernstein1D(poltmp1, var1min, var1max) << toBernstein1D(poltmp2, var1min, var1max);
			// cout << "L1 : " << endl; foreach(bpol_t p, L1) {cout << p << " = " ; VECTOR::print(cout,p.rep())<<endl;}
			
			// Second list of inequalities (2 in one variable t)
			poltmp1 = subs(dp2, 0 , (double) bb->smin());
			poltmp2 = subs(dp2, 0 , (double) bb->smax());
			
			poltmp1 = swap(poltmp1,0,1); // in order to have a polynomial with the variable x0 for toBernstein1D function.
			poltmp2 = swap(poltmp2,0,1);
			var1min = bb->tmin(); var1max = bb->tmax();
			L2 << toBernstein1D(poltmp1, var1min, var1max) << toBernstein1D(poltmp2, var1min, var1max);
			// cout << "L2 : " << endl; foreach(bpol_t p, L2) {cout << p << " = " ; VECTOR::print(cout,p.rep())<<endl;}
			
			// Third list of inequalities (just one in 2 variables (u,v))
			poltmp1 = dp3 ;
			poltmp1 = swap(swap(poltmp1,0,2),1,3); // in order to have a polynomial with the variables x0 and x1 for toBernstein2D function.
			var1min = bb->umin(); var1max = bb->umax(); var2min = bb->vmin(); var2max = bb->vmax();
			L3 << toBernstein2D(poltmp1, var1min, var1max, var2min, var2max);
			// cout << "L3 : " << endl; foreach(bpol_t p, L3) {cout << p << " = " ; VECTOR::print(cout,p.rep())<<endl;}
			return areOfSameSign(L1) && areOfSameSign(L2) && areOfSameSign(L3);
		break;
		default: 
			cout << __func__  << "ERROR in the variable choice" << endl;
			return false;
		break; 
	}
}

/* If f and g are 2 polynomials in 4 variables var="s t u v", i=0 (it corresponds to the variable s) and bb = [a0,b0]x[a1,b1]x[a2xb2]x[a3,b3],
we test if for all s in [a0,b0] the map (t,u,v)->g(s,t,u,v) has an appropriate monotony on the borders of the implicit surface f(s,t,u,v) = 0
i.e on the set {(t,u,v) on the faces of [a1,b1]x[a2xb2]x[a3,b3] such that f(s,t,u,v) = 0}.
We suppose that the 2 couples (s,t) and (u,v) are seperated. It is the case for an intersection curve of 2 rational surfaces. */

bool QSynapsPlugin::isBorderMonotonic4D(QString f, QString g, char * var, int i, QBoundingBox4D * bb)
{// cout << __func__ << bb->toString().toLatin1().data() << "..." << endl; getchar();
	using MPOLDST::diff;
	using MPOLDST::subs;
	
	// Construction of the data structures of the inequalities.

	// Firstly : 0 for the counterclockwise and 1 for counterclockwise.
	bool L[32][6]={{0,0,0,0,0,0},{1,1,1,1,1,1},{0,1,1,1,1,1},{0,0,1,1,1,1},{0,0,0,1,1,1},{0,0,0,0,1,1},{0,0,0,0,0,1}};
/*	L[0] = {0,0,0,0,0,0} -> it corresponds to clockwise monotonic rotation over the 6 faces.
	L[1] = {1,1,1,1,1,1} -> it corresponds to counterclockwise monotonic rotation over the 6 faces.
	L[2]...L[6] -> correspond to the first 5 cases when we fix the first vertex.*/
	
	// And then we build the other cases by using L[2]...L[6] (just shift the elements...)
	for (int i = 1; i <= 5; ++i)
	{
		for (int j = 0; j < 6; ++j)
		{
			L[2+i*5][j] = L[2][(j+i*5) % 6];
			L[3+i*5][j] = L[3][(j+i*5) % 6];
			L[4+i*5][j] = L[4][(j+i*5) % 6];
			L[5+i*5][j] = L[5][(j+i*5) % 6];
			L[6+i*5][j] = L[6][(j+i*5) % 6];
		}
		
	}
	// cout << "Display the cases : " << endl;
	// for (int i = 0; i < 32; ++i) {  cout << "Case "  << i << endl;  for (int j = 0; j<6; ++j) { cout << L[i][j]; } cout << endl;}
	
	// Now we code the inequalities (0 for the inequality < 0 and 1 for the inequality > 0).
	for (int i = 0; i < 32; ++i)
	{
		for (int j = 0; j < 6; j=j+2)
		{
			if (L[i][j] == 0) L[i][j] = 1;
			else L[i][j] = 0;
		}
	}
	// cout << "Display the cases : " << endl;
	// for (int i = 0; i < 32; ++i) {  cout << "Case "  << i << endl;  for (int j = 0; j<6; ++j) { cout << L[i][j]; } cout << endl;}
	
	// Construction of the expressions that we have to test
	mpol_t f1(f.toLatin1().data(), Variables(var));
	mpol_t f2(g.toLatin1().data(), Variables(var));
	mpol_t subsf1, subsf2;
	bpol_t bf1, bf2;
	bpol_t f2DotOrthf1[6];
	bpol_t d1f1, d2f1, d1f2, d2f2;
	mpol_t poltmp;
	int varDiff1, varDiff2;
	int tvar[3];
	int varSubs;
	double tborder[8]={bb->smin(), bb->smax(), bb->tmin(), bb->tmax(), bb->umin(), bb->umax(), bb->vmin(), bb->vmax()};
	double valSubs;
	
	switch(i)
	{ 
		case 0: // variable s is fixed
			tvar[0] = 1; tvar[1] = 2; tvar[2] = 3;
		break;
		case 1: // variable t is fixed
			tvar[0] = 2; tvar[1] = 3; tvar[2] = 0;
		break;
		case 2: // variable u is fixed
			tvar[0] = 3; tvar[1] = 0; tvar[2] = 1;
		break;
		case 3: // variable v is fixed
			tvar[0] = 0; tvar[1] = 1; tvar[2] = 2;
		break; 
		default: 
			cout << __func__  << "ERROR in the variable choice" << endl;
			return false;
		break; 
	}
	for (int j = 0; j < 6; ++j)
	{
		varDiff1 = tvar[j % 3]; varDiff2 = tvar[(j+1) % 3];
		varSubs = tvar[(j+2) % 3];
		if (j & 1) valSubs = tborder[2*varSubs] ; // (j & 1) <=> j is an odd number.
		else valSubs = tborder[2*varSubs + 1] ;
		
		double xmin = tborder[2*((varSubs + 1)%4)]; double ymin = tborder[2*((varSubs + 2)%4)]; double zmin = tborder[2*((varSubs + 3)%4)];
		double xmax = tborder[2*((varSubs + 1)%4)+1]; double ymax = tborder[2*((varSubs + 2)%4)+1]; double zmax = tborder[2*((varSubs + 3)%4)+1];

		subsf1 = subs(f1, varSubs, valSubs);
		subsf2 = subs(f2, varSubs, valSubs);
		bf1 = toBernstein3D(subsf1, xmin, xmax, ymin, ymax, zmin, zmax);
		bf2 = toBernstein3D(subsf2, xmin, xmax, ymin, ymax, zmin, zmax);
		
		// if a variable is eliminated, then, in the mpoldse case, we have to make a shift in order to consider the wanted variable.
		if (varSubs < varDiff1) 
		{
			diff(d1f1,bf1,varDiff1-1);// cout << " d1f1  = " << d1f1 << endl;
			diff(d1f2,bf2,varDiff1-1);// cout << " d1f2  = " << d1f2 << endl;
		}
		if (varSubs > varDiff1)
		{
			diff(d1f1,bf1,varDiff1);// cout << " d1f1  = " << d1f1 << endl;
			diff(d1f2,bf2,varDiff1);// cout << " d1f2  = " << d1f2 << endl;
		}
		if (varSubs < varDiff2) 
		{
			diff(d2f1,bf1,varDiff2-1);// cout << " d2f1  = " << d2f1 << endl;
			diff(d2f2,bf2,varDiff2-1);// cout << " d2f2  = " << d2f2 << endl;	
		}
		if (varSubs > varDiff2)
		{
			diff(d2f1,bf1,varDiff2); //cout << " d2f1  = " << d2f1 << endl;
			diff(d2f2,bf2,varDiff2); //cout << " d2f2  = " << d2f2 << endl;
		}
		
		/* Then the polynomial d1f1*d2f2 - d2f1*d1f2 does not depend on the variable varSubs so it is a polynomial in the 3 variables of indices 
		varSubs+1 (mod 4), varSubs+2 (mod 4) and varSubs+3 (mod 4). 
		We can possibly use MPOLDST::swap later to have the variables x0, x1 and x2, but probably it is not necessary.  */
		// double xmin = tborder[2*((varSubs + 1)%4)]; double ymin = tborder[2*((varSubs + 2)%4)]; double zmin = tborder[2*((varSubs + 3)%4)];
		// double xmax = tborder[2*((varSubs + 1)%4)+1]; double ymax = tborder[2*((varSubs + 2)%4)+1]; double zmax = tborder[2*((varSubs + 3)%4)+1];
		// f2DotOrthf1[j] = toBernstein3D(d1f1*d2f2 - d2f1*d1f2, xmin, xmax, ymin, ymax, zmin, zmax);
		f2DotOrthf1[j] = d1f1*d2f2 - d2f1*d1f2;
	}
	
	// Evaluate the tests.
	bool res = false;
	int count = 0;
	while (!res && (count < 32))
	{
		for (int i = 0; i< 6; ++i)
			if  (L[count][i] = 0) res = isNegativ(f2DotOrthf1[i]);
			else res = isPositiv(f2DotOrthf1[i]);
	}
	
	return res;
}

bool QSynapsPlugin::isOn(QPoint3f * p, QImplicitCurve2D * c)
{
    mpol_t eq(c->equation().toLatin1().data(), Variables("x y")) ;
    double pt[] = { p->x() , p->y() } ;
    return (eq(pt) == 0) ;
}

bool QSynapsPlugin::isOn(QPoint3f * p, QImplicitCurve2D * c, double tolerance)
{
    mpol_t eq(c->equation().toLatin1().data(), Variables("x y")) ;
    double pt[] = { p->x() , p->y() } ;
    return (fabs(eq(pt))<tolerance) ;
}

int QSynapsPlugin::degree(QImplicitCurve2D * c, int var)
{
    mpol_t eq(c->equation().toLatin1().data(), Variables("x y")) ;
    return ::SYNAPS::degree(eq, var) ;
}


int QSynapsPlugin::binomial(int n, int p) {
     return ::SYNAPS::binomial<int>(n,p);
}


void QSynapsPlugin::toBezierArray(QImplicitCurve2D * c, float * array, int * deg_x, int * deg_y)
{
    mpol_t eq(c->equation().toLatin1().data(), Variables("x y")) ;
    bpol_t bp = toBernstein2D(eq, 
			      c->boundingbox()->xmin(),
			      c->boundingbox()->xmax(), 
			      c->boundingbox()->ymin(),
			      c->boundingbox()->ymax());
//    *deg_x = ::SYNAPS::degree(eq,0) ; cout << __func__ << "degree in x" << *deg_x << endl ;
//    *deg_y = ::SYNAPS::degree(eq,1) ; cout << __func__ << "degree in y" << *deg_y << endl ;
//    array = new float[(*deg_x+1)*(*deg_y+1)] ;
    for(int i = 0 ; i < (*deg_x+1)*(*deg_y+1) ; i++) {
	array[i] = bp[i] ;
//	cout << i << " = " << array[i] << endl ;
    } 
}

void QSynapsPlugin::toBezierArray(QImplicitSurface * s, float * array, int * deg_x, int * deg_y)
{
    mpol_t eq(s->equation().toLatin1().data(), Variables("x y z")) ;
    bpol_t bp = toBernstein3D(eq, 
			      s->boundingbox()->xmin(),
			      s->boundingbox()->xmax(), 
			      s->boundingbox()->ymin(),
			      s->boundingbox()->ymax(),
			      s->boundingbox()->zmin(),
			      s->boundingbox()->zmax()) ;
//    *deg_x = ::SYNAPS::degree(eq,0) ; cout << __func__ << "degree in x: " << *deg_x << endl ;
//    *deg_y = ::SYNAPS::degree(eq,1) ; cout << __func__ << "degree in y: " << *deg_y << endl ;
//    array = new float[(*deg_x+1)*(*deg_y+1)] ;
    for(int i = 0 ; i < (*deg_x+1)*(*deg_y+1) ; i++) {
	array[i] = bp[i] ;
//	cout << i << " = " << array[i] << endl ;
    }
}

QString QSynapsPlugin::toLinearMonomialEquation(float * array, float xmin, float xmax, float ymin, float ymax)
{
    mpol_t x("x") ;
    mpol_t y("y") ;
    mpol_t xy("x*y") ;

    float w = (xmax-xmin)*(ymax-ymin) ;
    float l0 = array[0] ;
    float l1 = array[1] ; 
    float l2 = array[2] ;
    float l3 = array[3] ;

    mpol_t eq = (l1*ymin+l2*ymax-l0*ymax-l3*ymin)/w*x + (l1*xmax+l2*xmin-l0*xmax-l3*xmin)/w*y + (l0*xmax*ymax-l1*xmax*ymin-l2*xmin*ymax+l3*ymin*xmin)/w ;

    return toString(eq) ;
}

QString QSynapsPlugin::toQuadraticMonomialEquation(float * array, float xmin, float xmax, float ymin, float ymax)
{
    mpol_t x("x") ; mpol_t x2("x^2") ;
    mpol_t y("y") ; mpol_t y2("y^2") ;
    mpol_t xy("x*y") ;

    float w = (xmax-xmin)*(xmax-xmin)*(ymax-ymin)*(ymax-ymin) ;
    float l0 = array[0] ;
    float l1 = array[1] ;
    float l2 = array[2] ;
    float l3 = array[3] ;
    float l4 = array[4] ;
    float l5 = array[5] ;
    float l6 = array[6] ;
    float l7 = array[7] ;
    float l8 = array[8] ;

    mpol_t eq = (-2*l7*ymin*ymax-2*l5*ymin*ymin+l6*ymax*ymax+l8*ymin*ymin-2*l1*ymin*ymax+l0*ymax*ymax+l2*ymin*ymin-2*l3*ymax*ymax+4*l4*ymin*ymax)/w*x2
	+ (-2*l1*xmax*xmax+l2*xmax*xmax+l0*xmax*xmax-2*l3*xmin*xmax+4*l4*xmin*xmax+l6*xmin*xmin-2*l5*xmin*xmax-2*l7*xmin*xmin+l8*xmin*xmin)/w*y2
	+ (4 *l4* ymin* xmax - 4 *l5 *ymin *xmax - 4 *l3 *ymax *xmax + 4 *l4 *ymax *xmax+ 4 *l0 *ymax* xmax - 4 *l1* ymax* xmax - 4 *l1 *ymin* xmax + 4 *l2 *ymin* xmax+ 4* l4* ymax* xmin - 4 *l3 *ymax* xmin - 4 *l5 *ymin *xmin + 4 *l6 *ymax* xmin- 4 *l7 *ymax* xmin + 4 *l4* ymin* xmin - 4 *l7 *ymin* xmin + 4 *l8 *ymin* xmin)/w*xy
	+ (-4 *l4 *ymin* ymax* xmax + 2* l3* ymax*ymax * xmax + 2 *l5* ymin*ymin * xmax - 2 *l0 *ymax*ymax  *xmax- 2 *l2 *ymin*ymin * xmax + 2 *l3 *ymax*ymax  *xmin + 2 *l5 *ymin*ymin  *xmin - 2 *l6 *ymax*ymax  *xmin - 2 *l8 *ymin*ymin  *xmin + 4 *l1 *ymin *ymax *xmax - 4 *l4 *ymin *ymax *xmin   + 4 *l7 *ymin *ymax *xmin)/w*x
	+ (-2 *l0 *ymax *xmax*xmax  + 2 *l1 *ymax *xmax*xmax  + 2 *l1 *ymin *xmax*xmax  - 2 *l2 *ymin *xmax*xmax 
	   + 4 *l3 *ymax *xmin *xmax - 4 *l4 *ymax *xmin *xmax - 4 *l4 *ymin *xmin *xmax
	   - 2 *l6 *ymax *xmin*xmin  + 4 *l5 *ymin *xmin *xmax + 2 *l7 *ymax *xmin*xmin  + 2 *l7 *ymin *xmin*xmin 
	   - 2 *l8 *ymin *xmin*xmin)/w*y
	+ (-2 *l1 *ymin *ymax *xmax*xmax  + l2 *ymin*ymin  *xmax*xmax  + l0 *ymax*ymax  *xmax*xmax  - 2 *l3 *ymax*ymax  *xmin *xmax
	   + 4 *l4 *ymin *ymax *xmin *xmax - 2 *l5 *ymin*ymin  *xmin *xmax + l6 *ymax*ymax  *xmin*xmin 
	   - 2 *l7 *ymin *ymax *xmin*xmin  + l8 *ymin*ymin  *xmin*xmin) / w ;
    
    return toString(eq) ;
}

bool QSynapsPlugin::lineIntersection(
    float Ax, float Ay,
    float Bx, float By,
    float Cx, float Cy,
    float Dx, float Dy,
    float *X, float *Y) {

    float  distAB, theCos, theSin, newX, ABpos ;

    //  Fail if either line is undefined.
    if (Ax==Bx && Ay==By || Cx==Dx && Cy==Dy) return false ;

    //  (1) Translate the system so that point A is on the origin.
    Bx-=Ax; By-=Ay;
    Cx-=Ax; Cy-=Ay;
    Dx-=Ax; Dy-=Ay;

    //  Discover the length of segment A-B.
    distAB=sqrt(Bx*Bx+By*By);

    //  (2) Rotate the system so that point B is on the positive X axis.
    theCos=Bx/distAB;
    theSin=By/distAB;
    newX=Cx*theCos+Cy*theSin;
    Cy  =Cy*theCos-Cx*theSin; Cx=newX;
    newX=Dx*theCos+Dy*theSin;
    Dy  =Dy*theCos-Dx*theSin; Dx=newX;

    //  Fail if the lines are parallel.
    if (Cy==Dy) return false ;

    //  (3) Discover the position of the intersection point along line A-B.
    ABpos=Dx+(Cx-Dx)*Dy/(Dy-Cy);

    //  (4) Apply the discovered position to line A-B in the original coordinate system.
    *X=Ax+ABpos*theCos;
    *Y=Ay+ABpos*theSin;

    //  Success.
    return true ; 
}

/*************************************************************
 *! Computes the intersection point between an implicit curve
 *  and a vertical line of coordinate \param y between \param
 *  xmin and \param xmax.
 *
 *  Uses a 1D Bezier solver for the sake of efficiency.
 *************************************************************/

QList<QPoint3f *> QSynapsPlugin::intersectionX(QImplicitCurve2D * c, float y, float xmin, float xmax)
{
    QList<QPoint3f *> l ;

    mpol_t t(c->equation().toLatin1().data(), Variables("x y")) ;
    mpol_t p = ::SYNAPS::MPOLDST::subs(t, 1, y) ;
    upol_t u ; ::SYNAPS::let::convert(u, p, 0) ;

    Seq<double> results = solve(u, SlvBzStd<double,double>(1e-6)) ;

    for(Seq<double>::iterator it = results.begin() ; it != results.end() ; it++)
	if(xmin <= *it && *it <= xmax)
	    l << new QPoint3f((float)*it, y, 0.0) ;

    return l ;
}

/*************************************************************
 *! Computes the intersection point between an implicit curve
 *  and an horizo. line of coordinate \param x between \param
 *  ymin and \param ymax.
 *
 *  Uses a 1D Bezier solver for the sake of efficiency.
 *************************************************************/

QList<QPoint3f *> QSynapsPlugin::intersectionY(QImplicitCurve2D * c, float x, float ymin, float ymax)
{
    QList<QPoint3f *> l ;

    mpol_t t(c->equation().toLatin1().data(), Variables("x y")) ;
    mpol_t p = ::SYNAPS::MPOLDST::subs(t, 0, x) ;
    upol_t u ; ::SYNAPS::let::convert(u, p, 1) ;

    Seq<double> results = solve(u, SlvBzStd<double,double>(1e-6)) ;

    for(Seq<double>::iterator it = results.begin() ; it != results.end() ; it++)
	if(ymin <= *it && *it <= ymax)
	    l << new QPoint3f(x, (float)*it, 0.0) ;

    return l ;
}

/*************************************************************
 *! Computes the intersection point between two region edges.
 *  Provided for convenience, simply handle the different types
 *  of a QRegionEdge.
 *************************************************************/

QList<QPoint3f *> QSynapsPlugin::intersection(QRegionEdge * e1, QRegionEdge * e2, QBoundingBox * bb, double espilon)
{
    QList<QPoint3f *> l ;

    if(e1->object()->type() == "ImplicitCurve2D" && e2->object()->type() == "ImplicitCurve2D") {
	QList<QPoint3f *> pts ;
	pts << intersection((QImplicitCurve2D *)e1->object(), (QImplicitCurve2D *)e2->object(), bb, epsilon) ;
	foreach(QPoint3f * p, pts) p->setColor(Qt::blue) ;
	l << pts ;
    } else if(e1->object()->type() == "ImplicitCurve2D" && e2->object()->type() == "Cell") {

	float xmin = qMin(e2->source()->x(), e2->dest()->x()) ; 
	float ymin = qMin(e2->source()->y(), e2->dest()->y()) ; 
	float xmax = qMax(e2->source()->x(), e2->dest()->x()) ; 
	float ymax = qMax(e2->source()->y(), e2->dest()->y()) ; 

	// Adjust the edge to the bounding box
	if(xmin == xmax) {
	    ymin = qMax(ymin, bb->ymin()) ;
	    ymax = qMin(ymax, bb->ymax()) ;
	} else if(ymin == ymax) {
	    xmin = qMax(xmin, bb->xmin()) ;
	    xmax = qMin(xmax, bb->xmax()) ;
	}

	QList<QPoint3f *> pts ;
	if(xmin == xmax)
	    pts = intersectionY((QImplicitCurve2D *)e1->object(), xmin, ymin, ymax) ;
	else
	    pts = intersectionX((QImplicitCurve2D *)e1->object(), ymin, xmin, xmax) ;
	foreach(QPoint3f * p, pts) p->setColor(Qt::yellow) ;
	l << pts ;

    } else if(e1->object()->type() == "Cell" && e2->object()->type() == "ImplicitCurve2D") {

	float xmin = qMin(e1->source()->x(), e1->dest()->x()) ;
	float ymin = qMin(e1->source()->y(), e1->dest()->y()) ;
	float xmax = qMax(e1->source()->x(), e1->dest()->x()) ;
	float ymax = qMax(e1->source()->y(), e1->dest()->y()) ;

	// Adjust the edge to the bounding box
	if(xmin == xmax) {
	    ymin = qMax(ymin, bb->ymin()) ;
	    ymax = qMin(ymax, bb->ymax()) ;
	} else if(ymin == ymax) {
	    xmin = qMax(xmin, bb->xmin()) ;
	    xmax = qMin(xmax, bb->xmax()) ;
	}

	QList<QPoint3f *> pts ;
	if(xmin == xmax)
	    pts = intersectionY((QImplicitCurve2D *)e2->object(), xmin, ymin, ymax) ;
	else
	    pts = intersectionX((QImplicitCurve2D *)e2->object(), ymin, xmin, xmax) ;
	foreach(QPoint3f * p, pts) p->setColor(Qt::yellow) ;
	l << pts ;

    } else {
	float x, y ;
	QList<QPoint3f *> pts ;
	if(lineIntersection(e1->source()->x(), e1->source()->y(), e1->dest()->x(), e1->dest()->y(), e2->source()->x(), e2->source()->y(), e2->dest()->x(), e2->dest()->y(), &x, &y))
	    pts << new QPoint3f(x, y, 0.0) ;
	foreach(QPoint3f * p, pts) {
	    if(p->x() >= bb->xmin()
	    && p->x() <= bb->xmax()
	    && p->y() >= bb->ymin()
	    && p->y() <= bb->ymax()) {
		p->setColor(Qt::magenta) ;
		l << p ;
	    }
	}
    }

    return l ;
}

/*************************************************************
 *! Compute the intersection points of a RegionEdge with the
 *  border of the cell.
 *  
 *  Provide for convenience, adds a switch on the type of the
 *  egde.
 *************************************************************/
QList<QPoint3f *> QSynapsPlugin::intersectionPoints(QRegionEdge * e, QCell * bb, double epsilon)
{
    QList<QPoint3f *> l ;

    if(e->object()->type() == "ImplicitCurve2D") {
	l << intersectionPoints((QImplicitCurve2D *)e->object(), bb, epsilon) ;
    } else {
	float x, y ;
	if(lineIntersection(e->source()->x(), e->source()->y(), e->dest()->x(), e->dest()->y(), bb->xmin(), bb->ymin(), bb->xmax(), bb->ymin(), &x, &y))
	    l << new QPoint3f(x, y, 0.0) ;
	if(lineIntersection(e->source()->x(), e->source()->y(), e->dest()->x(), e->dest()->y(), bb->xmax(), bb->ymin(), bb->xmax(), bb->ymax(), &x, &y))
	    l << new QPoint3f(x, y, 0.0) ;
	if(lineIntersection(e->source()->x(), e->source()->y(), e->dest()->x(), e->dest()->y(), bb->xmax(), bb->ymax(), bb->xmin(), bb->ymax(), &x, &y))
	    l << new QPoint3f(x, y, 0.0) ;
	if(lineIntersection(e->source()->x(), e->source()->y(), e->dest()->x(), e->dest()->y(), bb->xmin(), bb->ymax(), bb->xmin(), bb->ymin(), &x, &y))
	    l << new QPoint3f(x, y, 0.0) ;
    }

    return l ;
}

QList<QPoint3f *> QSynapsPlugin::intersectionX(QRegionEdge * e, float y, float xmin, float xmax)
{
    QList<QPoint3f *> l ;

    if(e->object()->type() == "ImplicitCurve2D") {
	l << intersectionX((QImplicitCurve2D *)e->object(), y, xmin, xmax) ;
    } else {
	float x, y ;
	if(lineIntersection(e->source()->x(), e->source()->y(), e->dest()->x(), e->dest()->y(), xmin, y, xmax, y, &x, &y))
	    l << new QPoint3f(x, y, 0.0) ;
    }

    return l ;
}

QList<QPoint3f *> QSynapsPlugin::intersectionY(QRegionEdge * e, float x, float ymin, float ymax)
{
    QList<QPoint3f *> l ;

    if(e->object()->type() == "ImplicitCurve2D") {
	l << intersectionY((QImplicitCurve2D *)e->object(), x, ymin, ymax) ;
    } else {
	float x, y ;
	if(lineIntersection(e->source()->x(), e->source()->y(), e->dest()->x(), e->dest()->y(), x, ymin, x, ymax, &x, &y))
	    l << new QPoint3f(x, y, 0.0) ;
    }

    return l ;
}

Q_EXPORT_PLUGIN(QSynapsPlugin)
