/*****************************************************************************
 * A x e l
 *****************************************************************************
 * main
 * 2005-10-16
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# include <QtGui>
# include <QGui/Viewer.h>
# include <QGui/MainWindow.h>
# include <QGui/ScriptingDock.h>

# ifdef Q_WS_MAC
# include <QGui/CocoaUtil.h>
# endif

using namespace std ;
using namespace qglviewer ;
using namespace QAxel ;

void help(void)
{
    cout << "Usage: axel [options] [files]" << endl ;
    cout << endl ;
    cout << "Axel's default file format for [files] is .axl or .axs. Other input formalisms are handled in  " << endl ;
    cout << "[options]. General options can be given as arguments and are defined in the following sections." << endl ;
    cout << endl ;
    cout << "File Options :" << endl ;
    cout << "  -vect file       Import a .vect file" << endl ;
    cout << "  -off file        Import a .off file" << endl ;
    cout << "  -g2c file        Import a .g2 file containing a curve" << endl ;
    cout << "  -g2s file        Import a .g2 file containing a surface" << endl ;
    cout << endl ;
    cout << "Viewer Options :" << endl ;
    cout << "  -noaxis          Don't display axis" << endl ;
    cout << "  -xgrid           Display YZ grid" << endl ;
    cout << "  -ygrid           Display XZ grid" << endl ;
    cout << "  -zgrid           Display XY grid" << endl ;
    cout << "  -ortho           Set Orthographic mode (default is perspective)" << endl ;
    cout << "  -front           Set Front view" << endl ;
    cout << "  -bg color        Set the viewer's background color" << endl ;
    cout << "                   Possible values are : white, black, red, darkRed, green, darkGreen, blue, darkBlue" << endl ;
    cout << "                   cyan, darkCyan, magenta, darkMagenta, yellow, darkYellow, gray, darkGray, lightGray" << endl ;
}

int main(int argc, char** argv) 
{
    Q_INIT_RESOURCE(images) ;
    Q_INIT_RESOURCE(stylesheets) ;
	
    QApplication application(argc,argv);

    QStringList args ; QStack<QString> stack ;
    for(int i=argc-1 ; i>0 ; i--) stack.push(QString(argv[i])) ;
    if(stack.contains("-help") || stack.contains("--help") 
       || stack.contains("-?") || stack.contains("--?")) {
	help() ;
	return 0 ;
    }
	
    qsrand(QTime(0,0,0).secsTo(QTime::currentTime()));

# ifdef Q_OS_MAC
    CocoaUtil::initialize() ;

    QDir dir(QApplication::applicationDirPath());
    dir.cdUp(); dir.cd("Plugins");
    QApplication::addLibraryPath(dir.absolutePath());
# endif
	
    QPixmap pixmap(":images/axel_splash.png") ;
    QSplashScreen splash(pixmap) ;
    splash.show() ;
    qApp->processEvents() ;
	
    MainWindow w;
    w.setCurrentFile("");
    while(!stack.empty()) {
	QString s = stack.pop() ;
	// :: Handling files ::
	if(s == "-vect") 		w.importVectSlot(stack.pop()) ;
	else if(s == "-off")		w.importOffSlot(stack.pop()) ;
	else if(s == "-g2c") 		w.importG2CurveSlot(stack.pop()) ;
	else if(s == "-g2s")		w.importG2SurfaceSlot(stack.pop()) ;
	else if(s.contains(".axl"))	w.fileOpen(s) ;
	else if(s.contains(".axs")) { w.scriptingDock()->load(s) ; w.scriptingDock()->run() ; }
	// :: Handling visualization options
	else if(s == "-noaxis")		w.viewer()->setAxisIsDrawn(false) ;
	else if(s == "-xgrid")		w.viewer()->setYZGridIsDrawn(true) ;
	else if(s == "-ygrid")		w.viewer()->setXZGridIsDrawn(true) ;
	else if(s == "-zgrid")		w.viewer()->setXYGridIsDrawn(true) ;
	else if(s == "-ortho")		w.viewer()->camera()->setType(Camera::ORTHOGRAPHIC) ;
	else if(s == "-front")		{ w.viewer()->camera()->setPosition(Vec(0,0,2)) ; w.viewer()->camera()->setViewDirection(Vec(0,0,-1)) ; w.viewer()->camera()->setUpVector(Vec(0,1,0)) ; }
	else if(s == "-bg") 		w.viewer()->setBackgroundColor(QColor(stack.pop())) ;
	else						cout << "Unknown option: " << s.toLatin1().data() << endl ;
    }
			
    w.show();

	
    splash.finish(&w);
	
    return application.exec();
}
