########################################################
# A x e l Unix Makefile
# Julien Wintz
# Inria Sophia-Antipolis
########################################################

PREFIX=/usr/local
ACME=Inria
SOFT=Axel
VERSION=devel_$(shell date +"%Y%m%d")

DESTDIR = $(PREFIX)/$(ACME)/$(SOFT)

QTDIR=/usr/local/Trolltech/Qt-4.3.0
QT_MAJOR=4
QT_MINOR=3
QT_REVISION=0
QT_FRAMEWORKS=QtCore QtXml QtNetwork QtGui QtSql Qt3Support QtOpenGL

QSADIR=/usr/local/Trolltech/Qsa-1.2.2
QSA_MAJOR=1
QSA_MINOR=2
QSA_REVISION=1

THIRD_PARTY_LIBS=/usr/lib/libpng12.so.0 /lib/tls/i686/cmov/libc.so.6

AXEL_PLUGINS_DIR=../plugins
AXEL_PLUGINS=QSynaps QSisl QCore QCgal

NAME=axel
BIN_DIR=.
SAMPLES_DIR=../samples
RESOURCES_DIR=../resources

.PHONY: install uninstall

all: install

install: 
	@echo ----------------- Creating target directory --------------
	mkdir -p $(DESTDIR)
	@echo ----------------- Installing libs ------------------------
	mkdir -p $(DESTDIR)/lib
	for f in $(QT_FRAMEWORKS); do \
		cp -P $(QTDIR)/lib/lib$$f.so $(DESTDIR)/lib/ ; \
		cp -P $(QTDIR)/lib/lib$$f.so.$(QT_MAJOR) $(DESTDIR)/lib/ ; \
		cp -P $(QTDIR)/lib/lib$$f.so.$(QT_MAJOR).$(QT_MINOR) $(DESTDIR)/lib/ ; \
		cp -P $(QTDIR)/lib/lib$$f.so.$(QT_MAJOR).$(QT_MINOR).$(QT_REVISION) $(DESTDIR)/lib/ ; \
		cp -P $(QSADIR)/lib/libqsa.so $(DESTDIR)/lib/ ; \
		cp -P $(QSADIR)/lib/libqsa.so.$(QSA_MAJOR) $(DESTDIR)/lib/ ; \
		cp -P $(QSADIR)/lib/libqsa.so.$(QSA_MAJOR).$(QSA_MINOR) $(DESTDIR)/lib/ ; \
		cp -P $(QSADIR)/lib/libqsa.so.$(QSA_MAJOR).$(QSA_MINOR).$(QSA_REVISION) $(DESTDIR)/lib/ ; \
	done
	for f in $(THIRD_PARTY_LIBS); do \
		cp $$f $(DESTDIR)/lib/ ; \
	done
	@echo ----------------- Installing plugins ---------------------
	mkdir -p $(DESTDIR)/plugins
	for f in $(AXEL_PLUGINS); do \
		cp -P $(AXEL_PLUGINS_DIR)/lib$$f.so $(DESTDIR)/plugins/ ; \
	done
	@echo ----------------- Installing application -----------------
	mkdir -p $(DESTDIR)/bin
	cp -P $(BIN_DIR)/$(NAME) $(DESTDIR)/bin/
	@echo ----------------- Installing sample files ----------------
	mkdir -p $(DESTDIR)/samples
	rsync --exclude .svn -a $(SAMPLES_DIR)/* $(DESTDIR)/samples
	@echo ----------------- Installing resources files -------------
	install $(RESOURCES_DIR)/axel_run.sh /usr/bin/axel

uninstall:
	@echo ----------------- Removing target directory --------------
	rm -rvf $(DESTDIR)
	rm -rvf /usr/bin/axel

deb:
	@echo ----------------- Building -------------------------------
	$(MAKE)
	@echo ----------------- Creating deb package -------------------
	@mkdir -p ../packages
	cp ../resources/description-pak .
	checkinstall                                             \
		--type=debian                                        \
		--nodoc                                              \
		--pkgname="axel-modeler"                             \
		--pkgversion="0.3.4"                                 \
		--pkgrelease="1"                                     \
		--pkggroup="Applications/Science"                    \
		--pkglicense="/usr/share/common-licenses/GPL"        \
		--arch="i386"                                        \
		--strip=no                                           \
		--stripso=no                                         \
		--pakdir="../packages/"                              \
		--maintainer="jwintz@sophia.inria.fr"                \
		--install=no                                         \
		--backup=no                                          \
		--exclude=".svn"                                     \
		--default                                            \
		make install
	rm description-pak

rpm:
	@echo ----------------- Creating rpm package -------------------
	@cd ../packages && sudo alien --to-rpm ../packages/axel*deb

